#ifndef NORFLASH_H_
#define NORFLASH_H_

/******************************************************************************
	FDMREG.h
******************************************************************************/

#ifndef _FDMREG_H_
#define _FDMREG_H_


#define FDM_REG_BASE                0xC1FC1C00
#define FDM_REG_OFFSET              2

#define FDM_FLASH_REG_BASE          0x8000

//=============================================================================
//  Flash Data Management(FDM) interface side registers and constant definition 
//  According to 1603 Backend Register Definition version 1.8
//=============================================================================

//--------------------------------------------------------------------------
//  Interrupt Enable Register - 0x7000                      default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     R/W     R/W     RSV     R/W
//  B0      :   SKYCODE_DONE                ->  1:  enable, 0:  disable
//  B2      :   F_BSY                       ->  1:  enable, 0:  disable
//  B3      :   F_ERR                       ->  1:  enable, 0:  disable
//--------------------------------------------------------------------------
#define FDM_INT_EN_REG                  _REG08( FDM_REG_BASE + (0x00 << FDM_REG_OFFSET ))

#define INT_EN_DISABLE_ALL          0x00
#define INT_EN_SKYCODE_DOE          0x01
#define INT_EN_F_BSY                0x04
#define INT_EN_F_ERR                0x08


//--------------------------------------------------------------------------
//  Interrupt Status Register - 0x7001                      default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     R       R
//  B0      :   SKYCODE_DONE                ->  1:  enable, 0:  disable
//  B1      :   Reserved
//  B2      :   F_BSY                       ->  1:  enable, 0:  disable
//  B3      :   F_ERR                       ->  1:  enable, 0:  disable
//--------------------------------------------------------------------------
#define FDM_INT_STS_REG                 _REG08( FDM_REG_BASE + (0x01 << FDM_REG_OFFSET ))

#define INT_STS_SKYCODE_DOE         0x01
#define INT_STS_F_BSY               0x04
#define INT_STS_F_ERR               0x08


//--------------------------------------------------------------------------
//  Flash Signal Controller Register - 0x7004               default:    0011_1000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R/W     R/W     R       R/W     R/W     R/W     R/W     R/W
//  B0      :   WPb     ->  When regular Nand flash, Nand flash write-protect and reset signal.When OneNand flash, OneNand flash reset signal.
//  B1      :   ALE     ->  Flash Address latch
//  B2      :   CLE     ->  NAND flash command latch/AND Flash Command Enable( CDE )
//  B3      :   OEb     ->  Flash output enable
//  B4      :   WEb     ->  Flash Write enable
//  B5      :   RBb     ->  Flash ready/busy signal
//  B6      :   RSTb        ->  Flash Reset signal(AG_AND)
//  B7      :   SC      ->  Masked flash R/B signal, so H/W can not detect flash ready.
//--------------------------------------------------------------------------
#define FDM_FSIGNAL_CTRL_REG            _REG08( FDM_REG_BASE + (0x04 << FDM_REG_OFFSET ))

#define FSIGNAL_CTRL_WPb                0x01
#define FSIGNAL_CTRL_ALE                0x02
#define FSIGNAL_CTRL_CLE                0x04
#define FSIGNAL_CTRL_OEb                0X08
#define FSIGNAL_CTRL_WEb                0x10
#define FSIGNAL_CTRL_RBb                0x20
#define FSIGNAL_CTRL_RSTb               0x40
#define FSIGNAL_CTRL_DEFAULT            0x79
#define FSIGNAL_CTRL_SC                 0x80
#define FSIGNAL_CTRL_DEFAULT_OneNAND    0x39


//--------------------------------------------------------------------------
//  Flash Signal Mask Register - 0x7005                     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     R/W     R/W
//  B0      :   WE_L_MASK   ->  1 : Mask FWEb signal in low byte
//  B1      :   WE_H_MASK   ->  1 : Flash Address latch
//--------------------------------------------------------------------------
#define FDM_FSIGNAL_MASK_REG            _REG08( FDM_REG_BASE + (0x05 << FDM_REG_OFFSET ))

#define FSIGNAL_MASK_DEFAULT        0x00
#define FSIGNAL_MASK_WE_L           0x01
#define FSIGNAL_MASK_WE_H           0x02



//--------------------------------------------------------------------------
//  Chip Select Register - 0x7006                           default:    0000_1111
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     R/W     R/W     R/W     R/W
//  B0      :   CHIP_SEL0b  ->  0 : Chip0 enable    
//  B1      :   CHIP_SEL1b  ->  0 : Chip1 enable
//  B2      :   CHIP_SEL2b  ->  0 : Chip2 enable
//  B3      :   CHIP_SEL3b  ->  0 : Chip3 enable
//--------------------------------------------------------------------------
#define FDM_CHIP_SEL_REG                _REG08( FDM_REG_BASE + (0x06 << FDM_REG_OFFSET ))

#define CHIP_SEL0b                  0x01
#define CHIP_SEL1b                  0x02
#define CHIP_SEL2b                  0x04
#define CHIP_SEL3b                  0x08
#define CHIP_SEL_ALL                0x00
#define CHIP_SEL_DISSEL_ALL         0x0F
#define CHIP_SEL_0                  0xFE

//--------------------------------------------------------------------------
//  FW Reset Register - 0x7007                              default:    0000_0011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     R/W     R/W
//  B0      :   ECC_RSTb    ->  0 : ECC Reset
//  B1      :   BE_RSTb     ->  0 : BE Reset
//--------------------------------------------------------------------------
#define FDM_FW_RST_REG                  _REG08( FDM_REG_BASE + (0x07 << FDM_REG_OFFSET ))

#define FW_RST_ECC_RSTb             ((unsigned char)0x01)
#define FW_RST_BE_RSTb              ((unsigned char)0x02)



//--------------------------------------------------------------------------
//  Flash Auto Command Register - 0x7008                    default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R/W     R/W     R/W     R/W     R/W     R/W     R/W     R/W
//  B[7:0]  :   Skycode
//--------------------------------------------------------------------------
#define FDM_FLASH_CMD_REG               _REG08( FDM_REG_BASE + (0x08 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Skycode control Register - 0x7009                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     R/W     R/W     R/W
//  B0      :   STOP        -> 1 : Terminate skycode after the current operation is done
//  B1      :   FORCE_STOP  -> 1 : Terminate skycode immediately
//  B2      :   PAUSE_READ  -> 1 : Pause operation before read data to SRAM
//--------------------------------------------------------------------------
#define FDM_SKYCODE_CTRL_REG            _REG08( FDM_REG_BASE + (0x09 << FDM_REG_OFFSET ))

#define SKYCODE_CTRL_STOP           0x01
#define SKYCODE_CTRL_FORCE_STOP     0x02
#define SKYCODE_CTRL_PAUSE_READ     0x04



//--------------------------------------------------------------------------
//  Flash Status high byte Register - 0x700A                default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//  B[7:0]  :   Flash Status
//--------------------------------------------------------------------------
#define FDM_FLASH_STS_H_REG             _REG08( FDM_REG_BASE + (0x0A << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Flash Status low byte Register - 0x700B                 default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//  B[7:0<< FDM_REG_OFFSET ))   :   Flash Status
//--------------------------------------------------------------------------
#define FDM_FLASH_STS_L_REG             _REG08( FDM_REG_BASE + (0x0B << FDM_REG_OFFSET )) 




//--------------------------------------------------------------------------
//  Backend Circuit Status Register - 0x700C                default:    0000_1011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     R       R       R       R       R
//  B0      :   Flash busy/ready    -> 1 : ready,       0:  busy
//  B1      :   Backend busy/ready  -> 1 : ready,       0:  busy
//  B2      :   F_ERROR             -> 1 : flash error
//  B3      :   ALL_DATA_1          -> 1 : The data read from flash are all 0xFF
//  B4      :   FE_HANDSHAKE        -> 1 : The flash controller is handshaking with bufferman
//--------------------------------------------------------------------------
#define FDM_BACKEND_STS_REG             _REG08( FDM_REG_BASE + (0x0C << FDM_REG_OFFSET ))

#define BACKEND_STS_F_READY         0x01
#define BACKEND_STS_B_READY         0x02
#define BACKEND_STS_F_ERROR         0x04
#define BACKEND_STS_ALL_DATA_1      0x08
#define BACKEND_STS_FE_HANDSHAKE    0x10

//--------------------------------------------------------------------------
//  ECC Status Register - 0x700E                            default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//  B0      :   ERROR_L     -> 1 : Correctable Error
//  B[2:1]  :   SERR_L      -> ECC Correctable Error, Number of symbol error
//  B3      :   ECC_ERROR_L -> 1 : ECC uncorrectable error
//  B4      :   ERROR_H     -> 1 : Correctable Error
//  B[6:5]  :   SERR_H      -> ECC Correctable Error, Number of symbol error
//  B7      :   ECC_ERROR_H -> 1 : ECC uncorrectable error
//--------------------------------------------------------------------------
#define FDM_ECC_STS_REG                 _REG08( FDM_REG_BASE + (0x0E << FDM_REG_OFFSET ))

#define ECC_STS_ERROR_L             0x01
#define ECC_STS_SERR_L_MASK         0x06
#define ECC_STS_ECC_ERROR_L         0x08
#define ECC_STS_ERROR_H             0x10
#define ECC_STS_SERR_H_MASK         0x60
#define ECC_STS_ECC_ERROR_H         0x80
#define ECC_STS_ERROR               0x11
#define ECC_STS_ECC_ERROR           0x88




//--------------------------------------------------------------------------
//  Header ECC Status Register - 0x700F                     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//  B0      :   ERROR_L     -> 1 : Correctable Error
//  B[2:1]  :   SERR_L      -> ECC Correctable Error, Number of symbol error
//  B3      :   ECC_ERROR_L -> 1 : ECC uncorrectable error
//  B4      :   ERROR_H     -> 1 : Correctable Error
//  B[6:5]  :   SERR_H      -> ECC Correctable Error, Number of symbol error
//  B7      :   ECC_ERROR_H -> 1 : ECC uncorrectable error
//--------------------------------------------------------------------------
#define FDM_HEADER_ECC_STS_REG          _REG08( FDM_REG_BASE + (0x0F << FDM_REG_OFFSET ))
#define HEADER_ECC_STS_ERROR_L      0x01
#define HEADER_ECC_STS_SERR_L_MASK  0x06
#define HEADER_ECC_STS_ECC_ERROR_L  0x08
#define HEADER_ECC_STS_ERROR_H      0x10
#define HEADER_ECC_STS_SERR_H_MASK  0x60
#define HEADER_ECC_STS_ECC_ERROR_H  0x80
#define HEADER_ECC_STS_ERROR        0x11
#define HEADER_ECC_STS_ECC_ERROR    0x88




//--------------------------------------------------------------------------
//  Block Mask High Register - 0x7010                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     R/W     R/W     R/W     R/W     R/W
//  B[4:0]  :   Block Mask [20:16]
//--------------------------------------------------------------------------
#define FDM_BLK_MASK_H_REG              _REG08( FDM_REG_BASE + (0x10 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Block Mask Middle Register - 0x7011                     default:    0000_0011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      R/W     R/W     R/W     R/W     R/W
//  B[7:0]  :   Block Mask [15:8]
//--------------------------------------------------------------------------
#define FDM_BLK_MASK_M_REG              _REG08( FDM_REG_BASE + (0x11 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  MP Factor Register - 0x7012                             default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RW      R/W
//  B[1:0]  :   No of Bank (Plane)  -> 00 : 1 Bank
//                                  -> 01 : 2 Banks
//                                  -> 10 : 4 Banks
//                                  -> 11 : 8 Banks
//--------------------------------------------------------------------------
#define FDM_MP_FACTOR_REG               _REG08( FDM_REG_BASE + (0x12 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Die Factor Register - 0x7013                            default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RW      R/W
//  B[1:0]  :   Die Number  -> 00 : 1 Die
//                          -> 01 : 2 Dies
//                          -> 10 : 4 Dies 
//                          -> 11 : 8 Dies
//--------------------------------------------------------------------------
#define FDM_DIE_FACTOR_REG              _REG08( FDM_REG_BASE + (0x13 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Chip Factor Register - 0x7014                           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RW      R/W
//  B[1:0]  :   Chip Number -> 00 : 1 Chips
//                          -> 01 : 2 Chips
//                          -> 10 : 4 Chips 
//                          -> 11 : 8 Chips
//--------------------------------------------------------------------------
#define FDM_CHIP_FACTOR_REG             _REG08( FDM_REG_BASE + (0x14 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Die Factor Register - 0x7015                            default:    0000_0010
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RW      RW      R/W
//  B[2:0]  :   PARTI_SHIFT
//--------------------------------------------------------------------------
#define FDM_PARTI_SHIFT_REG             _REG08( FDM_REG_BASE + (0x15 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Block Shift Register - 0x7016                           default:    0000_1010
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RW      RW      RW      R/W
//  B[4:0]  :   Block Shift 
//--------------------------------------------------------------------------
#define FDM_BLK_SHIFT_REG               _REG08( FDM_REG_BASE + (0x16 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Bank Shift Register - 0x7017                            default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RW      RW      RW      RW      RW      R/W
//  B[5:0]  :   Bank Shift
//--------------------------------------------------------------------------
#define FDM_BANK_SHIFT_REG              _REG08( FDM_REG_BASE + (0x17 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Page Shift Register - 0x7018                            default:    0000_0110
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RW      RW      RW      R/W
//  B[3:0]  :   Page Shift
//--------------------------------------------------------------------------
#define FDM_PAGE_SHIFT_REG              _REG08( FDM_REG_BASE + (0x18 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Address cycle Register - 0x7019                         default:    0001_0001
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RSV     RSV     RW      RW
//  B[1:0]  :   ROW CYCLE
//  B4      :   COL_CYCLE
//--------------------------------------------------------------------------
#define FDM_ADDR_CYCLE_REG              _REG08( FDM_REG_BASE + (0x19 << FDM_REG_OFFSET ))

#define ADDR_CYCLE_ROW_MASK         0x03
#define ADDR_CYCLE_COL_MASK         0x10




//--------------------------------------------------------------------------
//  Flash Config Register - 0x701A                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   ONE NAND
//  B1      :   EDC EN
//  B2      :   STATUS_ADDR_EN
//  B3      :   HW_STS_CMD_EN
//  B4      :   R_DATA_OUT_EN
//  B5      :   R_DATA_IN_EN
//  B6      :   X16
//  B7      :   EDO
//--------------------------------------------------------------------------
#define FDM_FLASH_CFG_REG               _REG08( FDM_REG_BASE + (0x1A << FDM_REG_OFFSET ))

#define FLASH_CFG_ONE_NAND          0x01
#define FLASH_CFG_EDC               0x02
#define FLASH_CFG_STS_ADDR          0x04
#define FLASH_CFG_HW_STS_CMD        0x08
#define FLASH_CFG_R_DATA_OUT        0x10
#define FLASH_CFG_R_DATA_IN         0x20
#define FLASH_CFG_X16               0x40
#define FLASH_CFG_EDO               0x80




//--------------------------------------------------------------------------
//  Flash Write Config Register - 0x701B                    default:    0011_1011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RW      RW      RW      RW      RW      RW      RW
//  B[3:0]  :   TWP_TYPE
//  B[6:4]  :   TWP_DUTY_SEL
//--------------------------------------------------------------------------
#define FDM_FLASH_WR_CFG_REG            _REG08( FDM_REG_BASE + (0x1B << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Read Config Register - 0x701C                     default:    1011_1011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RW      RW      RW      RW      RW      RW      RW
//  B[3:0]  :   TRP_TYPE
//  B[6:4]  :   TRP_DUTY_SEL
//  B7      :   RE_LATCH        -> 0 : Use REb to latch data from flash,
//                              -> 1 : Negative edge of CLK to latch data from flash
//--------------------------------------------------------------------------
#define FDM_FLASH_RD_CFG_REG            _REG08( FDM_REG_BASE + (0x1C << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Read Config 2 Register - 0x701D                       default:    0000_1011
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RW      RW      RW      RW
//  B[2:0]  :   FB_TRP_DUTY
//  B3      :   FB_RE
//--------------------------------------------------------------------------
#define FDM_FLASH_RD_CFG2_REG           _REG08( FDM_REG_BASE + (0x1D << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  Flash ALE/CLE control Register - 0x701E                     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   TCLS_TYPE
//  B1      :   TALS_TYPE
//  B[4:2]  :   CMD_WAIT_CNT
//  B[7:5]  :   ADDR_WAIT_CNT
//--------------------------------------------------------------------------
#define FDM_FLASH_ALE_CLE_CTRL_REG      _REG08( FDM_REG_BASE + (0x1E << FDM_REG_OFFSET ))


//--------------------------------------------------------------------------
//  Flash Control 0 Register - 0x701F                       default:    0100_0001
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   PRG_ERROR_STOP
//  B1      :   BE_SRAM_DIS
//  B2      :   FW_FTL
//  B3      :   HD_INV
//  B4      :   BUS_INV
//  B5      :   WORD_MODE
//  B6      :   PAGE_H_EN
//  B7      :   SEQ_READ_EN
//--------------------------------------------------------------------------
#define FDM_FLASH_CTRL0_REG             _REG08( FDM_REG_BASE + (0x1F << FDM_REG_OFFSET ))

#define FSH_CTRL0_PRG_ERROR_STOP    0x01
#define FSH_CTRL0_BE_SRAM_DIS       0x02
#define FSH_CTRL0_FW_FTL            0x04
#define FSH_CTRL0_HD_INV            0x08
#define FSH_CTRL0_BUS_INV           0x10
#define FSH_CTRL0_WORD_MODE         0x20
#define FSH_CTRL0_PAGE_H_EN         0x40
#define FSH_CTRL0_SEQ_READ_EN       0x80
#define FSH_CTRL0_NOR_DEFAULT       0xB0



//--------------------------------------------------------------------------
//  Flash Control 1 Register - 0x7020                       default:    0011_1111
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   COL_ADDR_EN
//  B1      :   ROW_ADDR_EN
//  B2      :   CMD3_WAIT_BUF_READY
//  B3      :   HD_EN
//  B4      :   WAIT_BUSY
//  B5      :   WAIT_BUSY_TYPE
//  B6      :   FAIL_BIT_OPTION ->  1 : use status bit 1 to judge flash pass or fail
//                                  0 : use status bit 0
//  B7      :   CMD2_WAIT_BUF_READY
//--------------------------------------------------------------------------
#define FDM_FLASH_CTRL1_REG             _REG08( FDM_REG_BASE + (0x20 << FDM_REG_OFFSET ))

#define FSH_CTRL1_COL_ADDR_EN       0x01
#define FSH_CTRL1_ROW_ADDR_EN       0x02
//#define FSH_CTRL1_ECC_EN          0x04
#define FSH_CTRL1_CMD3_WAIT_BUF_READY   0x04
#define FSH_CTRL1_HD_EN             0x08
#define FSH_CTRL1_WAIT_BUSY         0x10
#define FSH_CTRL1_WAIT_BUSY_TYPE    0x20
//#define FSH_CTRL1_SEND_WHILE_BUSY 0x40
#define FSH_CTRL1_FAIL_BIT_OPTION   0x40
#define FSH_CTRL1_CMD2_WAIT_BUF_READY   0x80

#define FSH_CTRL1_DEFAULT       0x3F


//--------------------------------------------------------------------------
//  Flash Control 2 Register - 0x7021                       default:    0000_0111
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   HD_ECC_EN
//  B1      :   DAT_ECC_ERR_STOP
//  B2      :   HD_ECC_ERR_STOP
//  B3      :   SMALL_EDC_EN
//  B4      :   PAGE_HD_TYPE
//  B5      :   WAIT_BUSY_TYPE2 ->  1 : In write skycode, wait BUF_RDY_1K signal before writing cmd3 into flash
//  B6      :   AG_EDC_EN
//  B7      :   EDC_WAIT_BUF_1K ->  1 : wait BUF_RDY_1K before data transfer unless last partition
//                                  0 : not wait BUF_RDY_1K before data transfer
//--------------------------------------------------------------------------
#define FDM_FLASH_CTRL2_REG             _REG08( FDM_REG_BASE + (0x21 << FDM_REG_OFFSET ))

#define FSH_CTRL2_HD_ECC_EN         0x01
#define FSH_CTRL2_DAT_ECC_ERR_STOP  0x02
#define FSH_CTRL2_HD_ECC_ERR_STOP   0x04
#define FSH_CTRL2_SMALL_EDC_EN      0x08
//#define FSH_CTRL2_CMD3_WAIT_BUF_READY 0x10
#define FSH_CTRL2_PAGE_HD_TYPE      0x10
#define FSH_CTRL2_WAIT_BUFY_TYPE2   0x20
//#define FSH_CTRL2_FAIL_BIT_OPTION 0x40
#define FSH_CTRL2_AG_EDC_EN         0x40
#define FSH_CTRL2_EDC_WAIT_BUF_1K   0x80



//--------------------------------------------------------------------------
//  Flash Control 3 Register - 0x7022                       default:    0000_0100
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RW      RW      RW
//  B0      :   EDC_ACC_H
//  B1      :   EDC_ACC_L
//  B2      :   ECC_EN
//  B3      :   SEND_WHILE_BUSY
//  B4      :   USE_SERIAL_NOR_FLASH
//--------------------------------------------------------------------------
#define FDM_FLASH_CTRL3_REG             _REG08( FDM_REG_BASE + (0x22 << FDM_REG_OFFSET ))

#define FSH_CTRL3_EDC_ACC_H         0x01
#define FSH_CTRL3_EDC_ACC_L         0x02
//#define FSH_CTRL2_AG_EDC_EN           0x04
#define FSH_CTRL3_ECC_EN            0x04
#define FSH_CTRL3_SEND_WHILE_BUSY       0x08
#define FSH_CTRL3_USE_SERIAL_NOR_FLASH      0x10

#define FDM_FLASH_CTRL4_REG             _REG08( FDM_REG_BASE + (0x23 << FDM_REG_OFFSET ))
#define PAGE_ID_INC_DISABLE         0x01


//--------------------------------------------------------------------------
//  Flash Control 4 Register - 0x7023                       default:    0000_0100
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FLASH_CTRL_REG              _REG08( FDM_REG_BASE + (0x23 << FDM_REG_OFFSET ))

//--------------------------------------------------------------------------
//  Block ID High Register - 0x7058                         default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BID_H_REG                   _REG08( FDM_REG_BASE + (0x58 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Block ID Low Register - 0x7059                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BID_L_REG                	_REG08( FDM_REG_BASE + (0x59 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Block Status Register - 0x705A                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BS_L_REG                     _REG08( FDM_REG_BASE + (0x5A << FDM_REG_OFFSET ))

#define BS_STATUS_MASK              0x30
#define BS_STATUS_BAD_OR_SYS        0x00
#define BS_STATUS_NON_SEQUENCE      0x10
#define BS_STATUS_SEQUENCE          0x20
#define BS_STATUS_FREE              0x30
                                        
#define BS_ORDER_MASK               0x0C
#define BS_WL_MASK                  0x03


//--------------------------------------------------------------------------
//  Page ID Register - 0x705B                               default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_PID_L_REG                     _REG08( FDM_REG_BASE + (0x5B << FDM_REG_OFFSET ))


//--------------------------------------------------------------------------
//  Page ID Register - 0x705D                               default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
// B7~B2   :   BlockStatis[4:0]
// B1~B0   :   BlockID[9:8] 
//--------------------------------------------------------------------------
#define FDM_BID_H_MIRROR_REG            _REG08( FDM_REG_BASE + (0x5D << FDM_REG_OFFSET ))

//--------------------------------------------------------------------------
//  Page ID Register - 0x705E                               default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
// B7~B0   :   BlockID[7:0]
//--------------------------------------------------------------------------
#define FDM_BID_L_MIRROR_REG            _REG08( FDM_REG_BASE + (0x5E << FDM_REG_OFFSET ))

//--------------------------------------------------------------------------
//  Page ID Register - 0x705F                               default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
// B7~B0   :   PageID[7:0]
//--------------------------------------------------------------------------
#define FDM_PID_MIRROR_REG              _REG08( FDM_REG_BASE + (0x5F << FDM_REG_OFFSET ))

//--------------------------------------------------------------------------
//  MKB Register - 0x7060 ~ 0x706F                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
//  _FDM_DECL_ _REG_DECL_UB_            MKB0_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x60);
//  _FDM_DECL_ _REG_DECL_UB_            MKB1_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x61);
//  _FDM_DECL_ _REG_DECL_UB_            MKB2_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x62);
//  _FDM_DECL_ _REG_DECL_UB_            MKB3_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x63);
//  _FDM_DECL_ _REG_DECL_UB_            MKB4_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x64);
//  _FDM_DECL_ _REG_DECL_UB_            MKB5_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x65);
//  _FDM_DECL_ _REG_DECL_UB_            MKB6_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x66);
//  _FDM_DECL_ _REG_DECL_UB_            MKB7_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x67);
//  _FDM_DECL_ _REG_DECL_UB_            MKB8_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x68);
//  _FDM_DECL_ _REG_DECL_UB_            MKB9_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x69);
//  _FDM_DECL_ _REG_DECL_UB_            MKBA_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6A);
//  _FDM_DECL_ _REG_DECL_UB_            MKBB_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6B);
//  _FDM_DECL_ _REG_DECL_UB_            MKBC_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6C);
//  _FDM_DECL_ _REG_DECL_UB_            MKBD_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6D);
//  _FDM_DECL_ _REG_DECL_UB_            MKBE_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6E);
//  _FDM_DECL_ _REG_DECL_UB_            MKBF_REG                    _FDM_ADDR_(FDM_REG_BASE + 0x6F);

#define FDM_MKB0                    	_REG08( FDM_REG_BASE + (0x60 << FDM_REG_OFFSET ))
#define FDM_MKB1_REG                    _REG08( FDM_REG_BASE + (0x61 << FDM_REG_OFFSET ))
#define FDM_MKB2_REG                    _REG08( FDM_REG_BASE + (0x62 << FDM_REG_OFFSET ))
#define FDM_MKB3_REG                    _REG08( FDM_REG_BASE + (0x63 << FDM_REG_OFFSET ))
#define FDM_MKB4_REG                    _REG08( FDM_REG_BASE + (0x64 << FDM_REG_OFFSET ))
#define FDM_MKB5_REG                    _REG08( FDM_REG_BASE + (0x65 << FDM_REG_OFFSET ))
#define FDM_MKB6_REG                    _REG08( FDM_REG_BASE + (0x66 << FDM_REG_OFFSET ))
#define FDM_MKB7_REG                    _REG08( FDM_REG_BASE + (0x67 << FDM_REG_OFFSET ))
#define FDM_MKB8_REG                    _REG08( FDM_REG_BASE + (0x68 << FDM_REG_OFFSET ))
#define FDM_MKB9_REG                    _REG08( FDM_REG_BASE + (0x69 << FDM_REG_OFFSET ))
#define FDM_MKBA_REG                    _REG08( FDM_REG_BASE + (0x6A << FDM_REG_OFFSET ))
#define FDM_MKBB_REG                    _REG08( FDM_REG_BASE + (0x6B << FDM_REG_OFFSET ))
#define FDM_MKBC_REG                    _REG08( FDM_REG_BASE + (0x6C << FDM_REG_OFFSET ))
#define FDM_MKBD_REG                    _REG08( FDM_REG_BASE + (0x6D << FDM_REG_OFFSET ))
#define FDM_MKBE_REG                    _REG08( FDM_REG_BASE + (0x6E << FDM_REG_OFFSET ))
#define FDM_MKBF_REG                    _REG08( FDM_REG_BASE + (0x6F << FDM_REG_OFFSET ))


#define FDM_MKB_REG(X)                  _REG08(FDM_REG_BASE + (0x60 + ( X ) << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  SA3 Register - 0x7070                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_SA3_REG                     _REG08( FDM_REG_BASE + (0x70 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  SA2 Register - 0x7071                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_SA2_REG                     _REG08( FDM_REG_BASE + (0x71 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  SA1 Register - 0x7072                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_SA1_REG                     _REG08( FDM_REG_BASE + (0x72 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  SA0 Register - 0x7073                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_SA0_REG                     _REG08( FDM_REG_BASE + (0x73 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  CA1 Register - 0x7078                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_CA1_REG                     _REG08( FDM_REG_BASE + (0x78 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  CA0 Register - 0x7079                                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       R
//--------------------------------------------------------------------------
#define FDM_CA0_REG                     _REG08( FDM_REG_BASE + (0x79 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Enable Register - 0x707A                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     R       R       RSV     RSV     R       R
//--------------------------------------------------------------------------
#define FDM_FCE_REG                     _REG08( FDM_REG_BASE + (0x7A << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  FW SA3 for Low Chip Register - 0x7080                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA3_LC_REG               _REG08( FDM_REG_BASE + (0x80 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  FW SA2 for Low Chip Register - 0x7081                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA2_LC_REG               _REG08( FDM_REG_BASE + (0x81 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  FW SA1 for Low Chip Register - 0x7082                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA1_LC_REG               _REG08( FDM_REG_BASE + (0x82 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  FW SA0 for Low Chip Register - 0x7083                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA0_LC_REG               _REG08( FDM_REG_BASE + (0x83 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  FW SA3 for High Chip Register - 0x7084                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA3_HC_REG               _REG08( FDM_REG_BASE + (0x84 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  FW SA2 for High Chip Register - 0x7085                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA2_HC_REG               _REG08( FDM_REG_BASE + (0x85 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  FW SA1 for High Chip Register - 0x7086                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA1_HC_REG               _REG08( FDM_REG_BASE + (0x86 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  FW SA0 for High Chip Register - 0x7087                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_SA0_HC_REG               _REG08( FDM_REG_BASE + (0x87 << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  FW CA1 Register - 0x7088                                default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_CA1_REG                  _REG08( FDM_REG_BASE + (0x88 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  FW CA0 Register - 0x7089                                default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FW_CA0_REG                  _REG08( FDM_REG_BASE + (0x89 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Start Block No for Low Chip Register - 0x7091           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO2_LC_REG        _REG08( FDM_REG_BASE + (0x91 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Start Block No for Low Chip Register - 0x7092           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO1_LC_REG        _REG08( FDM_REG_BASE + (0x92 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Start Block No for Low Chip Register - 0x7093           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO0_LC_REG        _REG08( FDM_REG_BASE + (0x93 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Start Block No for High Chip Register - 0x7095          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO2_HC_REG        _REG08( FDM_REG_BASE + (0x95 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Start Block No for High Chip Register - 0x7096          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO1_HC_REG        _REG08( FDM_REG_BASE + (0x96 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Start Block No for High Chip Register - 0x7097          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_BLK_NO0_HC_REG        _REG08( FDM_REG_BASE + (0x97 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Destination Block No for Low Chip Register - 0x7099     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_H_LC_REG          _REG08( FDM_REG_BASE + (0x99 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Destination Block No for Low Chip Register - 0x709A     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_M_LC_REG          _REG08( FDM_REG_BASE + (0x9A << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Destination Block No for Low Chip Register - 0x709B     default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_L_LC_REG          _REG08( FDM_REG_BASE + (0x9B << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Destination Block No for High Chip Register - 0x709D    default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_H_HC_REG          _REG08( FDM_REG_BASE + (0x9D << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Destination Block No for High Chip Register - 0x709E    default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_M_HC_REG          _REG08( FDM_REG_BASE + (0x9E << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Destination Block No for High Chip Register - 0x709F    default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_BLK_L_HC_REG          _REG08( FDM_REG_BASE + (0x9F << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Start Page No Register - 0x70A0                         default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_START_PAGE_NO_REG           _REG08( FDM_REG_BASE + (0xA0 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Destination Page No Register - 0x70A4                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DEST_PAGE_NO_REG            _REG08( FDM_REG_BASE + (0xA4 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Partition No Register - 0x70A8                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_PARTI_NO_HI_REG             _REG08( FDM_REG_BASE + (0xA8 << FDM_REG_OFFSET ))
//#define FDM_PARTI_NO_REG                _REG08( FDM_REG_BASE + (0xA9 << FDM_REG_OFFSET ))


//--------------------------------------------------------------------------
//  Partition No Register - 0x70A9                          default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_PARTI_NO_LOW_REG            _REG08( FDM_REG_BASE + (0xA9 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  flash start address high Register - 0x70AC              default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FADR_H_REG                  _REG08( FDM_REG_BASE + (0xAC << FDM_REG_OFFSET ))
#define FDM_ONE_START_ADR_H_REG         _REG08( FDM_REG_BASE + (0xAC << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  flash start address low Register - 0x70AD               default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FADR_L_REG                  _REG08( FDM_REG_BASE + (0xAD << FDM_REG_OFFSET ))
#define FDM_ONE_START_ADR_L_REG         _REG08( FDM_REG_BASE + (0xAD << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Command 1 Register - 0x70B0                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD1_REG                    _REG08( FDM_REG_BASE + (0xB0 << FDM_REG_OFFSET ))
#define FDM_ONE_WR_DATA_H_REG           _REG08( FDM_REG_BASE + (0xB0 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Flash Command 2 Register - 0x70B1                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD2_REG                    _REG08( FDM_REG_BASE + (0xB1 << FDM_REG_OFFSET ))
#define FDM_ONE_WR_DATA_L_REG           _REG08( FDM_REG_BASE + (0xB1 << FDM_REG_OFFSET )) 




//--------------------------------------------------------------------------
//  Flash Command 3 Register - 0x70B2                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD3_REG                    _REG08( FDM_REG_BASE + (0xB2 << FDM_REG_OFFSET ))
#define FDM_ONE_WR_ADR_H_REG            _REG08( FDM_REG_BASE + (0xB2 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Flash Command 4 Register - 0x70B3                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD4_REG                    _REG08( FDM_REG_BASE + (0xB3 << FDM_REG_OFFSET ))
#define FDM_ONE_WR_ADR_L_REG            _REG08( FDM_REG_BASE + (0xB3 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Flash Command 5 Register - 0x70B4                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD5_REG                    _REG08( FDM_REG_BASE + (0xB4 << FDM_REG_OFFSET ))
#define FDM_ONE_RD_ADR_H_REG            _REG08( FDM_REG_BASE + (0xB4 << FDM_REG_OFFSET ))
#define ONE_RD_ADR_REG              (*((unsigned short int volatile x51_xdata *)(FDM_REG_BASE + 0xB4)))




//--------------------------------------------------------------------------
//  Flash Command 6 Register - 0x70B5                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD6_REG                    _REG08( FDM_REG_BASE + (0xB5 << FDM_REG_OFFSET ))
#define FDM_ONE_RD_ADR_L_REG            _REG08( FDM_REG_BASE + (0xB5 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Command 7 Register - 0x70B6                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD7_REG                    _REG08( FDM_REG_BASE + (0xB6 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Command 8 Register - 0x70B7                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD8_REG                    _REG08( FDM_REG_BASE + (0xB7 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Command Enable Register - 0x70BE                      default:    1111_1110
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_CMD_ENABLE_REG              _REG08( FDM_REG_BASE + (0xBE << FDM_REG_OFFSET ))

#define CMD_PTR_CMD_EN              0x01
#define CMD_DMY_CMD_EN              0x02
#define CMD_CMD1_EN                 0x04
#define CMD_CMD2_EN                 0x08
#define CMD_CMD3_EN                 0x10
#define CMD_CMD4_EN                 0x20
#define CMD_CMD5_EN                 0x40
#define CMD_CMD7_EN                 0x80



//--------------------------------------------------------------------------
//  SRAM select Register - 0x70C0                           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RW      RW      RW      RW      RW      RW
//  B0      :   SWAP1   enable swap mode on 512Bytes * 4 buffer
//  B1      :   SWAP2   
//  B[5:2]  :   
//--------------------------------------------------------------------------
#define FDM_SWAP_REG                    _REG08( FDM_REG_BASE + (0xC0 << FDM_REG_OFFSET ))

#define SWAP_MODE_1                 0x01
#define SWAP_MODE_2                 0x02
#define SWAP_MODE_3_MASK            0x3C
#define SWBP_CB                     0x01
#define SWBP_BUF0                   0x01
#define SWBP_BUF1                   0x01
#define SWBP_BUF2                   0x01
#define SWBP_BUF3                   0x01
#define SWBP_WH                     0x02
#define SWBP_VAR                    0x02
#define SWBP_CODE_COMMON            0x04
#define SWAP_CODE_BANK              0x08
#define SWAP_CODE_SWAP              0x10





//--------------------------------------------------------------------------
//  SRAM Status Register - 0x70C1                           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     R       R
//  B0      :   Buffer Ready
//  B1      :   Buffer Ready 1K 
//--------------------------------------------------------------------------
#define FDM_BUF_STS_REG                 _REG08( FDM_REG_BASE + (0xC1 << FDM_REG_OFFSET ))

#define BUF_STS_READY               0x01
#define BUF_STS_READY_1K            0x02




//--------------------------------------------------------------------------
//  SRAM CTL Register - 0x70C2                              default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RW      RW
//  B0      :   FW_BB512 -> 
//  B1      :   FW_BB512_MASK
//  B2  :   FIX_BSRAM
//--------------------------------------------------------------------------
#define FDM_BUF_CTL_REG                 _REG08( FDM_REG_BASE + (0xC2 << FDM_REG_OFFSET ))

#define BUF_CTL_BB512               0x01
#define BUF_CTL_BB512_MASK          0x02
#define BUF_CTL_FIX_BSRAM           0x04



//--------------------------------------------------------------------------
//  SRAM select Register - 0x70C0                           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RW      RW      RW      RW      RW      RW
//  B0      :   SWAP1   enable swap mode on 512Bytes * 4 buffer
//  B1      :   SWAP2   
//  B[5:2]  :   
//--------------------------------------------------------------------------
#define FDM_SWAP2_REG                   _REG08( FDM_REG_BASE + (0xC3 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  SRAM High Register - 0x70C4                             default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BSRAM_HIGH_REG              _REG08( FDM_REG_BASE + (0xC4 << FDM_REG_OFFSET ))

#define BSRAM_DEFAULT               0x00

#define BUF_BANK_BASE_H             0x20
#define BUF_BANK_BASE               0x2000

#define BSRAM_DEFAULT               0x00
#define BSRAM_BANK0_LOW             (BUF_BANK_BASE)
#define BSRAM_BANK0_LOW_H           (BUF_BANK_BASE_H)
#define BSRAM_BANK0_HIGH            (BUF_BANK_BASE + 0x0200)
#define BSRAM_BANK0_HIGH_H          (BUF_BANK_BASE_H + 0x02)
#define BSRAM_BANK1_LOW             (BUF_BANK_BASE + 0x0400)
#define BSRAM_BANK1_LOW_H           (BUF_BANK_BASE_H + 0x04)
#define BSRAM_BANK1_HIGH            (BUF_BANK_BASE + 0x0600)
#define BSRAM_BANK1_HIGH_H          (BUF_BANK_BASE_H + 0x06)



//--------------------------------------------------------------------------
//  SRAM Low Register - 0x70C5                              default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BSRAM_LOW_REG               _REG08( FDM_REG_BASE + (0xC5 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Data Length High Register - 0x70C8                      default:    0000_0001
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_LENGTH_HIGH_REG             _REG08( FDM_REG_BASE + (0xC8 << FDM_REG_OFFSET ))

#define DATA_LENGTH_DEFAULT     0x0100



//--------------------------------------------------------------------------
//  Data Length Low Register - 0x70C9                       default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_LENGTH_LOW_REG              _REG08( FDM_REG_BASE + (0xC9 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  DATA Block Count Register - 0x70CD                      default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   R       R       R       R       R       R       R       RW
//--------------------------------------------------------------------------
#define FDM_DAT_BLK_CNT_H_REG           _REG08( FDM_REG_BASE + (0xCD << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  DATA Block Count Register - 0x70CE                      default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DAT_BLK_CNT_M_REG           _REG08( FDM_REG_BASE + (0xCE << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  DATA Block Count Register - 0x70CF                      default:    0000_0001
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DAT_BLK_CNT_L_REG           _REG08( FDM_REG_BASE + (0xCF << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  DMA Count Register - 0x70D1                             default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RSV     RW
//--------------------------------------------------------------------------
#define FDM_DMA_CNT_H_REG               _REG08( FDM_REG_BASE + (0xD1 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  DMA Count Register - 0x70D2                             default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DMA_CNT_M_REG               _REG08( FDM_REG_BASE + (0xD2 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  DMA Count Register - 0x70D3                             default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_DMA_CNT_L_REG               _REG08( FDM_REG_BASE + (0xD3 << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  Page Count High Register - 0x70D4                           default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RSV     RSV     RSV     RSV     RSV     RSV     RSV     RW
//--------------------------------------------------------------------------
#define FDM_PAGE_CNT_H_REG              _REG08( FDM_REG_BASE + (0xD4 << FDM_REG_OFFSET ))



//--------------------------------------------------------------------------
//  Page Count Low Register - 0x70D5                        default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_PAGE_CNT_L_REG              _REG08( FDM_REG_BASE + (0xD5 << FDM_REG_OFFSET ))


//--------------------------------------------------------------------------
//  Busy Count Initial Register - 0x70D6                    default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_BUSY_CNT_INIT_REG           _REG08( FDM_REG_BASE + (0xD6 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Flash Defect Register - 0x70D7                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//  B0      :   CE0/Low
//  B1      :   CE0/High
//  B2      :   CE1/Low
//  B3      :   CE1/High
//  B4      :   CE2/Low
//  85      :   CE2/High
//  B6      :   CE3/Low
//  B7      :   CE3/High
//--------------------------------------------------------------------------
#define FDM_FSH_DEFECT_REG              _REG08( FDM_REG_BASE + (0xD7 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Manual mode data high Register - 0x70F0                 default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FDIO_H_REG                  _REG08( FDM_REG_BASE + (0xF0 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Manual mode data Low Register - 0x70F1                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FDIO_L_REG                  _REG08( FDM_REG_BASE + (0xF1 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Manual mode address high Register - 0x70F2                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FALE_H_REG                  _REG08( FDM_REG_BASE + (0xF2 << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  Manual mode Address Low Register - 0x70F3                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FALE_L_REG                  _REG08( FDM_REG_BASE + (0xF3 << FDM_REG_OFFSET ))




//--------------------------------------------------------------------------
//  Manual mode command high Register - 0x70F4                  default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FCLE_H_REG                  _REG08( FDM_REG_BASE + (0xF4 << FDM_REG_OFFSET ))





//--------------------------------------------------------------------------
//  Manual mode command Low Register - 0x70F5                   default:    0000_0000
//  BIT     :   B7      B6      B5      B4      B3      B2      B1      B0
//  R/W     :   RW      RW      RW      RW      RW      RW      RW      RW
//--------------------------------------------------------------------------
#define FDM_FCLE_L_REG                  _REG08( FDM_REG_BASE + (0xF5 << FDM_REG_OFFSET ))

#define FDM_CARDREADER_INT_REG              _REG08( FDM_FLASH_REG_BASE + (0x03 << FDM_REG_OFFSET ))
#define FDM_CARDREADER_INT_MASK_REG         _REG08( FDM_FLASH_REG_BASE + (0x04 << FDM_REG_OFFSET ))
#define FDM_CARDREADER_INT_STATUS_REG       _REG08( FDM_FLASH_REG_BASE + (0x05 << FDM_REG_OFFSET ))

#define FDM_FLASH_SIGNAL2			_REG08(FDM_REG_BASE + (0xf9 << FDM_REG_OFFSET))
#define FLASH_STS_RDY		   		0x10

#endif              // end of #ifndef _FDMREG_H_




#ifndef  _SYSTEMREG_H_
#define  _SYSTEMREG_H_

#define _REG(x)                              (*(volatile unsigned int   *)(x))

#define _REG32(x)                            (*(volatile unsigned int   *)(x))
#define _REG16(x)                            (*(volatile unsigned short *)(x))
#define _REG08(x)                            (*(volatile unsigned char  *)(x))

/**********************************************************************************
        GPIO
        0xC0FC_0000
**********************************************************************************/
#define GPIO_CTL_BASE                       0xC1FC0000

#define GPIO_DIR                            _REG32( GPIO_CTL_BASE + 0x00 )
#define GPIO_DATA                           _REG32( GPIO_CTL_BASE + 0x04 )
#define EGPIO_DIR                           _REG32( GPIO_CTL_BASE + 0x08 )
#define EGPIO_DATA                          _REG32( GPIO_CTL_BASE + 0x0C )
#define EGPIO_INT_TYPE                      _REG32( GPIO_CTL_BASE + 0x10 )
#define EGPIO_INT_STATUS                    _REG32( GPIO_CTL_BASE + 0x14 )


/**********************************************************************************
        PLL controller    
        0xC0FC_1800
**********************************************************************************/
#define PLL_CTL_BASE                        0xC0FC1800

#define PLL_CTL1                            _REG( PLL_CTL_BASE + 0 )
#define PLL_CTL2                            _REG( PLL_CTL_BASE + 4 )
#define PLL_RESET                           _REG( PLL_CTL_BASE + 8 )

/**********************************************************************************
        SDRAM controller    
        0xC0FC_7000~0xC0FC_70FF
**********************************************************************************/
#define SDRAM_CTL_BASE                      0xC0FC7000

#define SDRAM_ID                            _REG( SDRAM_CTL_BASE + 0 )
#define SDRAM_INT_CFG                       _REG( SDRAM_CTL_BASE + 4 )
#define SDRAM_CFG                           _REG( SDRAM_CTL_BASE + 8 )

#define DDR_CTL_BASE                        0xC0FCA000

#define DDR_ID                              _REG( DDR_CTL_BASE + 0x00 )
#define DDR_CFG0                            _REG( DDR_CTL_BASE + 0x04 )
#define DDR_CFG1                            _REG( DDR_CTL_BASE + 0x08 )
#define DDR_CFG2                            _REG( DDR_CTL_BASE + 0x0C )


/**********************************************************************************
        IDE controller  
        0xC0FC_9000~0xC0FC_90FF
**********************************************************************************/
#define IDE_CTL_BASE                        0xC0FC9000

#define IDE_ID                              _REG32( IDE_CTL_BASE + 0x00 )
#define IDE_PIO_TIMING                      _REG32( IDE_CTL_BASE + 0x04 )
#define IDE_CTL_STATUS                      _REG08( IDE_CTL_BASE + 0x08 )

#define IDE_DATA                            _REG16( IDE_CTL_BASE + 0xA0 )
// write
#define IDE_FEATURE                         _REG08( IDE_CTL_BASE + 0xA4 )
#define IDE_SECTOR_COUNT                    _REG08( IDE_CTL_BASE + 0xA8 )
#define IDE_SECTOR_NO                       _REG08( IDE_CTL_BASE + 0xAC )
#define IDE_CYCLINDER_LOW                   _REG08( IDE_CTL_BASE + 0xB0 )
#define IDE_CYCLINDER_HIGH                  _REG08( IDE_CTL_BASE + 0xB4 )
#define IDE_HEAD                            _REG08( IDE_CTL_BASE + 0xB8 )
#define IDE_COMMAND                         _REG08( IDE_CTL_BASE + 0xBC )
#define IDE_DEV_CONTROL                     _REG08( IDE_CTL_BASE + 0xD8 )

// read
#define IDE_ERROR                           IDE_FEATURE
#define IDE_STATUS                          IDE_COMMAND

#define IDE_LBA_LOW                         IDE_SECTOR_NO
#define IDE_LBA_MID                         IDE_CYCLINDER_LOW
#define IDE_LBA_HIGH                        IDE_CYCLINDER_HIGH


/**********************************************************************************
        Card Reader 
        0xC180_0000~0xC180_07FF
**********************************************************************************/



/**********************************************************************************
        USB2.0 Compliant Device Controller D-BUS    
        0xC182_0000~0xC182_00FF
**********************************************************************************/
#define USB_DBUS_BASE                       0xC1820000


#define USB_FIFO0                           _REG( USB_DBUS_BASE + (0x30<<F202CBUS_SHIFT) )
#define USB_FIFO2                           _REG( USB_DBUS_BASE + (0x32<<F202CBUS_SHIFT) )
#define USB_FIFO3                           _REG( USB_DBUS_BASE + (0x34<<F202CBUS_SHIFT) )

/**********************************************************************************
        I2S 
        0xC1FC_0400~0xC1FC_07FF
**********************************************************************************/

/**********************************************************************************
        Card Reader 
        0xC1FC_0800~0xC1FC_0BFF
**********************************************************************************/

/**********************************************************************************
        Card Reader Control 
        0xC1FC_0C00~0xC1FC_0FFF
**********************************************************************************/
#define CRCTL_BASE                          0xC1FC0C00
#define CRCTL_BUF_CTL                       _REG32( CRCTL_BASE + 0x00 )
#define CRCTL_BUF_CONFIRM                   _REG08( CRCTL_BASE + 0x04 )
#define CRCTL_BUF_DMA_BLK_LENGTH            _REG08( CRCTL_BASE + 0x08 )

// CRCTL_BUF_CTL
#define CR_USED_AS_RAM                      0x00000000
#define CR_XFER_FROM_CARD                   0x00000001
#define CR_XFER_TO_CARD                     0x00000002
#define CR_AUTO_MODE                        0x00000008
#define CR_ENABLE                           0x00000010
#define CR_BUFFER_0_FLAG                    0x00001000
#define CR_BUFFER_1_FLAG                    0x00002000
#define CR_BUFFER_2_FLAG                    0x00004000
#define CR_BUFFER_3_FLAG                    0x00008000
#define CR_CHK_CR_ALL_BUF_ACCESS_DONE       0x00020000                  // CRCTL_BUF_CTL.17
#define CR_CHK_CR_DMA_FINISH                0x00100000                  // CRCTL_BUF_CTL.20
#define CR_SET_TO_CLR_DMA_FINISH            0x00100000                  // CRCTL_BUF_CTL.20
#define CR_BUFFER_VALID                     1
#define CR_BUFFER_EMPTY                     0
#define CR_FLASH_MODE                       0x80000000                  // bit 31 = '1'
#define CR_CARDREADER_MODE                  0x00000000                  // bit 31 = '0'

// CRCTL_BUF_DMA_BLK_LENGTH
#define CR_MAX_DMA_BLOCK                    255

/**********************************************************************************
        AHB DMA 
        0xC1FC_1000~0xC1FC_13FF
**********************************************************************************/
#define DMA_BASE                            0xC1FC1000

#define DMA_CONFIG                          _REG32( DMA_BASE + 0x00 )
#define DMA_SRC_ADDR                        _REG32( DMA_BASE + 0x04 )
#define DMA_DEST_ADDR                       _REG32( DMA_BASE + 0x08 )
#define DMA_BYTE_SIZE                       _REG32( DMA_BASE + 0x0C )

// DMA_CONFIG
#define DMA_CLEAR_DIRECTION                 0xFFFFFF00 
#define DMA_SOURCE_SRAM                     0x00000000
#define DMA_SOURCE_CR_BANK                  0x00000001
#define DMA_SOURCE_I2S_AD                   0x00000002
#define DMA_SOURCE_USB_FIFO0                0x00000003
#define DMA_TARGET_SRAM                     0x00000000
#define DMA_TARGET_CR_BANK                  0x00000010
#define DMA_TARGET_I2S_AD                   0x00000020
#define DMA_TARGET_USB_FIFO0                0x00000030
#define DMA_CHK_DMA_XFER_DONE               0x00000100
#define DMA_SET_TO_CLR_DMA_XFER_FLAG        0x00000100
#define DMA_CHK_SOURCE_ERROR                0x00000200
#define DMA_CHK_TARGET_ERROR                0x00000400
#define DMA_EN_DMA_XFER_INT                 0x00001000
#define DMA_EN_SOURCE_ERROR_INT             0x00002000
#define DMA_EN_TARGET_ERROR_INT             0x00004000
#define DMA_SET_USB_HIGH_SPEED              0x00010000
#define DMA_CLR_USB_HIGH_SPEED              0xFFFEFFFF

// DMA_SRC_ADDR & DMA_DEST_ADDR
#define DMA_USB_FIFO_0_ADD                  0xC18200C0		// FUSB D-BUS AHB : FIFO0/FIFO1
#define DMA_USB_FIFO_2_ADD                  0xC18200C8		// FUSB D-BUS AHB : FIFO2
#define DMA_USB_FIFO_3_ADD                  0xC18200CC		// FUSB D-BUS AHB : FIFO3
#define DMA_CR_BUF_ADD                      0xC1800000

// DMA_BYTE_SIZE
#define DMA_MAX_LENGTH                      (1<<17)

/**********************************************************************************
        USB2.0 Compliant Device Controller C-BUS    
        0xC1FC_1400~0xC1FC_17FF
**********************************************************************************/
#define USB_CBUS_BASE                       0xC1FC1400
#define F202CBUS_SHIFT                      2

#define USB_MAIN_CTL                        _REG08( USB_CBUS_BASE + (0x00<<F202CBUS_SHIFT) )
        #define CHK_HIGH_SPEED         CHK_BIT_6
        #define EN_CHIP                SET_BIT_5
        #define DISEN_CHIP             CLR_BIT_5
        #define EN_SOFT_RST            SET_BIT_4
        #define CLR_SOFT_RST           CLR_BIT_4
        #define GO_TO_SUSPEND          SET_BIT_3
        #define CLR_GO_TO_SUSPEND      CLR_BIT_3
        #define EN_GLOBAL_INT          SET_BIT_2
        #define DISEN_GLOBAL_INT       CLR_BIT_2
        #define CHK_CAP_RMWKUP         CHK_BIT_0 
        #define SET_CAP_RMWKUP         SET_BIT_0 
        #define CLR_CAP_RMWKUP         CLR_BIT_0 
#define USB_DEV_ADR                         _REG08( USB_CBUS_BASE + (0x01<<F202CBUS_SHIFT) )
        #define CHK_AFT_CONF           CHK_BIT_7
        #define SET_AFT_CONF           SET_BIT_7
        #define CLR_AFT_CONF           CLR_BIT_7
#define USB_TST_EP                          _REG08( USB_CBUS_BASE + (0x02<<F202CBUS_SHIFT) )
        #define CLR_FIFO               SET_BIT_0
#define USB_PHY_TMS                         _REG08( USB_CBUS_BASE + (0x08<<F202CBUS_SHIFT) )
        #define SET_TST_PKT            SET_BIT_4
        #define CLR_TST_PKT            SET_BIT_4
        #define SET_TST_SE0NAK         SET_BIT_3
        #define CLR_TST_SE0NAK         SET_BIT_3
        #define SET_TST_KSTA           SET_BIT_2
        #define CLR_TST_KSTA           SET_BIT_2
        #define SET_TST_JSTA           SET_BIT_1
        #define CLR_TST_JSTA           SET_BIT_1
        #define SET_UNPLUG             SET_BIT_0
        #define CLR_UNPLUG             CLR_BIT_0
#define USB_VND_CTL                         _REG08( USB_CBUS_BASE + (0x09<<F202CBUS_SHIFT) )
#define USB_VND_STA                         _REG08( USB_CBUS_BASE + (0x0A<<F202CBUS_SHIFT) )
#define USB_CX_CSR                          _REG08( USB_CBUS_BASE + (0x0B<<F202CBUS_SHIFT) )
        #define SET_CX_FIFO_EMPTY      SET_BIT_5
        #define CLR_CX_FIFO_EMPTY      SET_BIT_5
        #define SET_CX_FIFO_FULL       SET_BIT_4
        #define CLR_CX_FIFO_FULL       SET_BIT_4
        #define SET_CX_FIFO_CLR        SET_BIT_3
        #define CLR_CX_FIFO_CLR        SET_BIT_3
        #define SET_CX_STL             SET_BIT_2
        #define CLR_CX_STL             CLR_BIT_2
        #define SET_TST_PKDONE         SET_BIT_1
        #define CLR_TST_PKDONE         CLR_BIT_1
        #define SET_CX_DONE            SET_BIT_0
        #define CLR_CX_DONE            CLR_BIT_0
#define USB_EP0_DP                          _REG08( USB_CBUS_BASE + (0x0C<<F202CBUS_SHIFT) )
#define USB_INT_MGRP                        _REG08( USB_CBUS_BASE + (0x10<<F202CBUS_SHIFT) )
        #define MASK_INT_SCR0          SET_BIT_0
        #define MASK_INT_SCR1          SET_BIT_1
        #define MASK_INT_SCR5          SET_BIT_5
        #define MASK_INT_SCR7          SET_BIT_7
        #define NOT_MASK_INT_SCR0      CLR_BIT_0
        #define NOT_MASK_INT_SCR1      CLR_BIT_1
        #define NOT_MASK_INT_SCR5      CLR_BIT_5
        #define NOT_MASK_INT_SCR7      CLR_BIT_7
#define USB_INT_MSKB0                       _REG08( USB_CBUS_BASE + (0x11<<F202CBUS_SHIFT) )
#define USB_INT_MSKB1                       _REG08( USB_CBUS_BASE + (0x12<<F202CBUS_SHIFT) )
        #define MASK_FIFO0_OUT_INT     0x03
        #define MASK_FIFO3_OUT_INT     0xC0
        #define NOT_MASK_FIFO0_OUT_INT 0xFC
        #define NOT_MASK_FIFO3_OUT_INT 0x3F
#define USB_INT_MSKB5                       _REG08( USB_CBUS_BASE + (0x16<<F202CBUS_SHIFT) )
        #define MASK_FIFO0_IN_INT      SET_BIT_0 
        #define MASK_FIFO2_IN_INT      SET_BIT_2 
        #define NOT_MASK_FIFO0_IN_INT  CLR_BIT_0 
        #define NOT_MASK_FIFO2_IN_INT  CLR_BIT_2 
#define USB_INT_MSKB7                       _REG08( USB_CBUS_BASE + (0x18<<F202CBUS_SHIFT) )
#define USB_FULLYEMPTY                      _REG08( USB_CBUS_BASE + (0x19<<F202CBUS_SHIFT) )
        #define CHK_EP3_EMPTY          CHK_BIT_1
        #define CHK_EP1_EMPTY          CHK_BIT_0
#define USB_INT_GRP                         _REG08( USB_CBUS_BASE + (0x20<<F202CBUS_SHIFT) )
        #define CHK_INT_SCR0           CHK_BIT_0
        #define CHK_INT_SCR1           CHK_BIT_1
        #define CHK_INT_SCR5           CHK_BIT_5
        #define CHK_INT_SCR7           CHK_BIT_7
        #define CLR_INT_SCR0           CLR_BIT_0
        #define CLR_INT_SCR1           CLR_BIT_1
        #define CLR_INT_SCR5           CLR_BIT_5
        #define CLR_INT_SCR7           CLR_BIT_7
#define USB_INT_SRCB0                       _REG08( USB_CBUS_BASE + (0x21<<F202CBUS_SHIFT) )
#define USB_INT_SRCB1                       _REG08( USB_CBUS_BASE + (0x22<<F202CBUS_SHIFT) )
        #define CHK_BULK_OUT           0x03
        #define CHK_ISOCHRONAL_OUT     0xC0
#define USB_INT_SRCB5                       _REG08( USB_CBUS_BASE + (0x26<<F202CBUS_SHIFT) )
        #define CHK_BULK_IN            0x01
        #define CHK_ISOCHRONAL_IN      0x04
#define USB_INT_SRCB7                       _REG08( USB_CBUS_BASE + (0x28<<F202CBUS_SHIFT) )
        #define CHK_RESM_INT           SET_BIT_3
        #define CHK_SUSP_INT           SET_BIT_2
        #define CHK_USBRST_INT         SET_BIT_1 
        #define CLR_RESM_INT           CLR_BIT_3
        #define CLR_SUSP_INT           CLR_BIT_2
        #define CLR_USBRST_INT         CLR_BIT_1 
        #define EN_RESM_INT            SET_BIT_3
        #define EN_SUSP_INT            SET_BIT_2
        #define EN_USBRST_INT          SET_BIT_1
        #define CLR_ALL_INT            0x00
#define USB_IDLE_CNT                        _REG08( USB_CBUS_BASE + (0x2F<<F202CBUS_SHIFT) )
#define USB_IEP1_CTL                        _REG08( USB_CBUS_BASE + (0x41<<F202CBUS_SHIFT) )
#define USB_IEP3_CTL                        _REG08( USB_CBUS_BASE + (0x45<<F202CBUS_SHIFT) )
        #define SET_TOGGLE_RESET_IEP   SET_BIT_4
        #define CLR_TOGGLE_RESET_IEP   CLR_BIT_4
        #define CHK_STALL_IEP          CHK_BIT_3
        #define SET_STALL_IEP          SET_BIT_3
        #define CLR_STALL_IEP          CLR_BIT_3
#define USB_OEP2_CTL                        _REG08( USB_CBUS_BASE + (0x63<<F202CBUS_SHIFT) )
#define USB_OEP4_CTL                        _REG08( USB_CBUS_BASE + (0x67<<F202CBUS_SHIFT) )
        #define SET_TOGGLE_RESET_OEP   SET_BIT_4
        #define CLR_TOGGLE_RESET_OEP   CLR_BIT_4
        #define CHK_STALL_OEP          CHK_BIT_3
        #define SET_STALL_OEP          SET_BIT_3
        #define CLR_STALL_OEP          CLR_BIT_3
#define USB_FIFO0_INST                      _REG08( USB_CBUS_BASE + (0xA0<<F202CBUS_SHIFT) )
#define USB_FIFO2_INST                      _REG08( USB_CBUS_BASE + (0xA2<<F202CBUS_SHIFT) )
#define USB_FIFO3_INST                      _REG08( USB_CBUS_BASE + (0xA3<<F202CBUS_SHIFT) )
        #define SET_FIFO_FRST          SET_BIT_4
        #define CLR_FIFO_FRST          CLR_BIT_4
        #define CHK_FIFO_XFERDONE      CHK_BIT_3
        #define SET_FIFO_XFERDONE      SET_BIT_3 
#define USB_FIFO0_BC                        _REG08( USB_CBUS_BASE + (0xB0<<F202CBUS_SHIFT) )
#define USB_FIFO2_BC                        _REG08( USB_CBUS_BASE + (0xB2<<F202CBUS_SHIFT) )
#define USB_FIFO3_BC                        _REG08( USB_CBUS_BASE + (0xB3<<F202CBUS_SHIFT) )
#define USB_FIFO0_DP                        _REG08( USB_CBUS_BASE + (0xC0<<F202CBUS_SHIFT) )
#define USB_FIFO2_DP                        _REG08( USB_CBUS_BASE + (0xC8<<F202CBUS_SHIFT) )
#define USB_FIFO3_DP                        _REG08( USB_CBUS_BASE + (0xCC<<F202CBUS_SHIFT) )


/**********************************************************************************
        USB2.0 Compliant Device Controller Wrapper  
        0xC1FC_1800~0xC1FC_1BFF
**********************************************************************************/
#define USB_WRAPPER_BASE                    0xC1FC1800


#define USB_EP0_CONTROL                     _REG08( USB_WRAPPER_BASE + (0x00<<F202CBUS_SHIFT) )
        #define SET_READ_CACHE_DONE    SET_BIT_0
        #define SET_WRITE_CACHE_DONE   SET_BIT_1
#define USB_BULK_IN_CBVC                    _REG08( USB_WRAPPER_BASE + (0x01<<F202CBUS_SHIFT) )
#define USB_INT_IN_CBVC                     _REG08( USB_WRAPPER_BASE + (0x02<<F202CBUS_SHIFT) )
#define USB_BUS_LOCK_REG                    _REG08( USB_WRAPPER_BASE + (0x03<<F202CBUS_SHIFT) )
        #define FREE_USB_BUS           0x56
        #define LOCK_USB_BUS           0x00  
#define USB_WRAPPER_INT                     _REG08( USB_WRAPPER_BASE + (0x04<<F202CBUS_SHIFT) )
        #define CHK_DETECT_MODE        CHK_BIT_7
        #define DETECT_USB_PLUG_OUT    SET_BIT_7
        #define DETECT_USB_PLUG_IN     CLR_BIT_7
        #define EN_VBUS_INT            SET_BIT_4
        #define DISEN_VBUS_INT         CLR_BIT_4
        #define CHK_VBUS_IN            SET_BIT_0

//#define USB_TTTT                            _REG08( USB_WRAPPER_BASE + 0x14 )



/**********************************************************************************
        SDRAM & DDR registers
**********************************************************************************/
#define REG_SDRAM_CFG1	0xC0FC7008
#define REG_SDRAM_CFG2	0xC0FC700C
#define REG_DDR_TIMING	0xC0FCA000
#define REG_DDR_CONFIG1	0xC0FCA008
#define REG_DDR_CONFIG2	0xC0FCA00C
#define REG_FUSE		0xC0FC1834

///////////////////////////////////////////////////////////////////////////////////
#define REG_CHIP_VER	0xC0FC1838


/**********************************************************************************
        NandFlash Controller (BackTop)  
        0xC1FC_1C00~0xC1FC_1FFF
**********************************************************************************/
//#include "FDMReg.h"

#endif   // _SYSTEMREG_H_




#ifndef LOADER_H_
#define LOADER_H_

//#include "Nftl.h"


#ifdef __KERNEL__
    #define KALIGN(x)       __attribute__((aligned(x)))
#else
    #define KALIGN(x)
#endif

//#define SYSTEM_PAGE_VALID                           (*(volatile unsigned int   *)(0x00020000))

#define CURRENT_SYSTEM_VERSION      0x00000100

#define IDE_SYS_PAGE_ADDR           0x00000000
#define NOR_SYS_PAGE_ADDR           0x00000000
#define SYSTEM_ID_LENGTH            16

#define KERNEL_MEM_ADDR             0x00030000

#define LOADER_BUF_SIZE             (512)

#define NFTL_DEFECT_TABLE_LBA       0x00000001

/**********************************************************************************
    NAND FLASH
**********************************************************************************/
//#include "NandIO.h"

#ifndef __KERNEL__
#pragma Align_members(1)
#endif

typedef struct _NAND_IMAGE_ {

    unsigned int   StartPart;
    unsigned int   PartitionNum;            // 1 partition = 512Bytes
    unsigned int   ImageStartAddr;
    unsigned int   ImageOffset;

}NAND_IMAGE, *PNAND_IMAGE;


typedef struct _NAND_SYS_PAGE_ {
    
    unsigned char           SystemPageID[SYSTEM_ID_LENGTH];         // 16 bytes      0
    unsigned int            SystemPageVersion;                      //  4 bytes	    16
    
}KALIGN(1) NAND_SYS_PAGE, *PNAND_SYS_PAGE;



#ifndef __KERNEL__
#pragma Align_members(4)
#endif


#endif /*LOADER_H_*/


#define NOR_ID_BYTE         3

#define NOR_ENABLE          ( FDM_FLASH_CTRL3_REG |= FSH_CTRL3_USE_SERIAL_NOR_FLASH )
#define NOR_DISABLE         ( FDM_FLASH_CTRL3_REG &= ~FSH_CTRL3_USE_SERIAL_NOR_FLASH )


//#define NOR_USE_CS          1
#define DEFAULT_NOR_USE_CS	1
#define NOR_CS_HIGH         ( FDM_CHIP_SEL_REG = CHIP_SEL_DISSEL_ALL )
#define NOR_CS_LOW          ( FDM_CHIP_SEL_REG &= ~(1 << gdwNorCE) )

#define NOR_WP_DISABLE      ( FDM_FSIGNAL_CTRL_REG |=  FSIGNAL_CTRL_WPb )
#define NOR_WP_ENABLE       ( FDM_FSIGNAL_CTRL_REG &= ~FSIGNAL_CTRL_WPb )

#endif /*NORFLASH_H_*/
