/*
 *	ARCmedia Monitor - Monitor the status of the ARCmedia engine
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	(c) Copyright 2005		ARC International plc
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/arc_mem.h>

MODULE_AUTHOR("ARC International");
MODULE_DESCRIPTION("ARC memory device");
MODULE_LICENSE("GPL");

static unsigned long uncached_addr; /* uncached region for mmap */
static unsigned long cached_addr; /* uncached region for mmap */
#define UNCACHED_SIZE (CONFIG_UNCACHED_REGION_SIZE * 1024)
#define CACHED_SIZE (CONFIG_CACHED_REGION_SIZE * 1024)

static int is_uncached_dev(struct inode *inode)
{
	return MINOR(inode->i_rdev) == ARC_UNCACHED_MEMORY_MINOR;
}

static int ioctl(struct inode *inode, struct file *file,
	unsigned int cmd, unsigned long arg)
{
	unsigned long *p = (unsigned long *)arg;
	unsigned long addr;
	int is_uncached = is_uncached_dev(inode);
	invalidate_region_t reg;
	int rc;

	switch (cmd) {

		case GET_PHYS:
		    	addr = is_uncached ? uncached_addr : cached_addr;
			return put_user(addr, p);
		case GET_LEN:
			return put_user(is_uncached ? UNCACHED_SIZE : CACHED_SIZE, p);
		case INVALIDATE_REGION:
			if (is_uncached)
			    	return -EINVAL;
			rc = copy_from_user(&reg, arg, sizeof(invalidate_region_t));
			if (rc)
			    	return rc;
			inv_dcache_range((unsigned long)reg.start, 
				(unsigned long)reg.start + reg.len);
			return 0;
		case FLUSH_DCACHE_REGION:
//			if (is_uncached)
//			    	return -EINVAL;
			rc = copy_from_user(&reg, arg, sizeof(invalidate_region_t));
			if (rc)
			    	return rc;
//printk("flush_dcache_range(%x,%x)\n", (unsigned long)reg.start, (unsigned long)reg.start + reg.len);
/*
			flush_dcache_range((unsigned long)reg.start, 
				(unsigned long)reg.start + reg.len);
*/
			flush_and_inv_dcache_all();
			return 0;

		default:
			break;
	}
	return -ENOTTY;
}

static int 
mmap(struct file *file, struct vm_area_struct * vma)
{
	unsigned long off = vma->vm_pgoff << PAGE_SHIFT;
	int is_uncached = is_uncached_dev(file->f_dentry->d_inode);
    
	if (off + vma->vm_end - vma->vm_start > (is_uncached ? UNCACHED_SIZE : CACHED_SIZE))
	    	return -EINVAL;

	if (is_uncached)
	{
		vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
		off += uncached_addr;
	}
	else
	    	off += cached_addr;

	vma->vm_pgoff = off >> PAGE_SHIFT;
	vma->vm_flags |= VM_IO | VM_RESERVED;

	if (remap_page_range(vma->vm_start, off, vma->vm_end - vma->vm_start, 
		vma->vm_page_prot))
		return -EAGAIN;
		
	return 0;
}


static struct file_operations fops = {
	owner:		THIS_MODULE,
	ioctl:		ioctl,
	mmap:		mmap,
};


static struct miscdevice cached_dev = {
	minor:		ARC_CACHED_MEMORY_MINOR,
	name:		"ARC cached memory",
	fops:		&fops,
};

static struct miscdevice uncached_dev = {
	minor:		ARC_UNCACHED_MEMORY_MINOR,
	name:		"ARC uncached memory",
	fops:		&fops,
};

static unsigned long setup_region(int size)
{
    	unsigned long addr = __get_free_pages(GFP_ATOMIC, get_order(size));
	unsigned long page;

	if (!addr)
	    	printk(KERN_CRIT "*** Failed to allocate %d bytes for arc memory region.", 
			size);
	
	for (page = addr; page < addr + size; page += PAGE_SIZE)
	    	SetPageReserved(virt_to_page(page));

	return addr;
}

static int __init arc_memory_init(void)
{
	int ret;

//	cached_addr = setup_region(CACHED_SIZE);
	cached_addr = END_MEM - UNCACHED_SIZE - CACHED_SIZE;
	ret = misc_register(&cached_dev);
	if (ret)
		return ret;
	
//	uncached_addr = setup_region(UNCACHED_SIZE);
	uncached_addr = END_MEM - UNCACHED_SIZE;
	ret = misc_register(&uncached_dev);
	if (ret)
		return ret;

	printk("Allocate cached_addr = 0x%.8x CACHED_SIZE = 0x%.8x\n", cached_addr, CACHED_SIZE);
	printk("Allocate uncached_addr=0x%.8x UNCACHED_SIZE=0x%.8x\n", uncached_addr, UNCACHED_SIZE);
	return 0;
}

static void __exit arc_memory_exit(void)
{
	free_pages(uncached_addr, get_order(UNCACHED_SIZE));
	free_pages(cached_addr, get_order(CACHED_SIZE));
	misc_deregister(&cached_dev);
	misc_deregister(&uncached_dev);
}

module_init(arc_memory_init);
module_exit(arc_memory_exit);

