/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * arch/arcnommu/kernel/irq.c
 *
 */

#include <linux/init.h>
#include <linux/irq.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/kernel_stat.h>

#include <asm/system.h>
#include <asm/signal.h>
#include <asm/errno.h>
#include <asm/arcregs.h>

#if 0 //wschen 2008-07-16
extern volatile unsigned long icse1, icse2, stat32, ilnk1, ilnk2, blnk;
#endif

extern int last_serial_int;

//FIXME :: 
#if 0 //wschen 2008-07-16
#define ARC_IRQ_DBG
#else
#undef ARC_IRQ_DBG
#endif

/* table for system interrupt handlers */

static struct irq_node *irq_list[NR_IRQS];

/* IRQ status spinlock - enable, disable */
static spinlock_t irq_controller_lock;

/* initialise the irq table */
void __init init_IRQ(void)
{
    int i;

    for(i = 0; i < NR_IRQS ; i++) {
		irq_list[i] = NULL;
    }

    /* set the base for the interrupt vector table */
    /* re-modify 20060804					 */
#ifdef CONFIG_CODE_OFFSET_FEATURE
	write_aux_reg(int_vector_base, CONFIG_CODE_OFFSET_TEXTBASE);
#else    
	write_aux_reg(int_vector_base, 0x80000000);
#endif
    /* Interrupts will be enabled in start_kernel in
     * init/main.c 
     */
#if 1 //wschen 2007-11-19 fix when booting, if IRQ 12 (USB device) interrupt comes before install service routine, then will cause system halted, so disable it
    {
      volatile unsigned int *usb_dev_reg = 0xC1FC1400;
      *usb_dev_reg &= 0xFFFFFFFB;
    }
#endif

}

/* setup the irq, insert the irq node into the list */

int setup_arc_irq(unsigned int irq, struct irq_node *node)
{
    unsigned long flags;
    struct irq_node *curr;

    if( !(irq_list[irq]) ) {
		irq_list[irq] = node;
		return 0;
    }
    if(!(irq_list[irq]->flags & IRQ_FLG_STD)) {
	    if(irq_list[irq]->flags & IRQ_FLG_LOCK) {
			printk("%s: IRQ %d from %s is not replaceable\n", __FUNCTION__, irq, 
				   irq_list[irq]->devname);
			return -EBUSY;
	    }
	    if (node->flags & IRQ_FLG_REPLACE) {
			printk("%s: %s can't replace IRQ %d from %s\n", __FUNCTION__, 
				   node->devname, irq, irq_list[irq]->devname);
			return -EBUSY;
	    }
    }

    /* the insertion needs to be done atomically */

    save_flags(flags);
    cli();

    curr = irq_list[irq];
    while(curr->next) 
		curr = curr->next;
    
    curr->next = node;

    restore_flags(flags);
    return 0;
}

int request_irq(unsigned int irq, void (*handler)( int , void *, struct pt_regs *),
				unsigned long flags, const char *devname, void *dev_id)
{
    struct irq_node *node;
    int retval;

    if(irq < 0 || irq >= NR_IRQS) {
		printk("%s: Requesting for unknown IRQ %d from %s\n", __FUNCTION__, irq, devname);
		return -ENXIO;
    }
    
    node = (struct irq_node *) kmalloc(sizeof(struct irq_node),GFP_KERNEL); 
    if(!node)
		return -ENOMEM;
    
    node->handler = handler;
    node->flags   = flags;
    node->dev_id  = dev_id;
    node->devname = devname;
		node->disable_depth = 0;
    node->next    = NULL;
    
    /* insert the new irq registered into the irq list */

    retval = setup_arc_irq(irq, node);
    if(retval)
		kfree(node);
    return retval;
}

/* free an irq node for the irq list */

void free_irq(unsigned int irq, void *dev_id)
{
	unsigned long flags;
	struct irq_node *curr, *node;

	if(irq < 0 || irq >= NR_IRQS) {
		printk("%s: Unknown IRQ %d\n", __FUNCTION__, irq);
		return;
	}
	save_flags(flags);
	cli();
	
	/* this case is if the first ndoe for that irq number itself has to be freed */
	node = curr = irq_list[irq];
	if(curr->dev_id == dev_id) {
		irq_list[irq] = node->next;
		kfree(curr);
		restore_flags(flags);
		return;
	}
	
	while(node) {
		if(curr->dev_id == dev_id) {
			node->next = curr->next;
			kfree(curr);
			restore_flags(flags);
			return;
		}
		node = curr;
		curr = curr->next;
	}
	restore_flags(flags);
	printk("%s: tried to remove invalid interrupt", __FUNCTION__);
}


/* handle the irq */
void process_interrupt(unsigned int irq, struct pt_regs *fp)
{
    struct irq_node *node;
#ifdef	ARC_IRQ_DBG
	int proc_cnt =0;
#endif	
    if(irq >= 0 && irq < NR_IRQS ) {
		kstat.irqs[0][irq]++;

		node = irq_list[irq];
		/* call all the ISR's in the list for that interrupt source */
		while(node) {
			node->handler(irq, node->dev_id, fp);
			node = node->next;
#ifdef	ARC_IRQ_DBG
			proc_cnt ++;
#endif	
		}
	}
	else
		panic("Can't process interrupt vector %u\n", irq);
#ifdef	ARC_IRQ_DBG
	if (!proc_cnt) 
	{
		unsigned *ptr;
		printk (KERN_ERR"Spurious interrupt : irq no %u\n", irq);
		printk (KERN_ERR"stack_place_holder 0x%08x\t bta 0x%08x\t lp_start 0x%08x\t lp_end 0x%08x\n", fp->stack_place_holder, fp->bta, fp->lp_start, fp->lp_end);
		printk (KERN_ERR"lp_count 0x%08x\t status32 0x%08x\t ret 0x%08x\t blink 0x%08x\t\n", fp->lp_count, fp->status32, fp->ret, fp->blink);
		printk (KERN_ERR"fp 0x%08x\t r26 0x%08x\t r12 0x%08x\t r11 0x%08x\t r10 0x%08x\t\n", fp->fp, fp->r26, fp->r12, fp->r11, fp->r10);
		printk (KERN_ERR"r9 0x%08x\t r8 0x%08x\t r7 0x%08x\t r6 0x%08x\t\n", fp->r9, fp->r8, fp->r7, fp->r6);
		printk (KERN_ERR"r5 0x%08x\t r4 0x%08x\t r3 0x%08x\t r2 0x%08x\t \n", fp->r5, fp->r4, fp->r3, fp->r2);
		printk (KERN_ERR"r1 0x%08x\t r0 0x%08x\t orig_r0 0x%08x\t orig_r8 0x%08x\t\n", fp->r1, fp->r0, fp->orig_r0, fp->orig_r8);
		printk (KERN_ERR"sp 0x%08x\t\n", fp->sp);
		printk (KERN_ERR"icause1 0x%08x\ticause2 0x%08x\n", icse1,icse2);
		printk (KERN_ERR"status 32 0x%08x\t ilink1 0x%08x\t ilink2 0x%08x\t blink 0x%08x\n", stat32, ilnk1, ilnk2, blnk);
		printk (KERN_ERR"Timer control reg 0x%08x, interrupt is %s\n", read_new_aux_reg(0x22), (read_new_aux_reg(0x22) & 4 ? "on" : "off"));
		ptr = 0xc0fc1014;
		printk (KERN_ERR"UART0 control 0x%08x, interrupt is %s\n",*ptr, (((*ptr & 0xc0) || (*ptr & 0x24) || (*ptr & 0x06) || (*ptr & 0x05))  ? "on" : "off"));
		ptr = 0xc0fc1114;
		printk (KERN_ERR"UART1 control 0x%08x, interrupt is %s\n",*ptr, (((*ptr & 0xc0) || (*ptr & 0x24) || (*ptr & 0x06) || (*ptr & 0x05))  ? "on" : "off"));
		ptr = 0xc0fc2004;
		printk (KERN_ERR"EMAC status 0x%08x, interrupts are %s\n",*ptr, (*ptr & 0x7 ? "present" : "absent"));
		
		printk("force jump to machine_restart(0)\n");
		machine_restart(0);
	}
#endif
	__cli();
	return;
}

int get_irq_list (char *buf)
{
	int i;
	struct irq_node *node;
	char *p = buf;

	for ( i=0 ; i<NR_IRQS; i++ ) {
		node = irq_list[i];
		if ( !node)
			continue; 
		p += sprintf (p, "%3d: %10u ", i, kstat_irqs(i));
		p += sprintf (p, "	%s", node->devname);
		for (node = node->next; node; node = node->next) {
			p += sprintf (p, "	%s", node->devname);
		}
		*p++ = '\n';
	}
	return (p - buf);

}
/* FIXME: implement if necessary */
unsigned long probe_irq_on(void)
{
	return 0;
}

/* FIXME: implement if necessary */
int probe_irq_off(unsigned long irqs)
{
	return 0;
}

/* FIXME: implement if necessary */
void init_irq_proc(void)
{
	/* place holder */
}


/**
 *		disable_irq - disable an irq and wait for completion
 *		@irq: Interrupt to disable
 *
 *		Disable the selected interrupt line.  We do this lazily.
 *
 *		This function may be called from IRQ context.
 */
void disable_irq(unsigned int irq)
{
	struct irq_node *node;
	unsigned long flags;
	unsigned long intbit;
	unsigned long reg;

	if ((irq >= 0 && irq < NR_IRQS) && (node = irq_list[irq])) {
		spin_lock_irqsave(&irq_controller_lock, flags);
		if (!node->disable_depth++) {
			intbit = ~(1<<irq);
			mask_interrupt(intbit);
		}
		spin_unlock_irqrestore(&irq_controller_lock, flags);
	}
}

/**
 *		enable_irq - enable interrupt handling on an irq
 *		@irq: Interrupt to enable
 *
 *		Re-enables the processing of interrupts on this IRQ line.
 *		Note that this may call the interrupt handler, so you may
 *		get unexpected results if you hold IRQs disabled.
 *
 *		This function may be called from IRQ context.
 */
void enable_irq(unsigned int irq)
{
	struct irq_node *node;
	unsigned long flags;
	unsigned long intbit;
	unsigned long reg;

	if ((irq >= 0 && irq < NR_IRQS) && (node = irq_list[irq])) {
 		spin_lock_irqsave(&irq_controller_lock, flags);
		if (!node->disable_depth) {
			printk("enable_irq(%u) unbalanced from 0x%x\n", irq,
						 __builtin_return_address(0));
		} else if (!--node->disable_depth) {
			intbit = 1<<irq;
			unmask_interrupt(intbit);			
		}
		spin_unlock_irqrestore(&irq_controller_lock, flags);
	}
}
