
/*
 * MTD driver for the SKY SPI Flash Memory support.
 *
 * sky-linux-2.4.29/drivers/mtd/devices/sky_spinor.c
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <asm/delay.h>
#include <asm/io.h>
#include "sky_spinor.h"

/* debugging */
/* #define SPINOR_DEBUG */
//#define SPINOR_DEBUG 1

//#define SPI_USE_DMA
#define MAX_PARTS 32

static char module_name[] = "sky_spinor";
static char FlashName[32];

#define MIN(a,b)        ((a) < (b) ? (a) : (b))
#define FALSE 	0
#define TRUE 	1

#define NB_OF(x) (sizeof (x) / sizeof (x[0]))

static int parts_no=0;

static void __init spidata_init(void);
int __init sky_spinor_init (void);
void __exit sky_spinor_exit (void);
static int sky_spinor_probe (struct mtd_info *);
static int sky_spinor_erase (struct mtd_info *mtd,struct erase_info *instr);
static int sky_spinor_read (struct mtd_info *mtd, loff_t from,size_t len,size_t *retlen,u_char *buf);
static int sky_spinor_write (struct mtd_info *mtd,loff_t to,size_t len,size_t *retlen,const u_char *buf);

extern void Nor_ManualWriteByte( unsigned char data );
extern unsigned int NorEraseSector( unsigned int addr );
extern unsigned int Nor_ManualWriteBuffer(unsigned int addr, unsigned int len, unsigned char* buffer );
extern unsigned int Nor_ManualReadBuffer(unsigned int addr, unsigned int len, unsigned char* buffer );
extern unsigned int Nor_DMAWriteBuffer(unsigned int addr, unsigned int len, unsigned char* buffer );
extern unsigned int Nor_DMAReadBuffer(unsigned int addr, unsigned int len, unsigned char* buffer );
extern unsigned int NorWaitForEraseDone(void);
extern unsigned int NorReadID(unsigned char *buffer);
extern unsigned int NorMDID(unsigned char *buffer);
/* Flash configuration table */
/* A25L80P_8MBIT */
static struct mtd_erase_region_info A25L80P_8MBIT_erase_regions[] =
{
   /* Sector 0-0 ~ 0-1 */
   { offset: 0x00000000, erasesize: 0x1000, numblocks: 2 },
   /* Sector 0-2 */
   { offset: 0x2000, erasesize: 0x2000, numblocks: 1 },
   /* Sector 0-3 */
   { offset: 0x4000, erasesize: 0x4000, numblocks: 1 },
   /* Sector 0-4 */
   { offset: 0x8000, erasesize: 0x8000, numblocks: 1 },
   /* Sector 1 ~ 15 */
   { offset: 0x10000, erasesize: 0x10000, numblocks: 15 }
};

/* MX25L1605_16MBIT */
static struct mtd_erase_region_info MX25L1605_16MBIT_erase_regions[] =
{
   /* Sector 0 ~ 31 */
   { offset: 0x0, erasesize: 0x10000, numblocks: 32 }
};

/* MX25L3205_32MBIT */
static struct mtd_erase_region_info MX25L3205_32MBIT_erase_regions[] =
{
   /* Sector 0 ~ 63 */
   { offset: 0x0, erasesize: 0x10000, numblocks: 64 }
};

#if 1 //wschen 2007-11-12
/* MX25L6405 */
static struct mtd_erase_region_info MX25L6405_64MBIT_erase_regions[] =
{
  /* Sector 0 ~ 127 */
  { offset: 0x0, erasesize: 0x10000, numblocks: 128 }
};
#endif

#if 1 //wschen 2007-08-22
/* SPANSION S25FL064A 64MBIT */
static struct mtd_erase_region_info S25FL064A_64MBIT_erase_regions[] =
{
  /* Sector 0 ~ 127 */
  { offset: 0x0, erasesize: 0x10000, numblocks: 128 }
};
#endif

#if 1 //wschen 2008-01-30
/* SPANSION S25FL128PF 128MBIT */
static struct mtd_erase_region_info S25FL128PF_128MBIT_erase_regions[] =
{
  /* Sector 0 ~ 255 */
  { offset: 0x0, erasesize: 0x10000, numblocks: 256 }
};

/* SPANSION S25FL128PFL 128MBIT */
static struct mtd_erase_region_info S25FL128PFL_128MBIT_erase_regions[] =
{
  /* Sector 0 ~ 63 */
  { offset: 0x0, erasesize: 0x40000, numblocks: 64 }
};
#endif

/* EN25B16_16MBIT */
static struct mtd_erase_region_info EN25B16_16MBIT_erase_regions[] =
{
   /* Sector 0 ~ 1 */
   { offset: 0x00000000, erasesize: 0x1000, numblocks: 2 },
   /* Sector 2 */
   { offset: 0x2000, erasesize: 0x2000, numblocks: 1 },
   /* Sector 3 */
   { offset: 0x4000, erasesize: 0x4000, numblocks: 1 },
   /* Sector 4 */
   { offset: 0x8000, erasesize: 0x8000, numblocks: 1 },
   /* Sector 5 ~ 35 */
   { offset: 0x10000, erasesize: 0x10000, numblocks: 31 }
};

/* EN25B16T_16MBIT */
static struct mtd_erase_region_info EN25B16T_16MBIT_erase_regions[] =
{
   /* Sector 0 ~ 30 */
   { offset: 0x00000000, erasesize: 0x10000, numblocks: 31 },
   /* Sector 31 */
   { offset: 0x001f0000, erasesize: 0x8000, numblocks: 1 },
   /* Sector 32 */
   { offset: 0x001f8000, erasesize: 0x4000, numblocks: 1 },
   /* Sector 33 */
   { offset: 0x001fc000, erasesize: 0x2000, numblocks: 1 },
   /* Sector 34 ~ 35 */
   { offset: 0x001fe000, erasesize: 0x1000, numblocks: 2 }
};

/* EN25P16_16MBIT */
static struct mtd_erase_region_info EN25P16_16MBIT_erase_regions[] =
{
   /* Sector 0 ~ 31 */
   { offset: 0x0, erasesize: 0x10000, numblocks: 32 }
};

/***********************************************************************/

static struct mtd_partition *sky_partitions;

static struct mtd_partition A25L80P_8MBIT_sky_partitions[] =
{
   {
	       name: "mtd0",
	     offset: 0x0,
	       size: 0x10000,
	 mask_flags: 0
   },
   {
	       name: "mtd1",
	     offset: 0x10000,
	       size: 0xf0000,
	 mask_flags: 0
   }
};

static struct mtd_partition MX25L1605_16MBIT_sky_partitions[] =
{
   {
	       name: "mtd0",
	     offset: 0x0,
	       size: 0x200000,
	 mask_flags: 0
   }
};

static struct mtd_partition MX25L3205_32MBIT_sky_partitions[] =
{
#if 0 //wschen 2007-06-11 
   {
	       name: "mtd0",
	     offset: 0x0,
	       size: 0x160000,
	 mask_flags: 0
   },
   {
	       name: "mtd1",
	     offset: 0x160000,
	       size: 0x2a0000,
	 mask_flags: 0
   }
#else
   {
	       name: "system partition",
	     offset: 0x0,
	       size: 0x3f0000,
	 mask_flags: 0
   },
   {
	       name: "config partition",
	     offset: 0x3f0000,
	       size: 0x010000,
	 mask_flags: 0
   }
#endif
};

#if 1 //wschen 2007-11-12
static struct mtd_partition MX25L6405_64MBIT_sky_partitions[] =
{
   {
	       name: "system partition",
	     offset: 0x0,
	       size: 0x7f0000,
	 mask_flags: 0
   },
   {
	       name: "config partition",
	     offset: 0x7f0000,
	       size: 0x010000,
	 mask_flags: 0
   }
};
#endif

#if 1 //wschen 2007-08-22
static struct mtd_partition S25FL064A_64MBIT_sky_partitions[] =
{
   {
	       name: "system partition",
	     offset: 0x0,
	       size: 0x7f0000,
	 mask_flags: 0
   },
   {
	       name: "config partition",
	     offset: 0x7f0000,
	       size: 0x010000,
	 mask_flags: 0
   }
};
#endif

#if 1 //wschen 2008-01-30
static struct mtd_partition S25FL128PF_128MBIT_sky_partitions[] =
{
   {
	       name: "system partition",
	     offset: 0x0,
	       size: 0xff0000,
	 mask_flags: 0
   },
   {
	       name: "config partition",
	     offset: 0xff0000,
	       size: 0x010000,
	 mask_flags: 0
   }
#if 1 //wschen 2008-02-19
   ,
   {
	       name: "sky partition",
	     offset: 0x0,
	       size: 0x0,
	 mask_flags: 0
   }
#endif
};

static struct mtd_partition S25FL128PFL_128MBIT_sky_partitions[] =
{
   {
	       name: "system partition",
	     offset: 0x0,
	       size: 0xfc0000,
	 mask_flags: 0
   },
   {
	       name: "config partition",
	     offset: 0xfc0000,
	       size: 0x040000,
	 mask_flags: 0
   }
#if 1 //wschen 2008-02-19
   ,
   {
	       name: "sky partition",
	     offset: 0x0,
	       size: 0x0,
	 mask_flags: 0
   }
#endif
};
#endif

static struct mtd_partition SK2850DEMO_16MBIT_sky_partitions[] =
{
   {
	       name: "mtd0",
	     offset: 0x0,
	       size: 0x200000,
	 mask_flags: 0
   },
};

/* Driver private data structure */
struct sky_spinor_data {
	struct 	mtd_info       *mtd;	
	struct 	mtd_partition  *parsed_parts;     /* parsed partitions */
};

static struct sky_spinor_data *spidata;

/***************************************************************************************************/

/* Probe SPI flash device
 * Function returns 0 for failure.
 * and flashconfig_tbl array index for success.
 */
static int 
sky_spinor_probe (struct mtd_info *mtd)
{
#if 0 //wschen 2008-01-30
	__u32 sig;
#else
	__u64 sig;
#endif

   	int flash_size;

	if (!spidata)
		spidata_init();
	
	if (!spidata) /* init failed */
		return 0;

#if 1 //wschen 2008-01-30
	memset(&sig, 0, sizeof(sig));
#endif

   	/* Read the signature on the flash device */
   	NorReadID( (unsigned char *)&sig);
#if 0 //wschen 2008-01-30
        printk("Flash Signature ID: 0x%.8x\n", sig);
#else
        printk("Flash Signature ID: 0x%.16llx\n", sig);
#endif


   	switch (sig) {
   	  case A25L80P_8MBIT_SIGNATURE:
   	    mtd->size = A25L80P_8MBIT_BYTE_COUNT;
   	    mtd->erasesize = A25L80P_8MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (A25L80P_8MBIT_erase_regions);
   	    mtd->eraseregions = A25L80P_8MBIT_erase_regions;
   	    parts_no = NB_OF(A25L80P_8MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&A25L80P_8MBIT_sky_partitions;
   	    sprintf(FlashName, "A25L80P 8MBIT");
   	    break;
   	  case MX25L1605_16MBIT_SIGNATURE:
   	    mtd->size = MX25L1605_16MBIT_BYTE_COUNT;
   	    mtd->erasesize = MX25L1605_16MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (MX25L1605_16MBIT_erase_regions);
   	    mtd->eraseregions = MX25L1605_16MBIT_erase_regions;
   	    parts_no = NB_OF(MX25L1605_16MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&MX25L1605_16MBIT_sky_partitions;
   	    sprintf(FlashName, "MX25L1605 16MBIT");
   	    break;
   	  case MX25L3205_32MBIT_SIGNATURE:
   	    mtd->size = MX25L3205_32MBIT_BYTE_COUNT;
   	    mtd->erasesize = MX25L3205_32MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (MX25L3205_32MBIT_erase_regions);
   	    mtd->eraseregions = MX25L3205_32MBIT_erase_regions;
   	    parts_no = NB_OF(MX25L3205_32MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&MX25L3205_32MBIT_sky_partitions;
   	    sprintf(FlashName, "MX25L3205 32MBIT");
   	    break;

#if 1 //wschen 2007-11-12
   	  case MX25L6405_64MBIT_SIGNATURE:
   	    mtd->size = MX25L6405_64MBIT_BYTE_COUNT;
   	    mtd->erasesize = MX25L6405_64MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (MX25L6405_64MBIT_erase_regions);
   	    mtd->eraseregions = MX25L6405_64MBIT_erase_regions;
   	    parts_no = NB_OF(MX25L6405_64MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&MX25L6405_64MBIT_sky_partitions;
   	    sprintf(FlashName, "MX25L6405 64MBIT");
   	    break;
#endif

#if 1 //wschen 2007-08-22
   	  case S25FL064A_64MBIT_SIGNATURE:
   	    mtd->size = S25FL064A_64MBIT_BYTE_COUNT;
   	    mtd->erasesize = S25FL064A_64MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (S25FL064A_64MBIT_erase_regions);
   	    mtd->eraseregions = S25FL064A_64MBIT_erase_regions;
   	    parts_no = NB_OF(S25FL064A_64MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&S25FL064A_64MBIT_sky_partitions;
   	    sprintf(FlashName, "S25FL064A 64MBIT");
   	    break;
#endif

#if 1 //wschen 2008-01-30
   	  case S25FL128PF_128MBIT_SIGNATURE:
   	    mtd->size = S25FL128PF_128MBIT_BYTE_COUNT;
   	    mtd->erasesize = S25FL128PF_128MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (S25FL128PF_128MBIT_erase_regions);
   	    mtd->eraseregions = S25FL128PF_128MBIT_erase_regions;
   	    parts_no = NB_OF(S25FL128PF_128MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&S25FL128PF_128MBIT_sky_partitions;
   	    sprintf(FlashName, "S25FL128PF 128MBIT");
   	    break;

   	  case S25FL128PFL_128MBIT_SIGNATURE:
   	    mtd->size = S25FL128PFL_128MBIT_BYTE_COUNT;
   	    mtd->erasesize = S25FL128PFL_128MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (S25FL128PFL_128MBIT_erase_regions);
   	    mtd->eraseregions = S25FL128PFL_128MBIT_erase_regions;
   	    parts_no = NB_OF(S25FL128PFL_128MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&S25FL128PFL_128MBIT_sky_partitions;
   	    sprintf(FlashName, "S25FL128PFL 128MBIT");
   	    break;
#endif

   	  case EN25_SIGNATURE:
   	    sig = 0;
   	    NorMDID((unsigned char *)&sig);
            printk("EN25 Series Flash Manufacturer/Device ID : 0x%.8x\n", sig);

            switch(sig)
            {
              case EN25B16_16MBIT_SIGNATURE:
   	        mtd->size = EN25B16_16MBIT_BYTE_COUNT;
   	        mtd->erasesize = EN25B16_16MBIT_SECTOR_SIZE;
   	        mtd->numeraseregions = NB_OF (EN25B16_16MBIT_erase_regions);
   	        mtd->eraseregions = EN25B16_16MBIT_erase_regions;
//   	        parts_no = NB_OF(EN25B16_16MBIT_sky_partitions);
   	        parts_no = NB_OF(SK2850DEMO_16MBIT_sky_partitions);
//   	        sky_partitions = (struct mtd_partition *)&EN25B16_16MBIT_sky_partitions;
   	        sky_partitions = (struct mtd_partition *)&SK2850DEMO_16MBIT_sky_partitions;
   	        sprintf(FlashName, "EN25B16 16MBIT");
   	        break;
              case EN25B16T_16MBIT_SIGNATURE:
   	        mtd->size = EN25B16T_16MBIT_BYTE_COUNT;
   	        mtd->erasesize = EN25B16T_16MBIT_SECTOR_SIZE;
   	        mtd->numeraseregions = NB_OF (EN25B16T_16MBIT_erase_regions);
   	        mtd->eraseregions = EN25B16T_16MBIT_erase_regions;
//   	        parts_no = NB_OF(EN25B16T_16MBIT_sky_partitions);
   	        parts_no = NB_OF(SK2850DEMO_16MBIT_sky_partitions);
//   	        sky_partitions = (struct mtd_partition *)&EN25B16T_16MBIT_sky_partitions;
   	        sky_partitions = (struct mtd_partition *)&SK2850DEMO_16MBIT_sky_partitions;
   	        sprintf(FlashName, "EN25B16T 16MBIT");
   	        break;
              case EN25P16_16MBIT_SIGNATURE:
   	        mtd->size = EN25P16_16MBIT_BYTE_COUNT;
   	        mtd->erasesize = EN25P16_16MBIT_SECTOR_SIZE;
   	        mtd->numeraseregions = NB_OF (EN25P16_16MBIT_erase_regions);
   	        mtd->eraseregions = EN25P16_16MBIT_erase_regions;
//   	        parts_no = NB_OF(EN25P16_16MBIT_sky_partitions);
   	        parts_no = NB_OF(SK2850DEMO_16MBIT_sky_partitions);
//   	        sky_partitions = (struct mtd_partition *)&EN25P16_16MBIT_sky_partitions;
   	        sky_partitions = (struct mtd_partition *)&SK2850DEMO_16MBIT_sky_partitions;
   	        sprintf(FlashName, "EN25P16 16MBIT");
   	        break;
              default:
                break;
            }
   	    break;
          default:
	    printk (KERN_WARNING "%s: Read of flash device signature failed!\n", module_name);
#if 1	//Robert 20070711, Do not assign default MTD info, NAND&Nor save_conf will conflict
	    sprintf(FlashName, "NoFlash");
	    mtd->size = 0;
#else
//   	  case MX25L3205_32MBIT_SIGNATURE:
	    printk (KERN_WARNING "%s: Use MX25L3205_32MBIT as default setting!\n", module_name);
   	    mtd->size = MX25L3205_32MBIT_BYTE_COUNT;
   	    mtd->erasesize = MX25L3205_32MBIT_SECTOR_SIZE;
   	    mtd->numeraseregions = NB_OF (MX25L3205_32MBIT_erase_regions);
   	    mtd->eraseregions = MX25L3205_32MBIT_erase_regions;
   	    parts_no = NB_OF(MX25L3205_32MBIT_sky_partitions);
   	    sky_partitions = (struct mtd_partition *)&MX25L3205_32MBIT_sky_partitions;
   	    sprintf(FlashName, "MX25L3205 32MBIT");
#endif
   	    break;
   	}

   	return (mtd->size);
}


static int 
sky_spinor_erase (struct mtd_info *mtd,struct erase_info *instr)
{
	struct opcodes *ptr_opcode;
	__u32 temp, reg;
	int finished = FALSE;

#ifdef SPINOR_DEBUG
   	printk ("%s(addr = 0x%.8x, len = %d)\n",__FUNCTION__,instr->addr,instr->len);
#endif

   	/* sanity checks */
   	if (instr->addr + instr->len > mtd->size) return (-EINVAL);

#ifdef SPINOR_DEBUG
   	printk("DEBUG in sky_spinor_erase addr:0x%.8x\n", instr->addr);
#endif

        /* check WIP status first*/
        NorWaitForEraseDone();
   	NorEraseSector(instr->addr);
   	NorWaitForEraseDone();

   	instr->state = MTD_ERASE_DONE;
   	if (instr->callback) instr->callback (instr);

#ifdef SPINOR_DEBUG
   	printk ("%s return\n",__FUNCTION__);
#endif
   	return (0);
}


static int 
sky_spinor_read (struct mtd_info *mtd, loff_t from,size_t len,size_t *retlen,u_char *buf)
{
#ifdef SPINOR_DEBUG
   	printk ("%s(from = 0x%.8x, len = %d)\n",__FUNCTION__,(__u32) from,(int)len);  
#endif

   	/* sanity checks */
   	if (!len) return (0);
   	if (from + len > mtd->size) return (-EINVAL);

   	/* we always read len bytes */
   	*retlen = len;

#ifdef SPI_USE_DMA
   	Nor_DMAReadBuffer(from, len, buf);
#else
   	Nor_ManualReadBuffer(from, len, buf);
#endif

   	return (0);
}

static int 
sky_spinor_write (struct mtd_info *mtd,loff_t to,size_t len,size_t *retlen,const u_char *buf)
{
	int done = FALSE, page_offset, bytes_left, finished;
	__u32 xact_len, spi_data = 0, opcode, reg;

#ifdef SPINOR_DEBUG
   	printk ("%s(to = 0x%.8x, len = %d)\n",__FUNCTION__,(__u32) to,len); 
#endif

   	*retlen = 0;
	
   	/* sanity checks */
   	if (!len) return (0);
   	if (to + len > mtd->size) return (-EINVAL);
	
	bytes_left = len;
	
	while (done == FALSE) {
		xact_len = bytes_left;

		/* 32-bit writes cannot span across a page boundary
		 * (256 bytes). This types of writes require two page
		 * program operations to handle it correctly.
		 */
		page_offset = (to & (SPINOR_PAGE_SIZE - 1)) + xact_len;

		if (page_offset > SPINOR_PAGE_SIZE) {
			xact_len -= (page_offset - SPINOR_PAGE_SIZE);
		}

#ifdef SPI_USE_DMA
		if (xact_len % 512 == 0)	// only do 512*n bytes 
		{
                  Nor_DMAWriteBuffer(to, xact_len, buf);
                }
                else
#endif
                {
                  Nor_ManualWriteBuffer(to, xact_len, buf);
                }

		bytes_left -= xact_len;
		to += xact_len;
		buf += xact_len;

   		*retlen += xact_len;

		if (bytes_left == 0) {
			done = TRUE;
		}
	}

   	return (0);
}

static void __init spidata_init(void)
{
  if (spidata)
    return;

  spidata = kmalloc(sizeof(struct sky_spinor_data), GFP_KERNEL);

  if (!spidata)
    return;
}

int __init 
sky_spinor_init (void)
{
   	int result, i, j;
	u32 len;
   	int ret;
	struct mtd_info *mtd;
	struct  mtd_partition  *mtd_parts;
	char *buf;
	struct mtd_partition *part;
	struct squashfs_super_block *sb;
	u32 config_start;

	spidata_init();

  	if (!spidata)
		return (-ENXIO);
	
   	mtd = kmalloc(sizeof(struct mtd_info), GFP_KERNEL);
   	if (!mtd) {
		kfree(spidata);
		return (-ENXIO);
	}
	
   	memset (mtd,0,sizeof (*mtd));
	
   	printk ("MTD driver for SKY SPI flash.\n");
   	printk ("%s: Probing for Serial flash ...\n", module_name);
   	if (!(ret = sky_spinor_probe (mtd))) {
    	  printk (KERN_WARNING "%s: Found no serial flash device\n", module_name);
	  kfree(mtd);
	  kfree(spidata);
          return (-ENXIO);
   	}

   	printk ("%s: Found SPI serial Flash. (%s)\n", module_name, FlashName);
   	printk ("%s: size : %d\n", module_name, mtd->size);

   	mtd->name = module_name;
   	mtd->type = MTD_NORFLASH;
   	mtd->flags = (MTD_CAP_NORFLASH|MTD_WRITEABLE);

//        mtd->size = 0x100000;
//        mtd->erasesize = 0x10000;
//   	mtd->numeraseregions = NB_OF (erase_regions);
//   	mtd->eraseregions = erase_regions;

   	mtd->module = THIS_MODULE;
   	mtd->erase = sky_spinor_erase;
   	mtd->read = sky_spinor_read;
//printk("mtd->read = 0x%.8x\n", (unsigned int)mtd->read);
   	mtd->write = sky_spinor_write;
	
#ifdef SPINOR_DEBUG
	printk (KERN_DEBUG
		   "mtd->name = %s\n"
		   "mtd->size = 0x%.8x (%uM)\n"
		   "mtd->erasesize = 0x%.8x (%uK)\n"
		   "mtd->numeraseregions = %d\n",
		   mtd->name,
		   mtd->size, mtd->size / (1024*1024),
		   mtd->erasesize, mtd->erasesize / 1024,
		   mtd->numeraseregions);

   	if (mtd->numeraseregions) {
	 	for (result = 0; result < mtd->numeraseregions; result++) {
	   		printk (KERN_DEBUG
			   "\n\n"
			   "mtd->eraseregions[%d].offset = 0x%.8x\n"
			   "mtd->eraseregions[%d].erasesize = 0x%.8x (%uK)\n"
			   "mtd->eraseregions[%d].numblocks = %d\n",
			   result,mtd->eraseregions[result].offset,
			   result,mtd->eraseregions[result].erasesize,mtd->eraseregions[result].erasesize / 1024,
			   result,mtd->eraseregions[result].numblocks);
         	}
    	}
#endif

#ifdef SPINOR_DEBUG
   for (result = 0; result < parts_no; result++)
	 printk (KERN_DEBUG
			 "\n\n"
			 "sky_partitions[%d].name = %s\n"
			 "sky_partitions[%d].offset = 0x%.8x\n"
			 "sky_partitions[%d].size = 0x%.8x (%uK)\n",
			 result,sky_partitions[result].name,
			 result,sky_partitions[result].offset,
			 result,sky_partitions[result].size,sky_partitions[result].size / 1024);
#endif

        /* add sky partitions */
        result = add_mtd_partitions (mtd,sky_partitions,parts_no);

	spidata->mtd = mtd;

   	return (result);
}

void __exit 
sky_spinor_exit (void)
{
    	if (spidata && spidata->parsed_parts) {
        	del_mtd_partitions (spidata->mtd);
		kfree(spidata->mtd);
		kfree(spidata);
    	}
}

module_init (sky_spinor_init);
module_exit (sky_spinor_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("SkyMedi - Robert");
MODULE_DESCRIPTION("MTD driver for SKY SPI Flash on 2850");

