#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/file.h>
#include <linux/mount.h>
#include <linux/smp_lock.h>
#include <linux/mman.h>

#define BADFS_MAGIC	0xBADF5001

static struct vfsmount *badfs_mnt;	/* returned by kern_mount() */

static struct inode *badfs_get_inode(struct super_block *sb, int mode)
{
  struct inode *inode = new_inode(sb);

  if (inode) {
    make_bad_inode(inode);
    inode->i_mode = mode;
  }
  return inode;
}

static struct super_block * badfs_read_super(struct super_block *sb, void *data, int silent)
{
  struct inode *inode = new_inode(sb);
  if (!inode)
    return NULL;

  make_bad_inode(inode);

  sb->s_magic = BADFS_MAGIC;
  sb->s_root = d_alloc(NULL, &(const struct qstr) { "badfs:", 6, 0 });
  if (!sb->s_root) {
    iput(inode);
    return NULL;
  }
  sb->s_root->d_sb = sb;
  sb->s_root->d_parent = sb->s_root;
  d_instantiate(sb->s_root, inode);
  return sb;
}

static DECLARE_FSTYPE(badfs_fs_type, "badfs", badfs_read_super, FS_NOMOUNT);

static int __init init_badfs_fs(void)
{
  int err = register_filesystem(&badfs_fs_type);

  if (!err) {
    badfs_mnt = kern_mount(&badfs_fs_type);
    if (IS_ERR(badfs_mnt)) {
      err = PTR_ERR(badfs_mnt);
      unregister_filesystem(&badfs_fs_type);
    }
  }
  return err;
}

module_init(init_badfs_fs)

static void disable_pwd(struct fs_struct *fs)
{
  struct inode *inode;
  struct dentry *dentry;
  inode = badfs_get_inode(badfs_mnt->mnt_sb, S_IFDIR|0755);
  if (!inode) {
    printk(KERN_ERR "disable_pwd(): can't allocate inode\n");
    return;
  }
  dentry = d_alloc(badfs_mnt->mnt_sb->s_root,
      &(const struct qstr){"dead_pwd", 8, 0});
  if (!dentry) {
    iput(inode);
    printk(KERN_ERR "disable_pwd(): can't allocate dentry\n");
    return;
  }
  d_instantiate(dentry, inode);
  dget(dentry);

  lock_kernel();
  set_fs_pwd(fs, badfs_mnt, dentry);
  unlock_kernel();
}

static void disable_root(struct fs_struct *fs)
{
  struct inode *inode;
  struct dentry *dentry;

  inode = badfs_get_inode(badfs_mnt->mnt_sb, S_IFDIR|0755);
  if (!inode) {
    printk(KERN_ERR "disable_root(): can't allocate inode\n");
    return;
  }
  dentry = d_alloc(badfs_mnt->mnt_sb->s_root,
      &(const struct qstr){"dead_root", 9, 0});
  if (!dentry) {
    iput(inode);
    printk(KERN_ERR "disable_root(): can't allocate dentry\n");
    return;
  }
  d_instantiate(dentry, inode);
  dget(dentry);

  lock_kernel();
  set_fs_root(fs, badfs_mnt, dentry);
  unlock_kernel();
}

static int check_dirs(struct vfsmount *mnt)
{
  struct task_struct *p;

repeat:
  if (signal_pending(current))
    return 0;

  read_lock(&tasklist_lock);
  for_each_task(p) {
    struct fs_struct *fs;

    /* get a reference to p->fs */
    task_lock(p);
    fs = p->fs;
    if (!fs) {
      task_unlock(p);
      continue;
    } else
      atomic_inc(&fs->count);
    task_unlock(p);

    if (fs->pwdmnt == mnt) {
      read_unlock(&tasklist_lock);
      disable_pwd(fs); /* may sleep */
      put_fs_struct(fs);
      goto repeat;
    }
    if (fs->rootmnt == mnt) {
      read_unlock(&tasklist_lock);
      disable_root(fs); /* may sleep */
      put_fs_struct(fs);
      goto repeat;
    }
    put_fs_struct(fs);
  }
  read_unlock(&tasklist_lock);

  return 1;
}

static struct file *get_bad_file(void)
{
  struct inode *i;
  struct dentry *d;
  struct file *f = get_empty_filp();
  if (!f) {
    printk(KERN_ERR "get_bad_file(): can't allocate file\n");
    goto err;
  }

  i = badfs_get_inode(badfs_mnt->mnt_sb, 0755);
  if (!i) {
    printk(KERN_ERR "get_bad_file(): can't allocate inode\n");
    goto err_inode;
  }

  d = d_alloc(badfs_mnt->mnt_sb->s_root,
      &(const struct qstr){"dead_file", 9, 0});
  if (!d) {
    printk(KERN_ERR "get_bad_file(): can't allocate dentry\n");
    goto err_dentry;
  }

  d_instantiate(d, i);

  f->f_vfsmnt = mntget(badfs_mnt);
  f->f_dentry = dget(d);
  f->f_op = i->i_fop;

  return f;

err_dentry:
  iput(i);
err_inode:
  filp_close(f, NULL);
err:
  return NULL;
}

static int check_files(struct vfsmount *mnt)
{
  struct task_struct *p;
  struct file *file, *badfile = NULL;
  struct inode *inode;

repeat:
  if (signal_pending(current)) {
    return 0;
  }

  /* Allocate a bad file in advance, so it's not done with
     files_lock held.  I'm not sure what the lock order issues are
     with doing that stuff with the lock held. */

  if (!badfile)
    badfile = get_bad_file();

  read_lock(&tasklist_lock);
  for_each_task(p) {
    unsigned int fd, j = 0;
    struct files_struct *files;

    /* get a reference to p->files */
    task_lock(p);
    files = p->files;
    if (!files) {
      task_unlock(p);
      continue;
    } else {
      atomic_inc(&files->count);
      spin_lock(&files->file_lock);
    }
    task_unlock(p);

    /* check if this process has open files here */
    while (1) {
      unsigned long set;

      fd = j * __NFDBITS;
      if (fd >= files->max_fdset || fd >= files->max_fds)
	break;
      set = files->open_fds->fds_bits[j++];
      while (set) {
	if (set & 1) {
	  file = files->fd[fd];
	  if (file) {
	    inode = file->f_dentry->d_inode;
	    if (inode && (file->f_vfsmnt == mnt)) {
	      /* If allocation failed, the forced unmount
		 fails.  This seems safer than just closing
		 the fd.  Note that it will retry until
		 either it succeeds or a signal is received. */

	      if (badfile) {
		// Preserve mode so as to get EIO rather than
		// EBADF.
		badfile->f_mode = file->f_mode;

		files->fd[fd] = badfile;
		badfile = NULL;
	      }

	      spin_unlock(&files->file_lock);
	      read_unlock(&tasklist_lock);

	      filp_close(file, files);
	      put_files_struct(files);

	      goto repeat;
	    }
	  }
	}
	fd++;
	set >>= 1;
      }
    }
    spin_unlock(&files->file_lock);
    put_files_struct(files);
  }
  read_unlock(&tasklist_lock);

  if (badfile)
    filp_close(badfile, NULL);

  return 1;
}

static int bad_readpage(struct file *f, struct page *p)
{
  return -EIO;
}


/* This consists of parts of functions called by munmap(),
   and parts called by mmap().  It switches the mapping
   to a badfs inode, using the same vma so there's no
   chance of failure. */

static void disable_mmap(struct vm_area_struct *vma)
{
  struct mm_struct *mm = vma->vm_mm;
  struct vm_area_struct *prev = mm->mmap;

  if (prev == vma)
    prev = NULL;
  else while (prev->vm_next != vma)
    prev = prev->vm_next;

  spin_lock(&mm->page_table_lock);
  rb_erase(&vma->vm_rb, &mm->mm_rb);
  mm->map_count--;
  mm->mmap_cache = NULL;

  if (prev)
    prev->vm_next = vma->vm_next;

  free_pgtables(mm, prev, vma->vm_start, vma->vm_end);
  spin_unlock(&mm->page_table_lock);

  remove_shared_vm_struct(vma);

  if (vma->vm_ops && vma->vm_ops->close)
    vma->vm_ops->close(vma);
  if (vma->vm_file)
    fput(vma->vm_file);

  vma->vm_flags &= VM_READ | VM_WRITE | VM_EXEC |
    VM_MAYREAD | VM_MAYWRITE | VM_MAYEXEC;
  vma->vm_ops = NULL;
  vma->vm_file = get_bad_file();
  vma->vm_private_data = NULL;
  vma->vm_next = NULL;

  generic_file_mmap(vma->vm_file, vma);
  insert_vm_struct(mm, vma);
}

static struct vm_area_struct *find_mmap_on_fs(struct mm_struct *mm,
    struct vfsmount *mnt)
{
  struct vm_area_struct *vma;
  struct file *file;
  struct inode *inode;

  /* check for mmap'd files and unmap them */
  for (vma = mm->mmap; vma; vma = vma->vm_next) {
    file = vma->vm_file;
    if (!file)
      continue;
    inode = file->f_dentry->d_inode;
    if (!inode || !inode->i_sb)
      continue;
    if (file->f_vfsmnt == mnt)
      return vma;
  }

  return NULL;
}

static int check_mmaps(struct vfsmount *mnt)
{
  struct list_head *node;

repeat:
  if (signal_pending(current))
    return 0;

  spin_lock(&mmlist_lock);

  list_for_each(node, &init_mm.mmlist) {
    struct mm_struct *mm = list_entry(node, struct mm_struct, mmlist);
    struct vm_area_struct *vma;

    spin_lock(&mm->page_table_lock);
    vma = find_mmap_on_fs(mm, mnt);
    spin_unlock(&mm->page_table_lock);

    if (vma) {
      atomic_inc(&mm->mm_users);
      spin_unlock(&mmlist_lock);

      down_write(&mm->mmap_sem);

      /* It might have gone away while neither page_table_lock
	 nor mmap_sem were held. */
      vma = find_mmap_on_fs(mm, mnt);
      if (vma)
	disable_mmap(vma);

      up_write(&mm->mmap_sem);
      mmput(mm);
      goto repeat;
    }
  }

  spin_unlock(&mmlist_lock);
  return 1;
}


void quiesce_filesystem(struct vfsmount *mnt)
{
  /* we do three passes through the task list, examining:
   *   1. p->fs{->pwd,root} that can keep this mnt busy
   *   2. p->files, i.e. open files (we do_close them)
   *   3. p->mm, i.e. mmaped files (we simply do_munmap them)
   * There is no guarantee that by the time we restart the loop
   * the amount of work to do in the loop has not increased.
   *
   * SAW: There's also no guarantee that all activity has ceased by
   * the time this is done, or that further references will not be
   * obtained after this is done (the latter could be fixed by
   * detaching from the namespace first).  If either happens, the
   * unmount will fail as if it weren't forced, and the user needs
   * to try again.  It'd be nice if the retries happened
   * automatically (either in the kernel or in the mount utility),
   * so that scripts don't break in weird cases...
   */

  if (!check_dirs(mnt))
    return;

  if (!check_files(mnt))
    return;

  if (!check_mmaps(mnt))
    return;
}

