/*
 * BRIEF MODULE DESCRIPTION
 *	ARC Pixel Graphics Unit framebuffer driver.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of	the GNU General	Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * THIS	SOFTWARE IS PROVIDED``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.	 IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the	 GNU General Public License along
 * with this program; if not, write	 to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright 2004-2006 ARC International
 *
 * The ARC PGU allows the user to send two video outputs to one display
 * simultaneously. From the hardware perspective, there is a main display
 * and a secondary overlay display. The main display can be run in RGB555 or
 * YUV 4:2:0 (aka YV12) modes, the overlay display in RGBA4444, RGBA5551 and
 * RGB555 with colour keying. Each display has a distinct buffer space. When
 * both displays are running, the contents of the overlay buffer are alpha
 * blended in hardware onto the contents of the primary buffer as the display
 * is rendered.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
/* Flowbee@960327 */
#include <linux/interrupt.h>
#include <asm/arcregs.h>
#include <asm/uaccess.h>
#include <asm/serial.h>

#include "arc_disp_param.h"
#include "arc_pgu.h"
#include "arc_vsync.h"
//#include "terawins_tcon.h"

#include <video/fbcon.h>
#include <video/fbcon-mfb.h>
#include <video/fbcon-cfb2.h>
#include <video/fbcon-cfb4.h>
#include <video/fbcon-cfb8.h>
#include <video/fbcon-cfb16.h>

/*----------------------------------------------------------------------------*/
/* Flowbee@960310 */
#define CACHED_UNCACHED_MEM_SIZE	((CONFIG_UNCACHED_REGION_SIZE + CONFIG_CACHED_REGION_SIZE) << 10)

#define ID "ARC PGU FB"

#define PGU_BASEADDR				0xC0FC8000 // FIXME take this from config or boot parameter
#define YPbPr_BASEADDR 				0xC0FC1300
#define TV_BASEADDR 				0xC0FC1500
#define SCALAR_BASEADDR				0xC0FC1400

#define CMAPSIZE 16

#define BASIC_ALIGNMENT 8
#define DEFAULT_ALIGN_SIZE 32
#define BUF_ALIGN(x, alignment) 	((x) & ~alignment)

#define VOLATILE_MEMSET(s,c,n) 		memset((void *) s, c, n)

/* Define the interrupt vector number. */
#define TIMER1_IRQ					4
#define PIX_IRQ						8
#define YPBPR_IRQ					28
#define SCALAR_IRQ					29

//== 2008-01-30 Flowbee ReInit terawins tcon parameters.
#define REDUCE_SCALE_DOWN_LINES		2
#define REDUCE_SIZE_AT_TV_VIDEO		0

//#define abs_sub(x,y) (x >= y ? (x-y) : (y-x))   jimmy, move to UI layer
/*----------------------------------------------------------------------------*/

DECLARE_MUTEX(pgu_sem);

/*----------------------------------------------------------------------------*/

struct arc_pgufb_par {
	struct fb_var_screeninfo var;

	int line_length;	/* in bytes */
	int cmap_len;			/* color-map length */
	unsigned long main_mode;
	unsigned long overlay_mode;
	int num_rgbbufs;
	int rgb_bufno;
	int main_is_fb;
};

/* Framebuffer driver info */
struct arc_pgufb_info {
	struct fb_info_gen gen;
	unsigned long fb_virt_start;
	unsigned long fb_size;
	unsigned long fb_phys;
	unsigned long yuv_max_size;
	unsigned long rgb_size;
	struct yuv_info yuv;
	struct yuv_info default_yuv;

	int mmaped;
	int nohwcursor;

	struct { unsigned red, green, blue, pad; } palette[256];

	unsigned long yuv_base[CONFIG_ARCPGU_YUVBUFS];

#if defined(FBCON_HAS_CFB16)
	u16 fbcon_cmap16[16];
#endif
};

/*----------------------------------------------------------------------------*/
static sky_scalar *scalar_ptr = (sky_scalar*) SCALAR_BASEADDR;
static sky_ypbpr *ypbpr_ptr = (sky_ypbpr*) YPbPr_BASEADDR;
static sky_tv *tv_ptr = (sky_tv*) TV_BASEADDR;
static volatile arc_pgu *pgu_ptr = (arc_pgu *) PGU_BASEADDR;
static int display_index = 0;
#if CONFIG_ARCPGU_TV_OUT_YPBPR
static int tv_interface = SKY_TV_YPBPR;
#else
static int tv_interface = SKY_TV_CVBS_SVIDEO;
#endif
struct known_displays *p_displays;
struct known_displays *p_current_display;
//static struct arc_pgufb_info fb_info;
struct arc_pgufb_info fb_info;
static struct arc_pgufb_par current_par;
static struct display disp;
static struct yuv_info tmpyuv;

/* Flowbee */
int itmpval;
static unsigned long val = 0;
volatile unsigned long vtval;
unsigned long volatile temp_val = 0;
static struct yuv_info dec_yuv;
unsigned long dec_yuv_base[CONFIG_ARCPGU_YUVBUFS];
unsigned char dec_num_buffers = CONFIG_ARCPGU_YUVBUFS;		// the decode buf nums.
unsigned char scalar_num_buffers = 2;	// mplayer allocate the scalar buf nums.
unsigned char disp_num_buffers = 0;		// the display buf nums.
unsigned char is_video_play = 0;		// 1: video play, 0: video not play.
unsigned char is_video_scale = 0;		// 1: scale, 0: not scale.
unsigned long mplayer_play_time = 0;	// video play time set by mplayer, get by skydemo.

//Barry 2008-12-09
unsigned long video_start_time = 0;	// video start time set by mplayer, get by skydemo.
unsigned long video_end_time = 0;		// video end time set by mplayer, get by skydemo.
unsigned long mplayer_flag = 0;		// mplayer flag set by mplayer, get by skydemo.

unsigned long mplayer_duration = 0;		// video duration set by mplayer, get by skydemo.
unsigned long video_time_onoff = 0;		// video time on/off flag.
int mplayer_status = MPLAYER_INIT; //Mel: int mplayer_status = 0;

unsigned long do_mplayer_uninit = 1;	// 1: do mplayer un-init function. 0: undo mplayer un-init function.
unsigned char the_same_resolution = 1;	// 1: the same resolution. 0: different resolution.

//== Flowbee 2007-11-12 Advertisment
unsigned char update_advertisment_frame = 0;
unsigned char update_advertisment_frame_cnt = 0;
unsigned char frame_cnt[CONFIG_ARCPGU_YUVBUFS];
unsigned char mplayer_advertisment_flag = 0;
struct area_info adver_video_info;
struct update_info adver_jpeg_info;
unsigned long rgb0_addr;
unsigned long rgb1_addr;
char simd_backup_buf[128*256];

//== Flowbee 2008-01-28 Re-Init T-CON parameters
unsigned char tcon_contrast = 0;
unsigned char tcon_brightness = 0;
extern struct _terawins_osd_info video_speed;
extern struct _terawins_osd_info subtitle;
extern int *TerawinsCmdTbl[TERAWINS_END*2];

//== Flowbee 2007-11-12 Net config
struct net_config net_info;
//== Jimmy 2008-06-11 Nas and RSS Config
struct nas_config nas_info;
struct rss_flickr_config rss_flickr_info;
struct rss_picasa_config rss_picasa_info;
struct rss_fc_config rss_fc_info;

struct _scalar_info
{
	unsigned int state;
	struct _buf_info info;
	sky_scalar yuv[3];			// yuv[0]-> Y info, yuv[1]-> U info, yuv[2]-> V info
};
static struct _scalar_info scalar;


volatile int yuv_queue_count = 0;
struct yuv_queue
{
  struct yuv_queue *prev;
  struct yuv_queue *next;
//  int yuvno;
  struct yuv_info user_defined_yuv;
} volatile yuv_queue_root;

void enqueue_user_defined_yuv(struct yuv_info *new_yuv);
void dequeue_yuv(void);
//-- Flowbee 2007-11-19
static int set_rgb_buf(unsigned char _buf);
static int set_yuv_buf(unsigned char _buf);
static int calculate_yuv_offsets(struct yuv_info *info);
static void copy_pgu(arc_pgu *src, arc_pgu *dest);
static int arc_pgu_check_dims(struct yuv_info *info);
//-- For advertisment
void move_buf_data_simd(unsigned char src, unsigned char dest);


//void deflicker(int fb);     jimmy, move to UI layer
extern struct consw *conswitchp;
extern struct consw fb_con;
extern unsigned long arc700_clkfreq;

/*----------------------------------------------------------------------------*/
int				 arc_pgufb_init(void);
void			 arc_pgufb_setup(char *options, int *ints);
static int arc_pgu_blank(int blank_mode, struct fb_info_gen *info);
static int arc_pgufb_ioctl(struct inode *inode, struct file *file, u_int cmd,
	u_long arg, int con, struct fb_info *info);

static void arc_pgu_setdims(struct yuv_info *info);
static void init_disp_parameter(void);
static int init_fb_info(void);
//static void init_pgu_ypbpr(void);
int init_pgu_ypbpr(void);
static void turn_all_osd_off(void);

static void arc_pgu_nocursor(struct display *p, int mode, int xx, int yy){};

static struct fb_ops arc_pgufb_ops = {
	owner:						THIS_MODULE,
	fb_get_fix:				fbgen_get_fix,
	fb_get_var:				fbgen_get_var,
	fb_set_var:				fbgen_set_var,
	fb_get_cmap:			fbgen_get_cmap,
	fb_set_cmap:			fbgen_set_cmap,
	fb_pan_display:		fbgen_pan_display,
	fb_ioctl:					arc_pgufb_ioctl
};

//-- T-CON function
void arc_tcon_on_off(unsigned char _on)
{
	tcon_on_off(_on);
}
void arc_tcon_pwm_on_off(unsigned char _on)
{
	tcon_pwm_on_off(_on);
}
void arc_tcon_backlight_on_off(unsigned char _on)
{
	tcon_backlight_on_off(_on);
}
void arc_tcon_set_pwm(unsigned char _val)
{
	tcon_set_pwm(_val);
}
void arc_tcon_set_brightness(unsigned char _val)
{
	tcon_set_brightness(_val);
}
void arc_tcon_set_contrast(unsigned char _val)
{
	tcon_set_contrast(_val);
}

/*----------------------------------------------------------------------------*/
static void wait_jiffies(unsigned long _num)
{
	unsigned long volatile tmp_jiffies;
	unsigned long _tmp;

	tmp_jiffies = jiffies + _num;
	do{
		_tmp ++;
	}while(jiffies < tmp_jiffies);
}

/*----------------------------------------------------------------------------*/
static void delay(unsigned long ns)
{
	do {
		unsigned long _x;
		for(_x=0; _x<ns; _x++){
			__asm__ __volatile__("nop");
		}
	} while(0);
}

//**	GPIO function.
/*--------------------------------------------------------------------------------*/
void set_gpio_bit(unsigned long _addr, unsigned long _mask)
{
	vtval = GIO_R32(_addr);
	vtval = GIO_R32(_addr);
	vtval |= _mask;
	GIO_W32(_addr, vtval);
}

/*--------------------------------------------------------------------------------*/
unsigned long get_gpio_bit(unsigned long _addr, unsigned long _mask)
{
	vtval = GIO_R32(_addr);
	vtval = GIO_R32(_addr);
	return (((vtval & _mask) > 0)? 1: 0);
}

/*--------------------------------------------------------------------------------*/
void clear_gpio_bit(unsigned long _addr, unsigned long _mask)
{
	vtval = GIO_R32(_addr);
	vtval = GIO_R32(_addr);
	vtval &= ~(_mask);
	GIO_W32(_addr, vtval);
}

#if 0
/*----------------------------------------------------------------------------*/
int HimaxTCONCmd[][2]=
{	/* Addr, Val */
	{  0x00, 0xCF  },	// interface: 601 port0, resolution: 1440x234 stripe.
	{  0x01, 0x52  },
	{  0x02, 0xFF  },
	{  0x03, 0x00  },
	{  0x04, 0x00  },
	{  0x05, 0x00  },
	{  0x06, 0x48  },
	{  0x07, 0x06  },
	{  0x08, 0x13  },
	{  0x09, 0x1C  },
	{  0x0A, 0x11  },
	{  0x0B, 0x15  },
	{  0x0C, 0x0A  },
	{  0x0D, 0x0F  },
	{  0x0E, 0x16  },
	{  0x0F, 0x44  },
	{  0x10, 0x13  },
	{  END_CMD, END_CMD  },
};

/*----------------------------------------------------------------------------*/
static void spi_start(unsigned char rw)
{
	/* Set SCL, SDI output */
	SPI_SCL_OUT;
	SPI_SDI_OUT;
	SPI_CS_OUT;
	/* Set SCL to high */
	SPI_CS_HI;
	SPI_SDI_HI;
	SPI_SCL_HI;
	delay(SPI_START_TIME);
	/* Set SPI read/write */
	SPI_SCL_LO;
	if (rw == SPI_WRITE){
		SPI_SDI_LO;
	}
	else{
		SPI_SDI_HI;
	}

	delay(SPI_CLOCK_TIME);
	SPI_SCL_HI;
	delay(SPI_CLOCK_TIME);
}

/*----------------------------------------------------------------------------*/
static void spi_stop(void)
{
	SPI_SCL_LO;
	delay(SPI_CLOCK_TIME);
	SPI_SCL_HI;
	delay(SPI_CLOCK_TIME);

	SPI_SDI_HI;
	SPI_CS_LO;
	delay(SPI_STOP_TIME);
}

/*----------------------------------------------------------------------------*/
static void spi_write(unsigned char reg_val, unsigned char bit_cnt)
{
	unsigned int i;

	i = (1 << (bit_cnt-1));

	for ( ; i>0; i>>=1 ){

		SPI_SCL_LO;

		if ( (i&reg_val) == 0 ){
			SPI_SDI_LO;
		}
		else{
			SPI_SDI_HI;
		}

		delay(SPI_CLOCK_TIME);

		SPI_SCL_HI;
		delay(SPI_CLOCK_TIME);
	}
}

/*----------------------------------------------------------------------------*/
static void spi_write_cmd(unsigned char reg_addr, unsigned char reg_val)
{
	unsigned char spi_addr = reg_addr;
	unsigned char spi_val = reg_val;

	spi_start(SPI_WRITE);
	spi_write(spi_addr, SPI_ADDR_CNT);
	spi_write(spi_val, SPI_DATA_CNT);
	spi_stop();
}

/*----------------------------------------------------------------------------*/
/**
 * init_himax8817a() - Init the Himax 8817A T-CON.
 *
 * Init the Himax 8817A T-CON for the CPT CLAA070WA03 TFT LCD.
 **/
static void init_himax8817a(void)
{
	int i = 0;
	/* Writing command to the T-CON via the SPI interface. */
	while(HimaxTCONCmd[i][0] != END_CMD){
		spi_write_cmd((unsigned char)HimaxTCONCmd[i][0], (unsigned char) HimaxTCONCmd[i][1]);
		i ++;
	}
}

#endif //~#if (CONFIG_ARCPGU_DISPTYPE > DISP_INNOLUX_800600_TFT)

#if 1
/******************************************************************************/
enum gpio_type
{
	SKY_GPIO 	= 0,
	SKY_EGPIO	= 1,
	SKY_GPIOH	= 2,
};

//== spi test
struct sky_spi_info
{
	uint32_t en;
	uint32_t cs_gio_type;
	uint32_t cs_gio_pin;
	uint32_t sclk_gio_type;
	uint32_t sclk_gio_pin;
	uint32_t sdio_gio_type;
	uint32_t sdio_gio_pin;
	uint32_t clk_cycle;
	uint32_t init_delay;
	uint32_t cmd_delay;
	uint32_t bit_len;
};

#define GPIO_DIR_ADDR				0xC1FC0000
#define GPIO_DATA_ADDR				0xC1FC0004
#define EGPIO_DIR_ADDR				0xC1FC0008
#define EGPIO_DATA_ADDR				0xC1FC000C
#define GPIOH_DIR_ADDR				0xC1FC0020
#define GPIOH_DATA_ADDR				0xC1FC0024

#define readb(addr) 				(*(volatile unsigned char *) (addr))
#define readw(addr) 				(*(volatile unsigned short *) (addr))
#define readl(addr) 				(*(volatile unsigned int *) (addr))
#define writeb(b,addr) 				(*(volatile unsigned char *) (addr) = (b))
#define writew(b,addr) 				(*(volatile unsigned short *) (addr) = (b))
#define writel(b,addr) 				(*(volatile unsigned int *) (addr) = (b))

#define readwbits(add, shift, mask)			((readl(add)&(mask))>>shift)  
#define writewbits(value, add, shift, mask)	writel((readl(add)&(~mask))|(value<<shift),add)  

#define REG_SET_BITS(value, add, shift, mask)	writel((readl(add)&(~(mask<<shift)))|((value&mask)<<shift),add)
#define REG_GET_BITS(add, shift, mask)			((readl(add)&(mask<<shift))>>shift) 

#define SPI_CS_HIGH                 REG_SET_BITS(1, gpio_data_addr[spi_pin_info.cs_gio_type], spi_pin_info.cs_gio_pin, 1);
#define SPI_CS_LOW                  REG_SET_BITS(0, gpio_data_addr[spi_pin_info.cs_gio_type], spi_pin_info.cs_gio_pin, 1);

#define SPI_CLK_HIGH                REG_SET_BITS(1, gpio_data_addr[spi_pin_info.sclk_gio_type], spi_pin_info.sclk_gio_pin, 1);
#define SPI_CLK_LOW                 REG_SET_BITS(0, gpio_data_addr[spi_pin_info.sclk_gio_type], spi_pin_info.sclk_gio_pin, 1);

#define SPI_SDI_HIGH                REG_SET_BITS(1, gpio_data_addr[spi_pin_info.sdio_gio_type], spi_pin_info.sdio_gio_pin, 1);
#define SPI_SDI_LOW                 REG_SET_BITS(0, gpio_data_addr[spi_pin_info.sdio_gio_type], spi_pin_info.sdio_gio_pin, 1);

struct sky_spi_info spi_pin_info;

uint32_t gpio_dir_addr[] = 
{
	GPIO_DIR_ADDR,
	EGPIO_DIR_ADDR,
	//GPIOH_DIR_ADDR,
};

uint32_t gpio_data_addr[] = 
{
	GPIO_DATA_ADDR,
	EGPIO_DATA_ADDR,
	//GPIOH_DATA_ADDR,
};

void sky_spi_pin_info_init(void)
{
	spi_pin_info.en 			= 1;
	spi_pin_info.cs_gio_type	= SKY_GPIO;
	spi_pin_info.cs_gio_pin		= 6;
	spi_pin_info.sclk_gio_type	= SKY_GPIO;
	spi_pin_info.sclk_gio_pin	= 4;
	spi_pin_info.sdio_gio_type	= SKY_GPIO;
	spi_pin_info.sdio_gio_pin	= 5;
	spi_pin_info.clk_cycle		= 150;
	spi_pin_info.init_delay		= 50;
	spi_pin_info.cmd_delay		= 100;
	spi_pin_info.bit_len		= 16;	
}

#if 0
uint32_t gpio_mux_addr[] = 
{
	GPIO_PIN_MUX_REG_ADDR,
	NULL,
	GPIOH_PIN_MUX_REG_ADDR,
};

void sky_spi_pin_mux_init(void)
{
	//-- cs
	if (gpio_mux_addr[spi_pin_info.cs_gio_type] != NULL) {
		REG_SET_BITS(1, gpio_mux_addr[spi_pin_info.cs_gio_type], spi_pin_info.cs_gio_pin, 1);
	}
	//-- sclk
	if (gpio_mux_addr[spi_pin_info.sclk_gio_type] != NULL) {
		REG_SET_BITS(1, gpio_mux_addr[spi_pin_info.sclk_gio_type], spi_pin_info.sclk_gio_pin, 1);
	}
	//-- sdio
	if (gpio_mux_addr[spi_pin_info.sdio_gio_type] != NULL) {
		REG_SET_BITS(1, gpio_mux_addr[spi_pin_info.sdio_gio_type], spi_pin_info.sdio_gio_pin, 1);
	}	
}
#endif

void sky_spi_pin_output(void)
{
	REG_SET_BITS(1, gpio_dir_addr[spi_pin_info.cs_gio_type], spi_pin_info.cs_gio_pin, 1);
	REG_SET_BITS(1, gpio_dir_addr[spi_pin_info.sclk_gio_type], spi_pin_info.sclk_gio_pin, 1);
	REG_SET_BITS(1, gpio_dir_addr[spi_pin_info.sdio_gio_type], spi_pin_info.sdio_gio_pin, 1);
}

void sky_spi_pin_init(void)
{
	// set pin info
	sky_spi_pin_info_init();
	// set pin mux to gpio
//	sky_spi_pin_mux_init();
	// set gpio output
	sky_spi_pin_output();
}

void sky_spi_start(void)
{
    SPI_CS_HIGH ;
    SPI_CLK_HIGH ;
    SPI_SDI_HIGH ;
    delay(spi_pin_info.init_delay);
    SPI_CS_LOW ;
	delay(spi_pin_info.init_delay);
//    SPI_CLK_LOW ;
//    udelay(spi_pin_info.init_delay);
}

void sky_spi_stop(void)
{
    SPI_CLK_HIGH ;
	delay(spi_pin_info.init_delay);
    SPI_SDI_HIGH ;
    delay(spi_pin_info.init_delay);
    SPI_CS_HIGH ;
	delay(spi_pin_info.cmd_delay);
}

void sky_spi_clock(void)  
{
	SPI_CLK_LOW;
	delay(spi_pin_info.clk_cycle);
	SPI_CLK_HIGH;
	delay(spi_pin_info.clk_cycle);
// 	SPI_CLK_LOW;
//	udelay(spi_pin_info.clk_cycle);
}

void sky_spi_send(uint32_t spi_data, uint32_t spi_len) 
{
	uint32_t i;
	uint32_t spi_mask;

	spi_mask = 1 << (spi_len - 1);
	
	for (i = 0; i < spi_len; i++) {
		if (spi_data & spi_mask) {
			SPI_SDI_HIGH;
		}
		else {
			SPI_SDI_LOW;
		}
		delay(1);
		spi_mask >>= 1; 
		sky_spi_clock();
	}
}

void sky_spi_write(uint32_t data, uint32_t len)
{
	printk("\n sky_spi_write: %08lx", data);
	sky_spi_start();	
	sky_spi_send(data, len);  //ILI9325_DEVICE|0x4
	sky_spi_stop();
}


#endif

/*************************************************************************
 *	LCD backlight control function
 *************************************************************************/
/*-----------------------------------------------------------------------*/
void backlight_on_off(unsigned char _on)
{
	if ( (display_index != DISP_NTSC)
		&& (display_index != DISP_PAL)
		&& (display_index != DISP_HDTV)){

		BL_OUT;
		if(_on == 1){
			BL_ON;
		}
		else{
			BL_OFF;
		}
	}
}

/*----------------------------------------------------------------------------*/
static void blank_yuv(struct yuv_info *info)
{
	int i;
	unsigned char y_val, cbcr_val;

	y_val = 0x00;
	cbcr_val = 0x80;

	for (i = 0; i < info->num_buffers; i++){
		// Reset the data in the Y buffer to 0.
		VOLATILE_MEMSET(((void*)fb_info.fb_virt_start) +
			fb_info.yuv_base[i], y_val, (info->width*info->height));
		// Reset the data in the U and V buffer to 0x88.
		VOLATILE_MEMSET(((void*)fb_info.fb_virt_start) +
			fb_info.yuv_base[i] + (info->width*info->height), cbcr_val,
			(info->width*info->height)/2);
	}
}

/*----------------------------------------------------------------------------*/
static void blank_yuv_buf(struct yuv_info *info, unsigned char buf_num)
{
	unsigned long y_addr;
	unsigned char y_val, cbcr_val;

    FB_MSG("\nblank_yuv_buf(%d)\n", buf_num);
	y_val = 0x00;
	cbcr_val = 0x80;

	y_addr = fb_info.fb_virt_start + fb_info.yuv_base[buf_num];
	// Reset the data in the Y buffer to 0.
	VOLATILE_MEMSET((void*)y_addr, y_val, info->u_offset);
	// Reset the data in the U and V buffer to 0x88.
	VOLATILE_MEMSET(((void*)y_addr + info->u_offset), cbcr_val,	(info->u_offset / 2));

	flush_dcache_range(y_addr, (fb_info.fb_virt_start + fb_info.yuv_base[buf_num + 1]));
}

/*----------------------------------------------------------------------------*/
static void blank_yuv_block(struct yuv_info *info, struct area_info *adv_info)
{
	unsigned long y_addr, cb_addr, cr_addr;
	unsigned long base_addr;
	unsigned long x, y, x_size, y_size;
	unsigned char y_val, cbcr_val;
	unsigned long i, j;

	FB_MSG("\n blank_yuv_block\n");
	x = adv_info->co_x & ~(0x01);
	y = adv_info->co_y & ~(0x01);
	x_size = adv_info->res_w & ~(0x01);
	y_size = adv_info->res_h & ~(0x01);	
	y_val = 0x00;
	cbcr_val = 0x80;

	for (j = 0; j < info->num_buffers; j++) {
		base_addr = fb_info.fb_virt_start + fb_info.yuv_base[j];
		y_addr = base_addr + (y * info->y_stride) + x;
		cb_addr = base_addr + info->u_offset + ((y/2) * info->u_stride) + (x/2);
		cr_addr = base_addr + info->v_offset + ((y/2) * info->v_stride) + (x/2);

		//-- Y block
		for (i = 0; i < y_size; i ++) {
			y_addr = y_addr + (i * info->y_stride);
			// Reset the data in the Y buffer to 0.
			VOLATILE_MEMSET((void*)y_addr, y_val, x_size);
		}
		//-- Cb block
		for (i = 0; i < (y_size/2); i ++) {
			cb_addr = cb_addr + (i * info->u_stride);
			// Reset the data in the U and V buffer to 0x88.
			VOLATILE_MEMSET((void*)cb_addr, cbcr_val, (x_size / 2));
		}
		//-- Cr block
		for (i = 0; i < (y_size/2); i ++) {
			cr_addr = cr_addr + (i * info->v_stride);
			// Reset the data in the U and V buffer to 0x88.
			VOLATILE_MEMSET((void*)cr_addr, cbcr_val, (x_size / 2));
		}
		//flush_dcache_range(y_addr, (fb_info.fb_virt_start + fb_info.yuv_base[buf_num + 1]));
	}	
}

/*----------------------------------------------------------------------------*/
static void blank_yuv_buf_block(struct yuv_info *info, struct area_info *adv_info, unsigned char buf_num)
{
	unsigned long y_addr, cb_addr, cr_addr;
	unsigned long base_addr;
	unsigned long x, y, x_size, y_size;
	unsigned char y_val, cbcr_val;
	unsigned long i, j;

	x = adv_info->co_x & ~(0x01);
	y = adv_info->co_y & ~(0x01);
	x_size = adv_info->res_w & ~(0x01);
	y_size = adv_info->res_h & ~(0x01);	
	y_val = 0x00;
	cbcr_val = 0x80;

	base_addr = fb_info.fb_virt_start + fb_info.yuv_base[buf_num];
	y_addr = base_addr + (y * info->y_stride) + x;
	cb_addr = base_addr + info->u_offset + ((y/2) * info->u_stride) + (x/2);
	cr_addr = base_addr + info->v_offset + ((y/2) * info->v_stride) + (x/2);

	//-- Y block
	for (i = 0; i < y_size; i ++) {
		y_addr = y_addr + (i * info->y_stride);
		// Reset the data in the Y buffer to 0.
		VOLATILE_MEMSET((void*)y_addr, y_val, x_size);
	}
	//-- Cb block
	for (i = 0; i < (y_size/2); i ++) {
		cb_addr = cb_addr + (i * info->u_stride);
		// Reset the data in the U and V buffer to 0x88.
		VOLATILE_MEMSET((void*)cb_addr, cbcr_val, (x_size / 2));
	}
	//-- Cr block
	for (i = 0; i < (y_size/2); i ++) {
		cr_addr = cr_addr + (i * info->v_stride);
		// Reset the data in the U and V buffer to 0x88.
		VOLATILE_MEMSET((void*)cr_addr, cbcr_val, (x_size / 2));
	}
	//flush_dcache_range(y_addr, (fb_info.fb_virt_start + fb_info.yuv_base[buf_num + 1]));
}

/*----------------------------------------------------------------------------*/
static void blank_rgb_buf(struct yuv_info *info, unsigned char buf_num)
{
	unsigned long y_addr;
	unsigned long uv_addr;
	unsigned long rgb_size;
	unsigned char y_val, cbcr_val;

	y_val = 0x00;
	cbcr_val = 0x80;
	rgb_size = fb_info.rgb_size / CONFIG_ARCPGU_RGBBUFS;

	y_addr = (fb_info.fb_virt_start) + (buf_num * rgb_size);
	uv_addr = (fb_info.fb_virt_start) + (buf_num * rgb_size) + info->u_offset;
	//FB_MSG("\nY: %08x, UV: %08x\n", y_addr, uv_addr);

	// Reset the data in the Y buffer to 0.
	VOLATILE_MEMSET((void*)y_addr, y_val, info->u_offset);
	// Reset the data in the U and V buffer to 0x88.
	VOLATILE_MEMSET((void*)uv_addr, cbcr_val, (info->u_offset / 2));

	flush_dcache_range(y_addr, (y_addr + rgb_size));
}

/*----------------------------------------------------------------------------*/
static void set_ypbpr_tve_regs(void)
{
	ypbpr_ptr->y_addr = pgu_ptr->frm_start;
	ypbpr_ptr->cb_addr = pgu_ptr->cru_frm_st + ypbpr_ptr->y_addr;
	ypbpr_ptr->cr_addr = pgu_ptr->crv_frm_st + ypbpr_ptr->y_addr;
	ypbpr_ptr->ycbcr_stride = ((pgu_ptr->stride<<16)|(pgu_ptr->cru_stride));
	ypbpr_ptr->rgb_addr = pgu_ptr->ol_frm_st;
	ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
	ypbpr_ptr->rgb_stride |= (pgu_ptr->ol_stride);
	ypbpr_ptr->rgb_range = pgu_ptr->ol_scn_line;

}

/*----------------------------------------------------------------------------*/
static void set_tv_output_interface(void)
{
	unsigned long tmp_long = 0;
	unsigned long reg_val;

	reg_val = ypbpr_ptr->disp_ctrl;
	tmp_long = reg_val & (CVBS_YPBPR_MASK);
	reg_val &= ~(CVBS_YPBPR_MASK);
#ifdef CONFIG_ARCPGU_TV_OUT_YPBPR //wschen 2007-06-06
	tv_interface = SKY_TV_YPBPR;
	if(tmp_long == YPBPR_EN){
		return;
	}
	reg_val |= YPBPR_EN;
	ypbpr_ptr->disp_ctrl = reg_val;
#else
	if( (display_index == DISP_HDTV) || (tv_interface == SKY_TV_YPBPR) ){
		tv_interface = SKY_TV_YPBPR;
		if(tmp_long == YPBPR_EN){
			return;
		}
		//-- YPbPr output
		reg_val |= YPBPR_EN;
		ypbpr_ptr->disp_ctrl = reg_val;
	}
	else if(tv_interface == SKY_TV_CVBS_SVIDEO){
		if(tmp_long == CVBS_EN){
			return;
		}
		//-- CVBS/S-Video output
		reg_val |= CVBS_EN;
		ypbpr_ptr->disp_ctrl = reg_val;
		#if 0
		tmp_long = tv_ptr->output_ctrl;
		while ( ((tmp_long & TVE_OUTPUT_REG_VOF_MASK) != VOF_CVBS_SVIDEO) ){
			tv_ptr->output_ctrl &= ~(TVE_OUTPUT_REG_VOF_MASK);
			tmp_long = tv_ptr->output_ctrl;
		}
		#endif
	}
	else{

	}
#endif
}

/*----------------------------------------------------------------------------*/
/*
 *		Set center.
 */
static void set_center(void)
{
	unsigned long blk_stripe;
	unsigned long black_val;

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV)){

		/* Setup black stripe.*/
		/* check frame dimensions(width). */
		if (p_current_display->xres > fb_info.yuv.width){
			blk_stripe = (p_current_display->xres - fb_info.yuv.width) >> 1;
			black_val = ((blk_stripe << 16) | (blk_stripe));
		}
		else{
			black_val = 0;
		}
		ypbpr_ptr->HBlank = black_val;
		if(ypbpr_ptr->HBlank != black_val){
			FB_MSG("\nBlackH error\n");
		}
		/* check frame dimensions(height). */
		if (p_current_display->yres > fb_info.yuv.height){
			blk_stripe = (p_current_display->yres - fb_info.yuv.height) >> 1;
			//--Set vertical black stride be multiple of 2.
			if( (blk_stripe%2) == 1 ){
				blk_stripe += 1;
			}
			black_val = ((blk_stripe << 16) | (blk_stripe));
		}
		else{
			black_val = 0;
		}
		ypbpr_ptr->VBlank = black_val;
		if(ypbpr_ptr->VBlank != black_val){
			FB_MSG("\nBlackV error\n");
		}
	}
	else{
		pgu_ptr->centreoffsets = ((p_current_display->xres - fb_info.yuv.width) >> 1)
				| (((p_current_display->yres - fb_info.yuv.height) >> 1) << 16);
	}
}


/*----------------------------------------------------------------------------*/
/*
 *	Turn on the signal output from specified module.
 */
static void start_arcpgu(void)
{
//	unsigned long blk_stripe;
//	unsigned long dim_w, dim_h;

	//FB_MSG("\nStart PGU\n");
	/* TV output */
	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV)){
		/* Enable TV module. */
		//--Flowbee 2007-07-31
		ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_MODULE_EN_MASK;

		//-- Turn on video DAC. (1: power off, 0: power on)
		VDAC_CTRL_REG &= ~(VDAC_EN_MASK);
	}
	else{
		/* Enable LCD output */
		pgu_ptr->statctrl |= STATCTRL_REG_DISP_EN_MASK;
		wait_jiffies(3);
		#if CONFIG_TERAWINS_TCON
		//-- turn on the terawins, and wait LCD module output signal.
		tcon_on_off(1);
		#else
		backlight_on_off(1);
    	#endif
	}
}


/*----------------------------------------------------------------------------*/
static void stop_arcpgu(void)
{
	unsigned long flg0 = 1, flg1 = 1;

//	FB_MSG("\nStop PGU");

	val = 0xF0000000;
	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV)){

		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = ypbpr_ptr->disp_ctrl;
			flg1 = (flg0 & YPBPR_CTRL_REG_FRAME_PTR_MASK);
			flg0 = (flg0 & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg0 == 0) || (flg1 == 0) ){
				break;
			}
		}
		//-- Turn off video DAC. (1: power off, 0: power on)
		VDAC_CTRL_REG |= (VDAC_EN_MASK);

		/* Disable the YPbPr and TV encoder. */
		ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_MODULE_EN_MASK);
	}
	else{
		#if CONFIG_TERAWINS_TCON
		//-- turn off the terawins
		tcon_on_off(0);
		#else
		backlight_on_off(0);
		#endif
		//-- Use the LCD interrupt.
		if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
			/* Enable the interrupt and clear the interrupt flag. */
			pgu_ptr->statctrl |= STATCTRL_INTR_EN_MASK;
			pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
			while((pgu_ptr->statctrl & STATCTRL_INTR_FLAG) == 0){
				;
			}
			//-- disable the LCD module.
			pgu_ptr->statctrl &= ~STATCTRL_REG_DISP_EN_MASK;
			//-- disable the LCD interrupt.
			pgu_ptr->statctrl &= ~(STATCTRL_INTR_EN_MASK);
		}
		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = pgu_ptr->statctrl;
			flg1 = (flg0 & STATCTRL_DISP_BUSY_MASK);
			flg0 = (flg0 & STATCTRL_BU_BUSY_MASK);
			if ( (flg0 == 0) && (flg1 == 0) ){
				break;
			}
		}
	}
}

/*----------------------------------------------------------------------------*/
/*
 *	stop_display()
 *		Turn off LCD module
 *		Check TV blank
 */
static void stop_display(void)
{
	unsigned long flg0 = 1, flg1 = 1;

	FB_MSG("\nstop_display\n");

	val = 0xF0000000;
	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV)){

		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = ypbpr_ptr->disp_ctrl;
			flg0 = (flg0 & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg0 == 0) ){
				break;
			}
		}
	}
	else{
		//-- Flowbee 2008-07-10 Check video resolution.
		if( (the_same_resolution == 1) && (do_mplayer_uninit == 0) ){
			return;
		}
		#if CONFIG_TERAWINS_TCON
		//-- turn off the terawins
		tcon_on_off(0);
		#else
		backlight_on_off(0);
		#endif
		//-- Use the LCD interrupt.
		if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
			/* Enable the interrupt and clear the interrupt flag. */
			pgu_ptr->statctrl |= STATCTRL_INTR_EN_MASK;
			pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
			while((pgu_ptr->statctrl & STATCTRL_INTR_FLAG) == 0){
				;
			}
			//-- disable the LCD module.
			pgu_ptr->statctrl &= ~STATCTRL_REG_DISP_EN_MASK;
			//-- disable the LCD interrupt.
			pgu_ptr->statctrl &= ~(STATCTRL_INTR_EN_MASK);
		}
		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = pgu_ptr->statctrl;
			flg1 = (flg0 & STATCTRL_DISP_BUSY_MASK);
			flg0 = (flg0 & STATCTRL_BU_BUSY_MASK);
			if ( (flg0 == 0) && (flg1 == 0) ){
				break;
			}
		}
	}
}

/*----------------------------------------------------------------------------*/
/*
 *	start_display()
 *		Turn on LCD module
 *		Wait 5 jiffies in TV mode
 */
static void start_display(void)
{
	FB_MSG("\nstart_display");

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV)){
		wait_jiffies(5);
	}
	else{
		/* Enable LCD output */
		pgu_ptr->statctrl |= STATCTRL_REG_DISP_EN_MASK;
		//-- turn on the terawins, and wait LCD module output signal.
		wait_jiffies(3);
		#if CONFIG_TERAWINS_TCON
		tcon_on_off(1);
		#else
		backlight_on_off(1);
		#endif
	}
}

/*----------------------------------------------------------------------------*/
/*
 *	Scalar function.
 *		Scale up	: x must be multiples of 16
 *		Scale down	: x must be multiples of 8
 */
#define SCALE_SHIFT				16
void set_scalar_regs(sky_scalar *p)
{
	/* Set the scalar register. */
	//-- reset scalar
	scalar_ptr->ctrl = 0xC0000040;
	scalar_ptr->ctrl = 0xC0000040;
	//-- set width and height.
	scalar_ptr->src_res = p->src_res;
	scalar_ptr->dest_res = p->dest_res;

	//-- set blank and div.
	scalar_ptr->blank = p->blank;
	scalar_ptr->div = p->div;

	//-- set address.
	scalar_ptr->src_addr = p->src_addr;
	scalar_ptr->dest_addr = p->dest_addr;

	//-- start scaling.
	scalar_ptr->ctrl = p->ctrl;
	scalar_ptr->ctrl = p->ctrl;

}

/*----------------------------------------------------------------------------*/
void set_adv_scalar_info(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	unsigned short in_x, in_y;
	unsigned short out_x, out_y;
	unsigned short disp_x, disp_y;
	unsigned short div_x, div_y;
	unsigned short blank_x, blank_y;
	unsigned short tmp_x, tmp_y;
	unsigned short adv_x, adv_y;
	unsigned short adv_sx, adv_sy;

	adv_x = p_current_display->xres;
	adv_y = p_current_display->yres;
	adv_sx = adver_video_info.res_w;
	adv_sy = adver_video_info.res_h;
	
	/* Flowbee@960525-TeraWin T_CON check. */
	tmp_x = adv_sx;
	tmp_y = adv_sy;	

	/* Calculate the scalar information. */
	in_x = p->info.dec_w;
	in_y = p->info.dec_rh;
	out_x = tmp_x;
	out_y = tmp_y;

	/* Calculate the output x and output y. */
	//-- We assume that input width is always the multiples of 16.
	//-- And the output width is always the multiples of 16.
	//-- case 1:  out_x > (2*in_x)
	if(out_x >= (2*in_x)){
		disp_x = 2*in_x;
		disp_y = (disp_x * in_y) / in_x;
		if(disp_y > out_y){
			disp_y = out_y;
			disp_x = (disp_y * in_x) / in_y;
		}
	}
	//-- case 2:  in_x < (out_x/2)
	else if(out_x <= (in_x/2)){
		disp_x = in_x/2;
		disp_y = (disp_x * in_y) / in_x;
	}
	//-- case 3:  (out_x/2) < in_x < (2*out_x)
	else{
		disp_x = out_x;
		disp_y = (disp_x * in_y) / in_x;
		if(disp_y > out_y){
			disp_y = out_y;
			disp_x = (disp_y * in_x) / in_y;
		}
		else if(disp_y < (in_y/2)){
			disp_y = in_y / 2;
			disp_x = (disp_y * in_x) / in_y;
		}
	}
//	FB_MSG("\ndisp: x=%d, y=%d", disp_x, disp_y);
	/* Redefine outx and outy.(the buffer size.) */
	disp_x = (disp_x + 0x1f) & (~0x1f);	// multiples of 32
	disp_y = (disp_y + 0x01) & (~0x01);	// multiples of 2

	//-- set scalar buffer width and height.
	p->info.scale_w = adv_x;
	p->info.scale_h = adv_y;

	//-- Flowbee test the display lines
//	disp_y -= 2;

	//-- calculate real scale width and height.
	//-- (disp_x / dec_w) = (disp_rx / dec_rw)
	//-- disp_rx = (disp_x * dec_rw) / dec_w;
	p->info.scale_rw = (disp_x * p->info.dec_rw) / p->info.dec_w;
	p->info.scale_rh = (disp_y * p->info.dec_rh) / p->info.dec_rh;

	if(p->info.scale_rw > p_current_display->xres){
		p->info.scale_rw = p_current_display->xres;
	}
//	if(p->info.dec_rh != p->info.dec_h){
//		p->info.scale_rh -= 2;
//	}

	//-- Scale down
	if(in_y > disp_y){
		FB_MSG("\n--- Reduce lines ---\n");
		p->info.scale_rh -= REDUCE_SCALE_DOWN_LINES;
	}

	//-- Set multiples of 8.
	p->info.scale_rw = p->info.scale_rw & (~0x000f);

	/* Calculate the blank x and blank y. */
	blank_x = (p->info.scale_w - p->info.scale_rw) >> 1;
	blank_y = (adv_sy - p->info.scale_rh) >> 1;

	/* Calculate the div x and div y. */
	div_x = (1 << SCALE_SHIFT) / disp_x;
	div_y = (1 << SCALE_SHIFT) / disp_y;

	/* Set scalar info structure. */
	//-- Y
	p->yuv[SCALAR_Y].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_Y].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_Y].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_Y].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_Y].ctrl		= 0xC0000001;

//	FB_MSG("\n\n-- Y\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_Y].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_Y].dest_res);
	FB_MSG("blank: %08x\n", p->yuv[SCALAR_Y].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_Y].div);

	in_x  = in_x >> 1;
	in_y  = in_y >> 1;
	out_x = out_x >> 1;
	out_y = out_y >> 1;
	disp_x = disp_x >> 1;
	disp_y = disp_y >> 1;

	/* Calculate the blank x and blank y. */
	blank_x = (blank_x) >> 1;
	blank_y = (blank_y) >> 1;

	/* Calculate the div x and div y. */
	div_x = (1 << SCALE_SHIFT) / disp_x;
	div_y = (1 << SCALE_SHIFT) / disp_y;

	//-- U
	p->yuv[SCALAR_U].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_U].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_U].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_U].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_U].ctrl		= 0xC0000005;

//	FB_MSG("\n-- U\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_U].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_U].dest_res);
	FB_MSG("blank: %08x\n", p->yuv[SCALAR_U].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_U].div);

	//-- V
	p->yuv[SCALAR_V].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_V].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_V].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_V].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_V].ctrl		= 0xC0000005;

//	FB_MSG("\n-- V\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_V].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_V].dest_res);
//	FB_MSG("blank: %08x\n", p->yuv[SCALAR_V].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_V].div);

	return;
}

/*----------------------------------------------------------------------------*/
void set_scalar_info(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	unsigned short in_x, in_y;
	unsigned short out_x, out_y;
	unsigned short disp_x, disp_y;
	unsigned short div_x, div_y;
	unsigned short blank_x, blank_y;
	unsigned short tmp_x, tmp_y;

	/* Flowbee@960525-TeraWin T_CON check. */
#if CONFIG_TERAWINS_TCON
	tmp_x = 640; 
	tmp_y = 480;
	tcon_parse_resolution((unsigned long)tmp_x, (unsigned long)tmp_y);
#else
	tmp_x = p_current_display->xres - REDUCE_SIZE_AT_TV_VIDEO;
	tmp_y = p_current_display->yres - REDUCE_SIZE_AT_TV_VIDEO;
#endif

	/* Calculate the scalar information. */
	in_x = p->info.dec_w;
	in_y = p->info.dec_h;
	out_x = tmp_x;
	out_y = tmp_y;

	/* Calculate the output x and output y. */
	//-- We assume that input width is always the multiples of 16.
	//-- And the output width is always the multiples of 16.
	//-- case 1:  out_x > (2*in_x)
	if(out_x >= (2*in_x)){
		disp_x = 2*in_x;
		disp_y = (disp_x * in_y) / in_x;
		if(disp_y > out_y){
			disp_y = out_y;
			disp_x = (disp_y * in_x) / in_y;
		}
	}
	//-- case 2:  in_x < (out_x/2)
	else if(out_x <= (in_x/2)){
		disp_x = in_x/2;
		disp_y = (disp_x * in_y) / in_x;
	}
	//-- case 3:  (out_x/2) < in_x < (2*out_x)
	else{
		disp_x = out_x;
		disp_y = (disp_x * in_y) / in_x;
		if(disp_y > out_y){
			disp_y = out_y;
			disp_x = (disp_y * in_x) / in_y;
		}
		else if(disp_y < (in_y/2)){
			disp_y = in_y / 2;
			disp_x = (disp_y * in_x) / in_y;
		}
	}
//	FB_MSG("\ndisp: x=%d, y=%d", disp_x, disp_y);
	/* Redefine outx and outy.(the buffer size.) */
	disp_x = (disp_x + 0x1f) & (~0x1f);	// multiples of 32
	disp_y = (disp_y + 0x01) & (~0x01);	// multiples of 2

	//-- set scalar buffer width and height.
	p->info.scale_w = disp_x;
	p->info.scale_h = disp_y;

	//-- Flowbee test the display lines
	disp_y -= 2;

	//-- calculate real scale width and height.
	//-- (disp_x / dec_w) = (disp_rx / dec_rw)
	//-- disp_rx = (disp_x * dec_rw) / dec_w;
	p->info.scale_rw = (disp_x * p->info.dec_rw) / p->info.dec_w;
	p->info.scale_rh = (disp_y * p->info.dec_rh) / p->info.dec_h;

	if(p->info.scale_rw > p_current_display->xres){
		p->info.scale_rw = p_current_display->xres;
	}
	if(p->info.dec_rh != p->info.dec_h){
		p->info.scale_rh -= 2;
	}

	//-- Scale down
	if(in_y > disp_y){
		FB_MSG("\n--- Reduce lines ---\n");
		p->info.scale_rh -= REDUCE_SCALE_DOWN_LINES;
	}

	//-- Set multiples of 8.
	p->info.scale_rw = p->info.scale_rw & (~0x000f);

	/* Calculate the blank x and blank y. */
	blank_x = (disp_x - disp_x) >> 1;
	blank_y = (disp_y - disp_y) >> 1;

	/* Calculate the div x and div y. */
	div_x = (1 << SCALE_SHIFT) / disp_x;
	div_y = (1 << SCALE_SHIFT) / disp_y;

	/* Set scalar info structure. */
	//-- Y
	p->yuv[SCALAR_Y].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_Y].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_Y].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_Y].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_Y].ctrl		= 0xC0000001;

//	FB_MSG("\n\n-- Y\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_Y].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_Y].dest_res);
//	FB_MSG("blank: %08x\n", p->yuv[SCALAR_Y].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_Y].div);

	in_x  = in_x >> 1;
	in_y  = in_y >> 1;
	out_x = out_x >> 1;
	out_y = out_y >> 1;
	disp_x = disp_x >> 1;
	disp_y = disp_y >> 1;

	/* Calculate the blank x and blank y. */
	blank_x = (disp_x - disp_x) >> 1;
	blank_y = (disp_y - disp_y) >> 1;

	/* Calculate the div x and div y. */
	div_x = (1 << SCALE_SHIFT) / disp_x;
	div_y = (1 << SCALE_SHIFT) / disp_y;

	//-- U
	p->yuv[SCALAR_U].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_U].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_U].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_U].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_U].ctrl		= 0xC0000005;

//	FB_MSG("\n-- U\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_U].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_U].dest_res);
//	FB_MSG("blank: %08x\n", p->yuv[SCALAR_U].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_U].div);

	//-- V
	p->yuv[SCALAR_V].src_res 	= (in_x << 16) | (in_y);
	p->yuv[SCALAR_V].dest_res 	= (disp_x << 16) | (disp_y);
	p->yuv[SCALAR_V].blank 		= (blank_x << 16) | (blank_y);
	p->yuv[SCALAR_V].div 		= (div_x << 16) | (div_y);
	p->yuv[SCALAR_V].ctrl		= 0xC0000005;

//	FB_MSG("\n-- V\n");
//	FB_MSG("src_res: %08x\n", p->yuv[SCALAR_V].src_res);
//	FB_MSG("dest_res: %08x\n", p->yuv[SCALAR_V].dest_res);
//	FB_MSG("blank: %08x\n", p->yuv[SCALAR_V].blank);
//	FB_MSG("div: %08x\n", p->yuv[SCALAR_V].div);

	return;
}

/*----------------------------------------------------------------------------*/
int define_adv_scalar_info(void)
{
	unsigned long i;
	int tmpval;

	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 1;
	//-- Flowbee 2008-05-14 mplayer in the init state
	is_video_play = 0;
	//-- Flowbee 2007-11-19	Clear RGB1 and switch to RGB1
	if (adver_video_info.clr_buf == 1) {
		blank_rgb_buf(&(fb_info.yuv), 1);
		stop_display();
		set_rgb_buf(1);
	}	
	dec_yuv.width = tmpyuv.width;
	dec_yuv.height = tmpyuv.height;
	/* Use tmpyuv.num_buffers to define the scalar buf nums. */
	//scalar_num_buffers = tmpyuv.num_buffers;
	FB_MSG("Dec: w=%ld, h=%ld, bufnum=%d", dec_yuv.width, dec_yuv.height, scalar_num_buffers);
	//-- modified to multiples of 16.
	if (dec_yuv.width & 0xf){
		dec_yuv.width = ((dec_yuv.width + 0xf) & ~0xf);
	}
	if (dec_yuv.height & 0xf){
		dec_yuv.height = ((dec_yuv.height + 0xf) & ~0xf);
	}
	//-- save the real width and height.
	scalar.info.dec_rw = tmpyuv.width;	// real video width
	scalar.info.dec_rh = tmpyuv.height;	// real video height
	scalar.info.dec_w = dec_yuv.width;	// decode buffer width
	scalar.info.dec_h = dec_yuv.height;	// decode buffer height
	FB_MSG("\n *** SCALAR INFO ***\n");
	//-- according to the dec buf resolution to set the scalar information.
	set_adv_scalar_info(&scalar);

	/* Allocate scalar buffer. */
	fb_info.yuv.width = scalar.info.scale_w;
	fb_info.yuv.height = scalar.info.scale_h;
	FB_MSG("\n Buf x:%ld, y:%ld", fb_info.yuv.width, fb_info.yuv.height);
	if (calculate_yuv_offsets(&(fb_info.yuv))){
		return -EFAULT;
	}
	if (adver_video_info.clr_buf == 1) {
		// blank YUV
		blank_yuv(&(fb_info.yuv));
	}
	update_advertisment_frame_cnt = 0;
	update_advertisment_frame = 0;
	for(tmpval = 0; tmpval < scalar_num_buffers; tmpval ++){
		frame_cnt[tmpval] = 0;
	}
	// Reserve YUV buf 0 for jpeg temp buffer
	tmpval = scale_buf_alloc();
	FB_MSG("\nReserve buf: %d\n", tmpval);
	
	//-- Check display dimension.
	if(arc_pgu_check_dims(&(fb_info.yuv)) == -1){
		// Switch to yuv 0
		fb_info.yuv.displayed_buffer = 0;
		set_yuv_buf(0);
		/* Flowbee@960518. */
		is_video_scale = 1;		// video using scalar function.
		//-- Flowbee 2008-05-14 mplayer start
		is_video_play = 1;
		start_display();	
		FB_MSG("\n The same resolution. \n");
		return 0;
	}
	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 0;
	if (adver_video_info.clr_buf != 1) {
		blank_rgb_buf(&(fb_info.yuv), 1);
		stop_display();
		set_rgb_buf(1);
		blank_yuv(&(fb_info.yuv));		
	}	
	stop_display();
	// Switch to yuv 0
	fb_info.yuv.displayed_buffer = 0;
	set_yuv_buf(0);
	arc_pgu_setdims(&(fb_info.yuv));
	//-- Flowbee 2007-11-19 Set center.
	set_center();

	/* Flowbee set overlay stride. + */
	ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
	ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
	pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);
	start_display();
	is_video_scale = 1;		// video using scalar function.
	//-- Flowbee 2008-05-14 mplayer start
	is_video_play = 1;
	return 0;
}

/*----------------------------------------------------------------------------*/
int define_scalar_info(void)
{
	unsigned long i;

	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 1;
	//-- Flowbee 2008-05-14 mplayer in the init state
	is_video_play = 0;
	//-- Flowbee 2007-11-19	Clear RGB1 and switch to RGB1
	blank_rgb_buf(&(fb_info.yuv), 1);
	stop_display();
	set_rgb_buf(1);

	dec_yuv.width = tmpyuv.width;
	dec_yuv.height = tmpyuv.height;
	/* Use tmpyuv.num_buffers to define the scalar buf nums. */
	//scalar_num_buffers = tmpyuv.num_buffers;
	FB_MSG("Dec: w=%ld, h=%ld, bufnum=%d", dec_yuv.width, dec_yuv.height, scalar_num_buffers);
	//-- modified to multiples of 16.
	if (dec_yuv.width & 0xf){
		dec_yuv.width = ((dec_yuv.width + 0xf) & ~0xf);
	}
	if (dec_yuv.height & 0xf){
		dec_yuv.height = ((dec_yuv.height + 0xf) & ~0xf);
	}
	//-- save the real width and height.
	scalar.info.dec_rw = tmpyuv.width;	// real video width
	scalar.info.dec_rh = tmpyuv.height;	// real video height
	scalar.info.dec_w = dec_yuv.width;	// decode buffer width
	scalar.info.dec_h = dec_yuv.height;	// decode buffer height
	FB_MSG("\n *** SCALAR INFO ***\n");
	//-- according to the dec buf resolution to set the scalar information.
	set_scalar_info(&scalar);

	/* Allocate scalar buffer. */
	fb_info.yuv.width = scalar.info.scale_w;
	fb_info.yuv.height = scalar.info.scale_h;
	FB_MSG("\n Buf x:%ld, y:%ld", fb_info.yuv.width, fb_info.yuv.height);
	if (calculate_yuv_offsets(&(fb_info.yuv))){
		return -EFAULT;
	}
	// blank YUV 0
//	blank_yuv_buf(&(fb_info.yuv), 0);
	blank_yuv(&(fb_info.yuv));
	fb_info.yuv.width = scalar.info.scale_rw;
	fb_info.yuv.height = scalar.info.scale_rh;
	FB_MSG("\n Real x:%ld, y:%ld", fb_info.yuv.width, fb_info.yuv.height);

	//-- Check display dimension.
	if(arc_pgu_check_dims(&(fb_info.yuv)) == -1){
		// Switch to yuv 0
		fb_info.yuv.displayed_buffer = 0;
		set_yuv_buf(0);
		/* Flowbee@960518. */
		is_video_scale = 1;		// video using scalar function.
		//-- Flowbee 2008-05-14 mplayer start
		is_video_play = 1;
		start_display();	
		FB_MSG("\n The same resolution. \n");
		return 0;
	}
	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 0;

	//down(&pgu_sem);
	stop_display();
	// Switch to yuv 0
	fb_info.yuv.displayed_buffer = 0;
	set_yuv_buf(0);
	arc_pgu_setdims(&(fb_info.yuv));
	//-- Flowbee 2007-11-19 Set center.
	set_center();

	/* Flowbee set overlay stride. + */
	ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
	ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
	pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);

	//-- Disable OSD
//	turn_all_osd_off();	// Flowbee 2008-02-21
	start_display();
	//-- Flowbee 2008-02-29 Clear yuv buffer to avoid scale down error.
	//up(&pgu_sem);

	//-- Flowbee 2007-11-19
	//blank_rgb_buf(&(fb_info.yuv), 1);
	//set_rgb_buf(1);

	/* Flowbee@960518. */
	is_video_scale = 1;		// video using scalar function.
	//-- Flowbee 2008-05-14 mplayer start
	is_video_play = 1;
	return 0;
}

/*----------------------------------------------------------------------------*/
void init_advertisment_info(void)
{
	//-- Video
	adver_video_info.co_x = 0;
	adver_video_info.co_y = 0;
	adver_video_info.res_w = 0;
	adver_video_info.res_w = 0;

	//-- JPEG (text)
	adver_jpeg_info.src_co_x = 0;
	adver_jpeg_info.src_co_y = 0;
	adver_jpeg_info.dst_co_x = 0;
	adver_jpeg_info.dst_co_y = 0;
	adver_jpeg_info.res_w = 0;
	adver_jpeg_info.res_h = 0;
}

/*----------------------------------------------------------------------------*/
int define_advertisment_info(void)
{
	int tmpval;

	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 1;
	//-- Flowbee 2008-05-14 mplayer in the init state
	is_video_play = 0;
	//-- Flowbee 2007-11-19	Clear RGB1 and switch to RGB1
	if (adver_video_info.clr_buf == 1) {
	blank_rgb_buf(&(fb_info.yuv), 1);
	stop_display();
	set_rgb_buf(1);
	}
	else {
		//fb_info.yuv.displayed_buffer = 0;
		//set_yuv_buf(0);
	}
	//-- Get the real video resolution.
	dec_yuv.width = tmpyuv.width;
	dec_yuv.height = tmpyuv.height;
	//-- modified to multiples of 16.
	if (dec_yuv.width & 0xf){
		dec_yuv.width = ((dec_yuv.width + 0xf) & ~0xf);
	}
	if (dec_yuv.height & 0xf){
		dec_yuv.height = ((dec_yuv.height + 0xf) & ~0xf);
	}
	//-- save the real width and height.
	scalar.info.dec_rw = (tmpyuv.width & ~(0x01));	// real video width
	scalar.info.dec_rh = (tmpyuv.height & ~(0x01));	// real video height
	scalar.info.dec_w = dec_yuv.width;	// decode buffer width
	scalar.info.dec_h = dec_yuv.height;	// decode buffer height	
	FB_MSG("adv Dec: w=%ld, h=%ld", scalar.info.dec_rw, scalar.info.dec_rh);	

	fb_info.yuv.width = p_current_display->xres;
	fb_info.yuv.height = p_current_display->yres;
	if (fb_info.yuv.width & 0xf){
		fb_info.yuv.width = ((fb_info.yuv.width + 0xf) & ~0xf);
	}
	/* Flowbee@960427. */
	if (fb_info.yuv.height & 0xf){
		fb_info.yuv.height = ((fb_info.yuv.height + 0xf) & ~0xf);
		if(fb_info.yuv.height > p_current_display->yres){
			fb_info.yuv.height = p_current_display->yres;
		}
	}
	/* Flowbee@960525-TeraWin T_CON check. */
	#if CONFIG_TERAWINS_TCON
	tcon_parse_resolution(fb_info.yuv.width, fb_info.yuv.height);
	#endif
	FB_MSG("\nadv BUF: x=%ld, y=%ld\n", fb_info.yuv.width, fb_info.yuv.height);
	if (calculate_yuv_offsets(&(fb_info.yuv))){
		return -EFAULT;
	}
	if (adver_video_info.clr_buf == 1) {
	// blank YUV
		blank_yuv(&(fb_info.yuv));
	}
	update_advertisment_frame_cnt = 0;
	update_advertisment_frame = 0;
	for(tmpval = 0; tmpval < scalar_num_buffers; tmpval ++){
		frame_cnt[tmpval] = 0;
	}
	// Reserve YUV buf 0 for jpeg temp buffer
	tmpval = scale_buf_alloc();
	FB_MSG("\nReserve buf: %d\n", tmpval);

	//-- Check display dimension.
	if(arc_pgu_check_dims(&(fb_info.yuv)) == -1){
		FB_MSG("\n The same resolution. \n");
		// Switch to yuv 0
		//fb_info.yuv.displayed_buffer = 0;
		//set_yuv_buf(0);
		is_video_play = 1;
		if (adver_video_info.clr_buf == 1) {
			start_display();
		}
		return 0;
	}
	//-- Flowbee 2008-07-10 Check video resolution
	the_same_resolution = 0;

	if (adver_video_info.clr_buf != 1) {
		blank_rgb_buf(&(fb_info.yuv), 1);
		stop_display();
		set_rgb_buf(1);
		blank_yuv(&(fb_info.yuv));		
	}
	stop_display();
	// Switch to yuv 0
	fb_info.yuv.displayed_buffer = 0;
	set_yuv_buf(0);
	arc_pgu_setdims(&(fb_info.yuv));
	set_center();
	ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
	ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
	pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);
	is_video_play = 1;
	start_display();
	return 0;
}

//static void blank_yuv(struct yuv_info *info)
/*----------------------------------------------------------------------------*/
//== SIMD DMA dimension needs multiples of 16 bytes.
void move_buf_data_simd(unsigned char src, unsigned char dest)
{
	struct _scalar_info si;
	struct _scalar_info *p = &si;
	int dma_block_x, dma_block_y, UVdma_block_y;
	int id_x, id_y;
	unsigned long y_addr, u_addr, v_addr;
	unsigned long y_offset_x, uv_offset_x, y_offset_y, uv_offset_y;
	unsigned char i;
	unsigned long backup_addr;

	//-- Set backup address
	backup_addr = (unsigned long)simd_backup_buf;
	//-- SIMD SRAM backup to backup address
	dma_out_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_out_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);

	//-- Setting decode and scale buffer address.
	p->yuv[SCALAR_Y].dest_addr = fb_info.fb_virt_start + fb_info.yuv_base[dest];
	p->yuv[SCALAR_U].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.u_offset;
	p->yuv[SCALAR_V].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.v_offset;

	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	//== JPEG data in YUV 0 for test
	y_addr = fb_info.fb_virt_start + fb_info.yuv_base[src];
	u_addr = y_addr + fb_info.yuv.u_offset;
	v_addr = y_addr + fb_info.yuv.v_offset;

	dma_block_x = fb_info.yuv.width >> 1;
	dma_block_y = fb_info.yuv.height >> 1;//120;//60;//120;
	if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_x = (fb_info.yuv.width >> 2);
	}
	if( (dma_block_y > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_y = (fb_info.yuv.height >> 2);
	}
	UVdma_block_y = dma_block_y;//90;

	//== JPEG addr
	set_dma_frame(6, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(7, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(8, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	// Update Y
	for (id_y = 0; id_y < (fb_info.yuv.height); id_y += dma_block_y){
		for (id_x = 0; id_x < (fb_info.yuv.width); id_x += dma_block_x){
			dma_in_sdm(6, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 1);
		}
	}
	// Update U V
	for (id_y = 0; id_y < ((fb_info.yuv.height) >> 1); id_y += UVdma_block_y){
		for (id_x = 0; id_x < ((fb_info.yuv.width) >> 1); id_x += dma_block_x){
			dma_in_sdm(7, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_in_sdm(8, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
		}
	}
	//-- Restore SIMD SRAM from backup address
	dma_in_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_in_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);	
}

/*----------------------------------------------------------------------------*/
//== SIMD DMA dimension needs multiples of 16 bytes.
void move_jpeg_simd(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	int dma_block_x, dma_block_y, UVdma_block_y;
	int id_x, id_y;
	unsigned long y_addr, u_addr, v_addr;
	unsigned long y_offset_x, uv_offset_x, y_offset_y, uv_offset_y;
	unsigned char i;
	unsigned long backup_addr;

	//-- Set backup address
	backup_addr = (unsigned long)simd_backup_buf;
	//-- SIMD SRAM backup to backup address
	dma_out_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_out_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);

	//-- Setting decode and scale buffer address.
	p->yuv[SCALAR_Y].dest_addr = fb_info.fb_virt_start + fb_info.yuv_base[p->info.scale_idx];
	p->yuv[SCALAR_U].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.u_offset;
	p->yuv[SCALAR_V].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.v_offset;

	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	//== Update JPEG
	if(update_advertisment_frame == 1){
		//== JPEG data in YUV 0 for test
		y_addr = fb_info.fb_virt_start + fb_info.yuv_base[0];
		u_addr = y_addr + fb_info.yuv.u_offset;
		v_addr = y_addr + fb_info.yuv.v_offset;

		y_offset_x = adver_jpeg_info.dst_co_x;
		uv_offset_x = y_offset_x >> 1;
		y_offset_y = adver_jpeg_info.dst_co_y;
		uv_offset_y = y_offset_y >> 1;

		dma_block_x = adver_jpeg_info.res_w >> 1;
		dma_block_y = adver_jpeg_info.res_h >> 1;//120;//60;//120;
		if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
			dma_block_x = (adver_jpeg_info.res_w >> 2);
		}
		if( (dma_block_y > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
			dma_block_y = (adver_jpeg_info.res_h >> 2);
		}
		UVdma_block_y = dma_block_y;//90;

		//== JPEG addr
		set_dma_frame(6, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
		set_dma_frame(7, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
		set_dma_frame(8, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

		if(frame_cnt[p->info.scale_idx] == 0){
			//FB_MSG("\n");
			for(i = 1; i < scalar_num_buffers; i++){
				//FB_MSG("(%d): %d", i, frame_cnt[i]);
			}
			frame_cnt[p->info.scale_idx] ++;
			update_advertisment_frame_cnt ++;
			//FB_MSG("\n O (%d, %d)", update_advertisment_frame_cnt, p->info.scale_idx);
		}
		else{
			// repeat dec buf, find the non-update buf
			for(i = 1; i < scalar_num_buffers; i++){
				if(frame_cnt[i] == 0){
					// update frame cnt
					frame_cnt[i] ++;
					update_advertisment_frame_cnt ++;
					//-- Set dest addr
					y_addr = fb_info.fb_virt_start + fb_info.yuv_base[i];
					u_addr = y_addr + fb_info.yuv.u_offset;
					v_addr = y_addr + fb_info.yuv.v_offset;
					//-- DST addr
					set_dma_frame(3, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
					set_dma_frame(4, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
					set_dma_frame(5, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

					//FB_MSG("\n X cnt: %d, buf: %d, idx: %d, num: %d", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					//FB_MSG("\n X (%d, %d, %d, %d)", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					break;
				}
			}
		}

		// Update Y
		for (id_y = 0; id_y < (adver_jpeg_info.res_h); id_y += dma_block_y){
			for (id_x = 0; id_x < (adver_jpeg_info.res_w); id_x += dma_block_x){
				dma_in_sdm(6, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + adver_jpeg_info.src_co_x), (id_y + adver_jpeg_info.src_co_y), 0, 1);
				dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
			}
		}
		// Update U V
		for (id_y = 0; id_y < ((adver_jpeg_info.res_h) >> 1); id_y += UVdma_block_y){
			for (id_x = 0; id_x < ((adver_jpeg_info.res_w) >> 1); id_x += dma_block_x){
				dma_in_sdm(7, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
				dma_in_sdm(8, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
			}
		}

		if(update_advertisment_frame_cnt >= (scalar_num_buffers-1)){
			update_advertisment_frame = 0;
	update_advertisment_frame_cnt = 0;
			//FB_MSG("\n");
			for(i = 0; i < scalar_num_buffers; i ++){
				//FB_MSG("%d ", frame_cnt[i]);
				if((frame_cnt[i] == 0) && (i > 0)){
					//FB_MSG("\n frmae cnt wrong: %d \n", i);
				}
				frame_cnt[i] = 0;
			}
			//FB_MSG("\nSTOP\n");
		}
	}
	//-- Restore SIMD SRAM from backup address
	dma_in_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_in_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);	
}

#if 1
/*----------------------------------------------------------------------------*/
//== SIMD DMA dimension needs multiples of 16 bytes.
void move_video_jpeg_simd(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	int dma_block_x, dma_block_y, UVdma_block_y;
	int id_x, id_y;
	unsigned long y_addr, u_addr, v_addr;
	unsigned long y_offset_x, uv_offset_x, y_offset_y, uv_offset_y;
	unsigned char i;
	unsigned long backup_addr;

	//-- Set backup address
	backup_addr = (unsigned long)simd_backup_buf;
	//backup_addr = (unsigned long)rgb1_addr;
	//-- SIMD SRAM backup to backup address
	dma_out_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_out_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);

	//-- Setting decode and scale buffer address.
	p->yuv[SCALAR_Y].src_addr = fb_info.fb_virt_start + dec_yuv_base[p->info.dec_idx];
	p->yuv[SCALAR_U].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.u_offset;
	p->yuv[SCALAR_V].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.v_offset;

	p->yuv[SCALAR_Y].dest_addr = fb_info.fb_virt_start + fb_info.yuv_base[p->info.scale_idx];
	p->yuv[SCALAR_U].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.u_offset;
	p->yuv[SCALAR_V].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.v_offset;

	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	//== Update JPEG
	if(update_advertisment_frame == 1){
		//== JPEG data in YUV 0 for test
		y_addr = fb_info.fb_virt_start + fb_info.yuv_base[0];
		u_addr = y_addr + fb_info.yuv.u_offset;
		v_addr = y_addr + fb_info.yuv.v_offset;

		y_offset_x = adver_jpeg_info.dst_co_x;
		uv_offset_x = y_offset_x >> 1;
		y_offset_y = adver_jpeg_info.dst_co_y;
		uv_offset_y = y_offset_y >> 1;

		dma_block_x = adver_jpeg_info.res_w >> 1;
		dma_block_y = adver_jpeg_info.res_h >> 1;//120;//60;//120;
		if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
			dma_block_x = (adver_jpeg_info.res_w >> 2);
		}
		if( (dma_block_y > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512))){
			dma_block_y = (adver_jpeg_info.res_h >> 2);
		}
		UVdma_block_y = dma_block_y;//90;

		//== JPEG addr
		set_dma_frame(6, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
		set_dma_frame(7, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
		set_dma_frame(8, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

		if(frame_cnt[p->info.scale_idx] == 0){
			//FB_MSG("\n");
			for(i = 1; i < scalar_num_buffers; i++){
				//FB_MSG("(%d): %d", i, frame_cnt[i]);
			}
			frame_cnt[p->info.scale_idx] ++;
			update_advertisment_frame_cnt ++;
			//FB_MSG("\n O (%d, %d)", update_advertisment_frame_cnt, p->info.scale_idx);
		}
		else{
			// repeat dec buf, find the non-update buf
			for(i = 1; i < scalar_num_buffers; i++){
				if(frame_cnt[i] == 0){
					// update frame cnt
					frame_cnt[i] ++;
					update_advertisment_frame_cnt ++;
					//-- Set dest addr
					y_addr = fb_info.fb_virt_start + fb_info.yuv_base[i];
					u_addr = y_addr + fb_info.yuv.u_offset;
					v_addr = y_addr + fb_info.yuv.v_offset;
					//-- DST addr
					set_dma_frame(3, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
					set_dma_frame(4, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
					set_dma_frame(5, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

					//FB_MSG("\n X cnt: %d, buf: %d, idx: %d, num: %d", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					//FB_MSG("\n X (%d, %d, %d, %d)", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					break;
				}
			}
		}

		// Update Y
		for (id_y = 0; id_y < (adver_jpeg_info.res_h); id_y += dma_block_y){
			for (id_x = 0; id_x < (adver_jpeg_info.res_w); id_x += dma_block_x){
				dma_in_sdm(6, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + adver_jpeg_info.src_co_x), (id_y + adver_jpeg_info.src_co_y), 0, 1);
				dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
			}
		}
		// Update U V
		for (id_y = 0; id_y < ((adver_jpeg_info.res_h) >> 1); id_y += UVdma_block_y){
			for (id_x = 0; id_x < ((adver_jpeg_info.res_w) >> 1); id_x += dma_block_x){
				dma_in_sdm(7, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
				dma_in_sdm(8, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
			}
		}

		if(update_advertisment_frame_cnt >= (scalar_num_buffers-1)){
	update_advertisment_frame = 0;
			update_advertisment_frame_cnt = 0;
			//FB_MSG("\n");
			for(i = 0; i < scalar_num_buffers; i ++){
				//FB_MSG("%d ", frame_cnt[i]);
				if((frame_cnt[i] == 0) && (i > 0)){
					//FB_MSG("\n frmae cnt wrong: %d \n", i);
				}
				frame_cnt[i] = 0;
			}
			//FB_MSG("\nSTOP\n");
		}
	}

	//== Update Video
	//-- SRC addr
	set_dma_frame(0, p->yuv[SCALAR_Y].src_addr, scalar.info.dec_w, scalar.info.dec_w, scalar.info.dec_h);
	set_dma_frame(1, p->yuv[SCALAR_U].src_addr, (scalar.info.dec_w >> 1), (scalar.info.dec_w >> 1), (scalar.info.dec_h >> 1));
	set_dma_frame(2, p->yuv[SCALAR_V].src_addr, (scalar.info.dec_w >> 1), (scalar.info.dec_w >> 1), (scalar.info.dec_h >> 1));
	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	y_offset_x = adver_video_info.co_x;
	uv_offset_x = y_offset_x >> 1;
	y_offset_y = adver_video_info.co_y;
	uv_offset_y = y_offset_y >> 1;

	dma_block_x = (scalar.info.dec_rw >> 1);
	dma_block_y = (scalar.info.dec_rh >> 1);//120;//60;//120;
	if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_x = (scalar.info.dec_rw >> 2);
	}
	if( (dma_block_y > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_y = (scalar.info.dec_rh >> 2);
	}
	UVdma_block_y = dma_block_y;//90;
	// Y
	for (id_y = 0; id_y < scalar.info.dec_rh; id_y += dma_block_y){
		for (id_x = 0; id_x < scalar.info.dec_rw; id_x += dma_block_x){
			dma_in_sdm(0, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
}
	}
	// UV
	for (id_y = 0; id_y < (scalar.info.dec_rh >> 1); id_y += UVdma_block_y){
		for (id_x = 0; id_x < (scalar.info.dec_rw >> 1); id_x += dma_block_x){
			dma_in_sdm(1, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);

			dma_in_sdm(2, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
		}
	}
	//-- Restore SIMD SRAM from backup address
	dma_in_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_in_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);
}
#else
/*----------------------------------------------------------------------------*/
//== SIMD DMA dimension needs multiples of 16 bytes.
void move_video_jpeg_simd(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	int dma_block_x, dma_block_y, UVdma_block_y;
	int id_x, id_y;
	unsigned long y_addr, u_addr, v_addr;
	unsigned long y_offset_x, uv_offset_x, y_offset_y, uv_offset_y;
	unsigned char i;
	unsigned long backup_addr;

	//-- Set backup address
	backup_addr = (unsigned long)simd_backup_buf;
	//-- SIMD SRAM backup to backup address
	dma_out_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_out_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);

	//-- Setting decode and scale buffer address.
	p->yuv[SCALAR_Y].src_addr = fb_info.fb_virt_start + dec_yuv_base[p->info.dec_idx];
	p->yuv[SCALAR_U].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.u_offset;
	p->yuv[SCALAR_V].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.v_offset;

	p->yuv[SCALAR_Y].dest_addr = fb_info.fb_virt_start + fb_info.yuv_base[p->info.scale_idx];
	p->yuv[SCALAR_U].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.u_offset;
	p->yuv[SCALAR_V].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.v_offset;

#if 1
	//-- SRC addr
	set_dma_frame(0, p->yuv[SCALAR_Y].src_addr, scalar.info.dec_w, scalar.info.dec_w, scalar.info.dec_h);
	set_dma_frame(1, p->yuv[SCALAR_U].src_addr, (scalar.info.dec_w >> 1), (scalar.info.dec_w >> 1), (scalar.info.dec_h >> 1));
	set_dma_frame(2, p->yuv[SCALAR_V].src_addr, (scalar.info.dec_w >> 1), (scalar.info.dec_w >> 1), (scalar.info.dec_h >> 1));

	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	//== Update Video
	y_offset_x = adver_video_info.co_x;
	uv_offset_x = y_offset_x >> 1;
	y_offset_y = adver_video_info.co_y;
	uv_offset_y = y_offset_y >> 1;

	dma_block_x = (scalar.info.dec_rw >> 1);
	dma_block_y = (scalar.info.dec_rh >> 1);//120;//60;//120;
	if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_x = (scalar.info.dec_rw >> 2);
	}
	if(dma_block_y > 255){
		dma_block_y = (scalar.info.dec_rh >> 2);
	}
	UVdma_block_y = dma_block_y;//90;

	// Y
	for (id_y = 0; id_y < scalar.info.dec_rh; id_y += dma_block_y){
		for (id_x = 0; id_x < scalar.info.dec_rw; id_x += dma_block_x){
			dma_in_sdm(0, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
		}
	}
	// UV
	for (id_y = 0; id_y < (scalar.info.dec_rh >> 1); id_y += UVdma_block_y){
		for (id_x = 0; id_x < (scalar.info.dec_rw >> 1); id_x += dma_block_x){
			dma_in_sdm(1, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);

			dma_in_sdm(2, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
		}
	}
#else
	//-- SRC addr
	set_dma_frame(0, p->yuv[SCALAR_Y].src_addr, dec_yuv.width, dec_yuv.width, dec_yuv.height);
	set_dma_frame(1, p->yuv[SCALAR_U].src_addr, (dec_yuv.width >> 1), (dec_yuv.width >> 1), (dec_yuv.height >> 1));
	set_dma_frame(2, p->yuv[SCALAR_V].src_addr, (dec_yuv.width >> 1), (dec_yuv.width >> 1), (dec_yuv.height >> 1));

	//-- DST addr
	set_dma_frame(3, p->yuv[SCALAR_Y].dest_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
	set_dma_frame(4, p->yuv[SCALAR_U].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
	set_dma_frame(5, p->yuv[SCALAR_V].dest_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

	//== Update Video
	y_offset_x = adver_video_info.co_x;
	uv_offset_x = y_offset_x >> 1;
	y_offset_y = adver_video_info.co_y;
	uv_offset_y = y_offset_y >> 1;

	dma_block_x = (dec_yuv.width >> 1);
	dma_block_y = (dec_yuv.height >> 1);//120;//60;//120;
	if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
		dma_block_x = (dec_yuv.width >> 2);
	}
	if(dma_block_y > 255){
		dma_block_y = (dec_yuv.height >> 2);
	}
	UVdma_block_y = dma_block_y;//90;

	// Y
	for (id_y = 0; id_y < dec_yuv.height; id_y += dma_block_y){
		for (id_x = 0; id_x < dec_yuv.width; id_x += dma_block_x){
			dma_in_sdm(0, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
		}
	}
	// UV
	for (id_y = 0; id_y < (dec_yuv.height >> 1); id_y += UVdma_block_y){
		for (id_x = 0; id_x < (dec_yuv.width >> 1); id_x += dma_block_x){
			dma_in_sdm(1, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);

			dma_in_sdm(2, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 1);
			dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
		}
	}
#endif
	//== Update JPEG
	if(update_advertisment_frame == 1){
		//== JPEG data in YUV 0 for test
		y_addr = fb_info.fb_virt_start + fb_info.yuv_base[0];
		u_addr = y_addr + fb_info.yuv.u_offset;
		v_addr = y_addr + fb_info.yuv.v_offset;

		y_offset_x = adver_jpeg_info.dst_co_x;
		uv_offset_x = y_offset_x >> 1;
		y_offset_y = adver_jpeg_info.dst_co_y;
		uv_offset_y = y_offset_y >> 1;

		dma_block_x = adver_jpeg_info.res_w >> 1;
		dma_block_y = adver_jpeg_info.res_h >> 1;//120;//60;//120;
		if( (dma_block_x > 255) || ((dma_block_x * dma_block_y) > (0x8000 - 512)) ){
			dma_block_x = (adver_jpeg_info.res_w >> 2);
		}
		if(dma_block_y > 255){
			dma_block_y = (adver_jpeg_info.res_h >> 2);
		}
		UVdma_block_y = dma_block_y;//90;

		//== JPEG addr
		set_dma_frame(6, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
		set_dma_frame(7, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
		set_dma_frame(8, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

		if(frame_cnt[p->info.scale_idx] == 0){
			//FB_MSG("\n");
			for(i = 1; i < scalar_num_buffers; i++){
				//FB_MSG("(%d): %d", i, frame_cnt[i]);
			}
			frame_cnt[p->info.scale_idx] ++;
			update_advertisment_frame_cnt ++;
			//FB_MSG("\n O (%d, %d)", update_advertisment_frame_cnt, p->info.scale_idx);
		}
		else{
			// repeat dec buf, find the non-update buf
			for(i = 1; i < scalar_num_buffers; i++){
				if(frame_cnt[i] == 0){
					// update frame cnt
					frame_cnt[i] ++;
					update_advertisment_frame_cnt ++;
					//-- Set dest addr
					y_addr = fb_info.fb_virt_start + fb_info.yuv_base[i];
					u_addr = y_addr + fb_info.yuv.u_offset;
					v_addr = y_addr + fb_info.yuv.v_offset;
					//-- DST addr
					set_dma_frame(3, y_addr, fb_info.yuv.width, fb_info.yuv.width, fb_info.yuv.height);
					set_dma_frame(4, u_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));
					set_dma_frame(5, v_addr, (fb_info.yuv.width >> 1), (fb_info.yuv.width >> 1), (fb_info.yuv.height >> 1));

					//FB_MSG("\n X cnt: %d, buf: %d, idx: %d, num: %d", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					//FB_MSG("\n X (%d, %d, %d, %d)", update_advertisment_frame_cnt, i, p->info.scale_idx, scalar_num_buffers);
					break;
				}
			}
		}

		// Update Y
		for (id_y = 0; id_y < (adver_jpeg_info.res_h); id_y += dma_block_y){
			for (id_x = 0; id_x < (adver_jpeg_info.res_w); id_x += dma_block_x){
				dma_in_sdm(6, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + adver_jpeg_info.src_co_x), (id_y + adver_jpeg_info.src_co_y), 0, 1);
				dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, (id_x + y_offset_x), (id_y + y_offset_y), 0, 1);
			}
		}
		// Update U V
		for (id_y = 0; id_y < ((adver_jpeg_info.res_h) >> 1); id_y += UVdma_block_y){
			for (id_x = 0; id_x < ((adver_jpeg_info.res_w) >> 1); id_x += dma_block_x){
				dma_in_sdm(7, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
				dma_in_sdm(8, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + (adver_jpeg_info.src_co_x >> 1)), (id_y + (adver_jpeg_info.src_co_y >> 1)), 0, 1);
				dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, (id_x + uv_offset_x), (id_y + uv_offset_y), 0, 1);
			}
		}

		if(update_advertisment_frame_cnt >= (scalar_num_buffers-1)){
			update_advertisment_frame = 0;
			update_advertisment_frame_cnt = 0;
			//FB_MSG("\n");
			for(i = 0; i < scalar_num_buffers; i ++){
				//FB_MSG("%d ", frame_cnt[i]);
				if((frame_cnt[i] == 0) && (i > 0)){
					FB_MSG("\n frmae cnt wrong: %d \n", i);
				}
				frame_cnt[i] = 0;
			}
			//FB_MSG("\nSTOP\n");
		}
	}
	//-- Restore SIMD SRAM from backup address
	dma_in_sdm(31, 0, 128, 128, backup_addr, 128, 1, 0, 0, 0, 1);
	dma_in_sdm(31, 128, 128, 128, (backup_addr + 128), 128, 255, 0, 0, 0, 1);
}
#endif

/*----------------------------------------------------------------------------*/
void set_scalar_buf_addr(struct _scalar_info *_info)
{
	struct _scalar_info *p = _info;
	/* Setting decode and scale buffer address. */
	p->yuv[SCALAR_Y].src_addr = fb_info.fb_virt_start + dec_yuv_base[p->info.dec_idx];
	p->yuv[SCALAR_U].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.u_offset;
	p->yuv[SCALAR_V].src_addr = p->yuv[SCALAR_Y].src_addr + dec_yuv.v_offset;

	p->yuv[SCALAR_Y].dest_addr = fb_info.fb_virt_start + fb_info.yuv_base[p->info.scale_idx];
	p->yuv[SCALAR_U].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.u_offset;
	p->yuv[SCALAR_V].dest_addr = p->yuv[SCALAR_Y].dest_addr + fb_info.yuv.v_offset;

//	FB_MSG("\nsrc_Y: %08x\n", p->yuv[SCALAR_Y].src_addr);
//	FB_MSG("src_U: %08x\n", p->yuv[SCALAR_U].src_addr);
//	FB_MSG("src_V: %08x\n", p->yuv[SCALAR_V].src_addr);
//	FB_MSG("dest_Y: %08x\n", p->yuv[SCALAR_Y].dest_addr);
//	FB_MSG("dest_U: %08x\n", p->yuv[SCALAR_U].dest_addr);
//	FB_MSG("dest_V: %08x\n", p->yuv[SCALAR_V].dest_addr);
}

/*----------------------------------------------------------------------------*/
/*
 *	OSD function.
 */
extern void init_alpha_table(uint8 alpha_src);
extern void rgb2yuv_blend(struct yuv_info *info, uint32 rgb_src, uint32 yuv_src, uint32 yuv_dest,  uint32 start_line, uint32 total_line, uint8 all);

/*----------------------------------------------------------------------------*/
unsigned long osd_buf_addr[6];
struct _osd_info osd_info;
unsigned char sw_osd_flag = FB_FALSE;
unsigned char alpha_val = 0;

void init_software_osd(void)
{
	unsigned long tmp;
	/* Init alpha table. */
	alpha_val = (unsigned char)((pgu_ptr->statctrl & STATCTRL_OL_VAL_MASK) >> STATCTRL_OL_VAL_SHIFT);
	init_alpha_table(alpha_val);

	/* Init osd info. */
	osd_info.rgb_src = 0;
	osd_info.yuv_src = 0;
	osd_info.yuv_dest = 0;
	osd_info.yuv_bkg = 0;
	osd_info.yuv_osd = 0;
	osd_info.start_line = 0;
	osd_info.total_line = fb_info.yuv.height;
	osd_info.blend_all = 0;
	osd_info.current_mode = 0;

	/* Init osd buf address. */
	tmp = (p_current_display->xres * p_current_display->yres * (p_current_display->bpp / 8));
	osd_buf_addr[YUV_0] = fb_info.fb_virt_start + fb_info.yuv_base[0];
	osd_buf_addr[YUV_1] = fb_info.fb_virt_start + fb_info.yuv_base[1];
	osd_buf_addr[RGB_0] = fb_info.fb_virt_start;
	osd_buf_addr[RGB_1] = fb_info.fb_virt_start + tmp;
	osd_buf_addr[RGB_2] = END_MEM - CACHED_UNCACHED_MEM_SIZE - tmp;
	osd_buf_addr[B_TMP] = fb_info.fb_virt_start;


	FB_MSG("\nosd_buf_addr[YUV_0]=%08lx, osd_buf_addr[YUV_1]=%08lx, osd_buf_addr[RGB_0]=%08lx, osd_buf_addr[RGB_1]=%08lx, osd_buf_addr[RGB_2]=%08lx\n",
			osd_buf_addr[YUV_0], osd_buf_addr[YUV_1], osd_buf_addr[RGB_0], osd_buf_addr[RGB_1], osd_buf_addr[RGB_2]);

}

/*----------------------------------------------------------------------------*/
/*
 *	Disable all osd functions.
 */
static void turn_all_osd_off(void)
{
	//-- Software OSD for TV and LCD.
	sw_osd_flag = FB_FALSE;
	//-- Hardware OSD for TV and LCD.
	ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_OL_EN_MASK);
	pgu_ptr->statctrl &= ~(STATCTRL_OL_EN_MASK);

	#if CONFIG_TERAWINS_TCON
	tcon_clear_all_osd_item();
	#endif
	return;
}

/*----------------------------------------------------------------------------*/
/*
 *	Update OSD.
 */
void update_osd(void)
{
	unsigned long tmp, flg0, flg1;
	//FB_MSG("\nUPDATE! \n");
	flush_dcache_all();

	/* Alpha blending. */
	rgb2yuv_blend(&(fb_info.yuv), osd_buf_addr[osd_info.rgb_src], osd_buf_addr[osd_info.yuv_bkg],
		osd_buf_addr[osd_info.yuv_osd], osd_info.start_line, osd_info.total_line, osd_info.blend_all);
	flush_dcache_all();

	//FB_MSG("\nStart:\ny=%08x, \nu=%08x, \nv=%08x\n", ypbpr_ptr->y_addr, ypbpr_ptr->cb_addr, ypbpr_ptr->cr_addr);

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){

		tmp = 0xF0000000;
		while( tmp-- > 0x02 ) {
			flg0 = (ypbpr_ptr->disp_ctrl & YPBPR_CTRL_REG_FRAME_PTR_MASK);
			flg1 = (ypbpr_ptr->disp_ctrl & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg0 == 0) || (flg1 == 0) ){
				break;
			}
		}
	}
	/* frame start address. */
	tmp = osd_buf_addr[osd_info.yuv_osd];
	flg0 = pgu_ptr->cru_frm_st;
	flg1 = pgu_ptr->crv_frm_st;

	/* Switch to OSD buffer. */
	pgu_ptr->frm_start = tmp;

	/* Flowbee test the TV software osd. */
	ypbpr_ptr->y_addr = tmp;
	ypbpr_ptr->cb_addr = tmp + flg0;
	ypbpr_ptr->cr_addr = tmp + flg1;
	//FB_MSG("\nEnd:\ny=%08x, \nu=%08x, \nv=%08x\n", ypbpr_ptr->y_addr, ypbpr_ptr->cb_addr, ypbpr_ptr->cr_addr);

}


/*----------------------------------------------------------------------------*/
/*
 *	Turn on the overlay function.
 */
static void start_overlay(void)
{
	/* Software OSD for TV and LCD. */
	sw_osd_flag = FB_TRUE;
}

/*----------------------------------------------------------------------------*/
/*
	Turn off the overlay function.
*/
static void stop_overlay(void)
{
//	unsigned long tmp;
//	unsigned long cnt;
	//FB_MSG("\nOSD off\n");
	if(sw_osd_flag == FB_FALSE){
		FB_MSG("Disable twice.");
		return;
	}
	sw_osd_flag = FB_FALSE;

	/* switch back to the display buf. */
	do_switch(osd_info.yuv_bkg);

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
	|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL) ){
		wait_jiffies(3);
	}
}

/*----------------------------------------------------------------------------*/

static int calculate_dec_yuv_offsets(struct yuv_info *info)
{
	unsigned long tmpsize;
	int i;

	/* Flowbee: work out base offset. */
	/* Now we're motoring. Use this for U,V size now. */
	tmpsize = BUF_ALIGN( ((info->width/2 * info->height/2)+ info->alignment), info->alignment);

	info->u_offset = BUF_ALIGN(((info->width * info->height) + info->alignment), info->alignment);
	info->v_offset = BUF_ALIGN((info->u_offset + tmpsize + info->alignment), info->alignment);

	/* This contains the length of the buffer now. */
	tmpsize += info->v_offset;
	info->start_offset = fb_info.yuv_base[DISP_YUVBUF_NUMS];

	if ( (info->start_offset) > (CONFIG_RESERVE_FB_MEM_SIZE) ){
		return (-1);
	}

	/* Work out base offsts */
	dec_yuv_base[0] = BUF_ALIGN((info->start_offset + info->alignment),
		info->alignment);

	/* Find the total num of decode buffers.
		(total size) - (start_address) / (decode buffer size)
	*/
#if 0	// Flowbee@960422
	info->num_buffers = (CONFIG_RESERVE_FB_MEM_SIZE - dec_yuv_base[0]
						- CACHED_UNCACHED_MEM_SIZE) / (tmpsize);
#else
	info->num_buffers = (fb_info.yuv_max_size + fb_info.rgb_size
							- dec_yuv_base[0]) / (tmpsize);
#endif
	if (info->num_buffers > CONFIG_ARCPGU_YUVBUFS){
		info->num_buffers = CONFIG_ARCPGU_YUVBUFS;
	}
	else if (info->num_buffers == 0){
		printk("\nIn calculate_dec_yuv_offsets: buf_num = %ld\n", info->num_buffers);
		return (-1);
	}
	dec_num_buffers = info->num_buffers;
//	FB_MSG("\nDec Buf: %d\n", dec_num_buffers);

	for (i = 1; i < info->num_buffers; i++){
		dec_yuv_base[i] = BUF_ALIGN((dec_yuv_base[i-1] + tmpsize +
		    info->alignment), info->alignment);
	}

	return 0;
}


/*----------------------------------------------------------------------------*/

static int calculate_yuv_offsets(struct yuv_info *info)
{
	unsigned long tmpsize;
	int i;

	/* Check requested size - error if larger than the display */
#if 0
#if 0	/* Flowbee@960427. */
 	if ((p_current_display->yres < info->height)
		|| (p_current_display->xres < info->width)){
		printk("/nIn calculate_yuv_offsets(): Buffer size error,");
		printk("\nNew size(%d x %d) > Init size(%d x %d)",
			info->width, info->height, p_current_display->xres, p_current_display->yres);
 		return 1;
	}
#else
 	if ((p_current_display->xres < info->width)){
		printk("/nIn calculate_yuv_offsets(): Buffer size error,");
		printk("\nNew size(%d x %d) > Init size(%d x %d)",
			info->width, info->height, p_current_display->xres, p_current_display->yres);
 		return 1;
	}
#endif
#endif
	/* Calculate the yuv_buffer_numbers. */
	info->num_buffers = fb_info.yuv_max_size / ((info->height * info->width) + ((info->width * info->height)/2) + (4 * info->alignment));
	if ( info->num_buffers > CONFIG_ARCPGU_YUVBUFS ){
		info->num_buffers = CONFIG_ARCPGU_YUVBUFS;
	}
	disp_num_buffers = info->num_buffers;
//	FB_MSG("\nDisp Buf: %d\n", disp_num_buffers);

	/* Now we're motoring. Use this for U,V size now. */
	/* Flowbee@960418. */
	tmpsize = (info->width/2 * info->height/2);
	tmpsize = BUF_ALIGN((tmpsize + info->alignment), info->alignment);

	info->u_offset = BUF_ALIGN(((info->width * info->height) + info->alignment),info->alignment);
	info->v_offset = BUF_ALIGN((info->u_offset + tmpsize + info->alignment),info->alignment);

	/* This now contains the length of the buffer */
	tmpsize += info->v_offset;

	/* Work out base offsts */
	fb_info.yuv_base[0] = BUF_ALIGN((info->start_offset + info->alignment),
		info->alignment);

	//-- Flowbee 2008-01-16
//	for (i = 1; i < info->num_buffers; i++){
	for (i = 1; i < CONFIG_ARCPGU_YUVBUFS; i++){
		fb_info.yuv_base[i] = BUF_ALIGN((fb_info.yuv_base[i-1] + tmpsize +
			    info->alignment), info->alignment);
	}
	info->y_stride = info->width;
	info->u_stride = info->width/2;
	info->v_stride = info->width/2;
	return 0;
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_check_dims(struct yuv_info *info)
{
	if( (pgu_ptr->frm_dim == (ENCODE_PGU_DIMS((info->width-1), (info->height-1)))) &&
		(pgu_ptr->stride == info->y_stride) &&
		(pgu_ptr->cru_stride == info->u_stride) &&
		(pgu_ptr->crv_stride == info->v_stride) &&
		(pgu_ptr->cru_frm_st == info->u_offset) &&
		(pgu_ptr->crv_frm_st == info->v_offset)
	){
		return -1;
	}
	return 0;
}

/*----------------------------------------------------------------------------*/
/*
	Set FB dimensions which include y, u and v line stride.
	Set U and V offset.(It is the offset from Y start address.)
*/
static void arc_pgu_setdims(struct yuv_info *info)
{
/*	printk(KERN_INFO "Setting dims to: width %d, height %d, stride %d, u stride %d, v stride %d, u offset %d, v offset %d\n", */
/*				 info->width, info->height,fb_info.yuv.y_stride,fb_info.yuv.u_stride,fb_info.yuv.v_stride,fb_info.yuv.u_offset,fb_info.yuv.v_offset); */
	//FB_MSG("\nSet dim: x=%d, y=%d\n", info->width, info->height);
	pgu_ptr->frm_dim = ENCODE_PGU_DIMS((info->width-1), (info->height-1)); /* FIXME Last line is hidden to hide the green stripe */
	pgu_ptr->stride = info->y_stride;
	pgu_ptr->cru_stride = info->u_stride;
	pgu_ptr->crv_stride = info->v_stride;
	pgu_ptr->cru_frm_st = info->u_offset;
	pgu_ptr->crv_frm_st = info->v_offset;

	// set the YUV stride.
	ypbpr_ptr->ycbcr_stride = (info->y_stride << 16) | info->u_stride;
	if(ypbpr_ptr->ycbcr_stride != ((info->y_stride << 16) | info->u_stride)){
		FB_MSG("Dim ERROR !!");
	}
}

/*----------------------------------------------------------------------------*/
static int set_rgb_buf(unsigned char _buf)
{
	unsigned long flg0 = 0;
	unsigned long y_addr;
	unsigned long u_offset;
	unsigned long v_offset;

    FB_MSG("set_rgb_buf(%d)\n", _buf);

	/* frame start address. */
	y_addr = ((unsigned long)((void*)fb_info.yuv.virt_base)) + (_buf * (fb_info.rgb_size / CONFIG_ARCPGU_RGBBUFS));
	u_offset = fb_info.yuv.u_offset;
	v_offset = fb_info.yuv.v_offset;

	val = 0xF0000000;
	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){

		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = ypbpr_ptr->disp_ctrl;
			flg0 = (flg0 & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg0 == 0) ){
				break;
			}
		}
	}

	/* Flowbee@960309 */
	pgu_ptr->frm_start = y_addr;
	ypbpr_ptr->y_addr = y_addr;
	ypbpr_ptr->cb_addr = y_addr + u_offset;
	ypbpr_ptr->cr_addr = y_addr + v_offset;
	//FB_MSG("\nRGB Y:%08x, U:%08x, V:%08x", y_addr, y_addr + u_offset, y_addr + v_offset);
	FB_MSG("\nSet RGB: %d\n", _buf);
	wait_jiffies(5);

	return 0;
}

/*----------------------------------------------------------------------------*/
static int set_yuv_buf(unsigned char _buf)
{
	unsigned long flg0 = 0;
	unsigned long y_addr;
	unsigned long u_offset;
	unsigned long v_offset;

    FB_MSG("set_yuv_buf(%d)\n", _buf);

	/* frame start address. */
	y_addr = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[_buf];
	u_offset = fb_info.yuv.u_offset;
	v_offset = fb_info.yuv.v_offset;

	val = 0xF0000000;
	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){

		/* Check busy flag */
		while( val-- > 0x02 ) {
			flg0 = ypbpr_ptr->disp_ctrl;
			flg0 = (flg0 & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg0 == 0) ){
				break;
			}
		}
	}

	/* Flowbee@960309 */
	pgu_ptr->frm_start = y_addr;
	ypbpr_ptr->y_addr = y_addr;
	ypbpr_ptr->cb_addr = y_addr + u_offset;
	ypbpr_ptr->cr_addr = y_addr + v_offset;
	//FB_MSG("\nRGB Y:%08x, U:%08x, V:%08x", y_addr, y_addr + u_offset, y_addr + v_offset);
	//FB_MSG("\nSet RGB: %d\n", _buf);
	//wait_jiffies(5);

	return 0;
}


/*----------------------------------------------------------------------------*/
static int set_yuv_pointer(void)
{
	int newbuf = fb_info.yuv.displayed_buffer;
	unsigned long flg0 = 0, flg1 = 0;
	unsigned long tmp = 0;
	val = 0;

	//-- Flowbee 2008-01-16	not check frame buffer number.
	#if 0
	if (newbuf > fb_info.yuv.num_buffers){
		return -1;
	}
	#else
	tmp = fb_info.yuv_base[newbuf] + (fb_info.yuv.u_offset * 3) / 2;
	if(tmp > fb_info.fb_size){
		FB_MSG("\nFrame buffer addr(%08lx) bigger than buffer size(%08lx)\n", tmp, fb_info.yuv_max_size);
		return -1;
	}
	#endif

	/* frame start address. */
	tmp = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[newbuf];
	#if 1
	flg0 = fb_info.yuv.u_offset;
	flg1 = fb_info.yuv.v_offset;
	#else
	flg0 = pgu_ptr->cru_frm_st;
	flg1 = pgu_ptr->crv_frm_st;
	#endif

	/* Flowbee@960518. Blank the last pixel of each line. */
	if( (is_video_scale == 1) && (scalar.info.scale_rw == scalar.info.scale_w) ){
		//== if scale up
		for(val = 1; val <= scalar.info.scale_rh; val++){
			*((unsigned char *)(tmp + (val * scalar.info.scale_w) - 1)) = 0;
		}
		#if 1
		for(val = 1; val <= (scalar.info.scale_rh / 2); val++){
			*((unsigned char *)(tmp + flg0 + (val * (scalar.info.scale_w / 2)) - 1)) = 128;
		}
		for(val = 1; val <= (scalar.info.scale_rh / 2); val++){
			*((unsigned char *)(tmp + flg1 + (val * (scalar.info.scale_w / 2)) - 1)) = 128;
		}
		#endif
		flush_dcache_range(tmp, (tmp + (2*flg0)));
	}
	if( (is_video_scale == 1) && (scalar.info.dec_w > scalar.info.scale_w) ){
		#if (REDUCE_SCALE_DOWN_LINES > 0)
		pgu_ptr->frm_start = tmp + (fb_info.yuv.y_stride * REDUCE_SCALE_DOWN_LINES);
		pgu_ptr->cru_frm_st = (flg0 - (fb_info.yuv.y_stride * REDUCE_SCALE_DOWN_LINES)) + (fb_info.yuv.u_stride * REDUCE_SCALE_DOWN_LINES/2);
		pgu_ptr->crv_frm_st = (flg1 - (fb_info.yuv.y_stride * REDUCE_SCALE_DOWN_LINES)) + (fb_info.yuv.v_stride * REDUCE_SCALE_DOWN_LINES/2);
		#else
		//== if scale down
		//-- Clear first line at Y, Cb, Cr
		VOLATILE_MEMSET(tmp, 0x00, (fb_info.yuv.y_stride*4));
		// Reset the data in the U and V buffer to 0x88.
		VOLATILE_MEMSET(tmp + flg0, 0x80, (fb_info.yuv.u_stride*2));
		VOLATILE_MEMSET(tmp + flg1, 0x80, (fb_info.yuv.v_stride*2));
		#endif
	}
	else{
	     pgu_ptr->frm_start = tmp;
	}

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){

		//-- Clear status bit
		ypbpr_ptr->disp_ctrl |= (YPBPR_CTRL_REG_FRAME_BODY_READY_MASK
								| YPBPR_CTRL_REG_FRAME_PTR_READY_MASK);
		//-- Enable YPbPr ISR
		ypbpr_ptr->disp_ctrl |= (YPBPR_CTRL_REG_FRAME_BODY_INTEN_MASK
								| YPBPR_CTRL_REG_FRAME_PTR_INTEN_MASK);
	}

	//deflicker(newbuf);  jimmy, move to UI layer

	return 0;
}

/*----------------------------------------------------------------------------*/
/*
	Set the start address of user defined YUV buffer.
*/
static int set_user_defined_yuv_pointer(struct yuv_info *tmpyuv)
{
	unsigned long start_offset, y_stride, u_stride, v_stride;
    unsigned long ycbcr_stride, cb_addr, cr_addr;
    volatile unsigned long flg0 = 0, flg1 = 0;
	unsigned long tmp = 0;

    start_offset = tmpyuv->start_offset;
    y_stride = tmpyuv->y_stride;
    u_stride = tmpyuv->u_stride;
    v_stride = tmpyuv->v_stride;
    ycbcr_stride = (y_stride<<16)|(u_stride);
    cb_addr = tmpyuv->u_offset;
    cr_addr = tmpyuv->v_offset;

	tmp = 0xF0000000;

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){
		/* Check frame body flag which will be set at the blank period. */
		while( tmp-- > 0x02 ) {
			flg1 = (ypbpr_ptr->disp_ctrl & YPBPR_CTRL_REG_FRAME_BODY_MASK);
			if ( (flg1 == 0) ){
				break;
			}
		}

		//-- TV or HDTV
        ypbpr_ptr->y_addr = start_offset;
        ypbpr_ptr->cb_addr = cb_addr;
        ypbpr_ptr->cr_addr = cr_addr;
        if (pgu_ptr->stride != y_stride){
           pgu_ptr->stride = y_stride;
           pgu_ptr->cru_stride = u_stride;
           pgu_ptr->crv_stride = v_stride;
           ypbpr_ptr->ycbcr_stride = ycbcr_stride;
        }
	}
	else{
	    cb_addr = tmpyuv->u_offset - start_offset;
        cr_addr = tmpyuv->v_offset - start_offset;

		if(pgu_ptr->stride != y_stride)
        {
//			stop_arcpgu();
			/* Flowbee@960605. */
			//-- Use the LCD interrupt.
	  		if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
	  			/* Enable the interrupt and clear the interrupt flag. */
	  			pgu_ptr->statctrl |= STATCTRL_INTR_EN_MASK;
	  			pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
	  			while((pgu_ptr->statctrl & STATCTRL_INTR_FLAG) == 0){
	  				;
	  			}
	  			//-- disable the LCD module.
	  			pgu_ptr->statctrl &= ~STATCTRL_REG_DISP_EN_MASK;
	  			//-- disable the LCD interrupt.
	  			pgu_ptr->statctrl &= ~(STATCTRL_INTR_EN_MASK);
	  		}
	  		/* Check busy flag */
	  		while( tmp-- > 0x02 ) {
	  			flg0 = pgu_ptr->statctrl;
	  			flg1 = (flg0 & STATCTRL_DISP_BUSY_MASK);
	  			flg0 = (flg0 & STATCTRL_BU_BUSY_MASK);
	  			if ( (flg0 == 0) && (flg1 == 0) ){
	  				break;
	  			}
	  		}

            pgu_ptr->stride = y_stride;
            pgu_ptr->cru_stride = u_stride;
            pgu_ptr->crv_stride = v_stride;
	   	    pgu_ptr->frm_start = start_offset;
    	   	pgu_ptr->cru_frm_st = cb_addr;
        	pgu_ptr->crv_frm_st = cr_addr;

			/* Enable the PGU module. */
			pgu_ptr->statctrl |= STATCTRL_REG_DISP_EN_MASK;
        }
        //-- Digital
   	    pgu_ptr->frm_start = start_offset;
       	pgu_ptr->cru_frm_st = cb_addr;
        pgu_ptr->crv_frm_st = cr_addr;

	}

	return 0;
}

/*----------------------------------------------------------------------------*/
static int set_rgb_pointer(void)
{
	if (current_par.rgb_bufno > CONFIG_ARCPGU_RGBBUFS){
		return -1;
	}
	pgu_ptr->frm_start = (unsigned int)	( fb_info.fb_virt_start
		+ (current_par.rgb_bufno * p_current_display->xres * p_current_display->yres * (p_current_display->bpp / 8)) );

	pgu_ptr->stride = p_current_display->xres * (p_current_display->bpp / 8);

	return 0;
}

/*----------------------------------------------------------------------------*/
static void copy_pgu(arc_pgu *src, arc_pgu *dest)
{
	/* FIXME remove and replace with a memcpy */
	dest->clk_cfg = src->clk_cfg;
	dest->disp_dim = src->disp_dim;
	dest->hsync_cfg = src->hsync_cfg;
	dest->vsync_cfg = src->vsync_cfg;
	dest->frm_start = src->frm_start;
	dest->stride = src->stride;
	dest->frm_dim = src->frm_dim;
	dest->statctrl = src->statctrl;
	dest->cru_frm_st = src->cru_frm_st;
	dest->cru_stride = src->cru_stride;
	dest->crv_frm_st = src->crv_frm_st;
	dest->crv_stride = src->crv_stride;
	dest->ol_frm_st = src->ol_frm_st;
	dest->ol_stride = src->ol_stride;
	dest->ol_scn_line = src->ol_scn_line;
	dest->ol_col_key = src->ol_col_key;
	dest->centreoffsets = src->centreoffsets;
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_setmode(void)
{
	int dispmode;
//	int i;
//	unsigned long freq;
//	unsigned long clk_div;
//	unsigned long high;

	/*	printk(KERN_INFO "arc_pgufb: in arc_pgufb_setmode, lcd index is %d\n",display_index); */
	// get the overlay display mode
	dispmode = current_par.overlay_mode & STATCTRL_OL_FMT_MASK; /* current controller mode, not display mode. Make this boot param. */

	/* initialise controller */
	pgu_ptr->disp_dim = ENCODE_PGU_DIMS((p_current_display->xres-1), (p_current_display->yres-1));
	calculate_yuv_offsets(&(fb_info.yuv));
	arc_pgu_setdims(&fb_info.yuv);

	if ( (current_par.main_mode == STATCTRL_PIX_FMT_YUV420) ){
		set_yuv_pointer();
	}
	else{
		set_rgb_pointer();
	}

	blank_yuv(&(fb_info.yuv));
	memset((void *)fb_info.fb_virt_start,0,fb_info.rgb_size);

	pgu_ptr->hsync_cfg = p_current_display->hsync;
	pgu_ptr->vsync_cfg = p_current_display->vsync;

	pgu_ptr->clk_cfg = p_current_display->clkcontrol;

	FB_MSG(KERN_INFO "arc_pgu_setmode: fbbase 0x%lx, ybase 0x%lx, ubase 0x%lx, vbase 0x%lx\n",
		fb_info.fb_virt_start, pgu_ptr->frm_start, pgu_ptr->cru_frm_st,	pgu_ptr->crv_frm_st);

	// pgu overlay settings
	pgu_ptr->ol_frm_st = fb_info.fb_virt_start;
	pgu_ptr->ol_stride = p_current_display->xres * 2; // Always
	pgu_ptr->ol_scn_line = ENCODE_PGU_DIMS((0), (p_current_display->yres-1));
	pgu_ptr->ol_col_key = 0;
	pgu_ptr->centreoffsets = 0;
	pgu_ptr->statctrl |= (STATCTRL_CONT_B_MASK | current_par.main_mode | current_par.overlay_mode | p_current_display->control);
	wait_jiffies(1);

	return 0;
}

/*----------------------------------------------------------------------------*/
static void arc_pgu_detect(void)
{
	/*
	 *	This function should detect the current video mode settings
	 *	and store it as the default video mode
	 */
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_encode_fix(struct fb_fix_screeninfo *fix,
		const void *_par, struct fb_info_gen *_info)
{
	struct arc_pgufb_info *info = (struct arc_pgufb_info *) _info;
	struct arc_pgufb_par *par = (struct arc_pgufb_par *) _par;
	struct fb_var_screeninfo *var = &par->var;

	memset(fix, 0, sizeof(struct fb_fix_screeninfo));
	fix->smem_start = fb_info.fb_virt_start;
	/*	fix->smem_start = info->fb_phys; */
	fix->smem_len = info->fb_size;
	fix->mmio_start = (unsigned int) pgu_ptr;
	fix->mmio_len = sizeof(arc_pgu);
	fix->type = FB_TYPE_PACKED_PIXELS;
	fix->type_aux = 0;
	fix->visual = (var->bits_per_pixel == 8) ? FB_VISUAL_PSEUDOCOLOR : FB_VISUAL_TRUECOLOR;
	fix->ywrapstep = 0;
	fix->xpanstep = 1;
	fix->ypanstep = 1;
	fix->line_length = current_par.line_length;
	return 0;
}

/*----------------------------------------------------------------------------*/
static void set_color_bitfields(struct fb_var_screeninfo *var)
{
	switch (var->bits_per_pixel)
	{
		case 16:
		case 15:
		{
			if (current_par.main_is_fb)
			{
					var->red.offset = 10;
					var->red.length = 5;
					var->green.offset = 5;
					var->green.length = 5;
					var->blue.offset = 0;
					var->blue.length = 5;
					var->transp.offset = 0;
					var->transp.length = 0;
			}
			else
			{
				switch(current_par.overlay_mode)
				{
					case STATCTRL_OL_FMT_RGBA4444:
						var->red.offset = 8;
						var->red.length = 4;
						var->green.offset = 4;
						var->green.length = 4;
						var->blue.offset = 0;
						var->blue.length = 4;
						var->transp.offset = 12;
						var->transp.length = 4;
						break;
					case STATCTRL_OL_FMT_RGBA5551:
						var->red.offset = 10;
						var->red.length = 5;
						var->green.offset = 5;
						var->green.length = 5;
						var->blue.offset = 0;
						var->blue.length = 5;
						var->transp.offset = 15;
						var->transp.length = 1;
						break;
					case STATCTRL_OL_FMT_RGB555:
					default:
						var->red.offset = 10;
						var->red.length = 5;
						var->green.offset = 5;
						var->green.length = 5;
						var->blue.offset = 0;
						var->blue.length = 5;
						var->transp.offset = 0;
						var->transp.length = 0;
						break;
				}
			}
			break;
		}
		default:
			var->red.offset = 0;
			var->red.length = 0;
			var->green.offset = 0;
			var->green.length = 0;
			var->blue.offset = 0;
			var->blue.length = 0;
			var->transp.offset = 0;
			var->transp.length = 0;
			break;
	}
	var->red.msb_right = 0;
	var->green.msb_right = 0;
	var->blue.msb_right = 0;
	var->transp.msb_right = 0;
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_decode_var(const struct fb_var_screeninfo *var,
		void *_par, struct fb_info_gen *_info)
{

	struct arc_pgufb_par *par = (struct arc_pgufb_par *)_par;

	/*
	 * Don't allow setting any of these yet: xres and yres don't
	 * make sense for LCD panels.
	 */
	if (var->xres != p_current_display->xres ||
			var->yres != p_current_display->yres) {
		return -EINVAL;
	}
	if(var->bits_per_pixel != p_current_display->bpp) {
		return -EINVAL;
	}

	memset(par, 0, sizeof(struct arc_pgufb_par));
	par->var = *var;

	switch (var->bits_per_pixel) {
		case 16:
			par->var.bits_per_pixel = 16;
			break;
		default:
			printk("color depth %d bpp not supported\n",
						 var->bits_per_pixel);
			return -EINVAL;

	}
	set_color_bitfields(&par->var);
	par->cmap_len = (par->var.bits_per_pixel == 8) ? 256 : 16;
	return 0;
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_encode_var(struct fb_var_screeninfo *var,
		const void *par, struct fb_info_gen *_info)
{
	*var = ((struct arc_pgufb_par *)par)->var;
	set_color_bitfields(var);
	return 0;
}

/*----------------------------------------------------------------------------*/
static void arc_pgu_get_par(void *_par, struct fb_info_gen *_info)
{
	*(struct arc_pgufb_par *)_par = current_par;
}

/*----------------------------------------------------------------------------*/
static void arc_pgu_set_par(const void *par, struct fb_info_gen *info)
{
	/* nothing to do: we don't change any settings */
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_getcolreg(unsigned regno, unsigned *red, unsigned *green,
			 unsigned *blue, unsigned *transp,
			 struct fb_info *info)
{

	struct arc_pgufb_info* i = (struct arc_pgufb_info*)info;

	if (regno > 255)
		return 1;

	*red		= i->palette[regno].red;
	*green	= i->palette[regno].green;
	*blue		= i->palette[regno].blue;
	*transp = 0;

	return 0;
}

/*----------------------------------------------------------------------------*/
static int arc_pgu_setcolreg(unsigned regno, unsigned red, unsigned green,
			 unsigned blue, unsigned transp,
			 struct fb_info *info)
{
	struct arc_pgufb_info* i = (struct arc_pgufb_info *)info;

	if (regno > 255)
		return 1;

	i->palette[regno].red		 = red;
	i->palette[regno].green	 = green;
	i->palette[regno].blue	 = blue;

	switch(p_current_display->bpp) {
#ifdef FBCON_HAS_CFB16
	case 16:
		i->fbcon_cmap16[regno] =
			((red & 0xf800) >> 10) |
			((green & 0xf800) >> 5) |
			((blue & 0xf800));
		break;
#endif
	default:
		break;
	}

	return 0;
}

/*----------------------------------------------------------------------------*/
static int	arc_pgu_blank(int blank_mode, struct fb_info_gen *_info)
{
	switch (blank_mode) {
	case VESA_NO_BLANKING:
		/* turn on panel */
/* 		if (p_current_display->bl_active_high) pgu_ptr->statctrl |= STATCTRL_BACKLIGHT_MASK; */
/* 		else pgu_ptr->statctrl &= ~STATCTRL_BACKLIGHT_MASK; */
/* 		pgu_ptr->statctrl |= (STATCTRL_CONT_B_MASK); */
/* 		pgu_ptr->statctrl |= STATCTRL_REG_DISP_EN_MASK; */
		break;

	case VESA_VSYNC_SUSPEND:
	case VESA_HSYNC_SUSPEND:
	case VESA_POWERDOWN:
		/* turn off panel */
		//		stop_arcpgu();
		break;
	default:
		break;
	}
	return 0;
}

/*----------------------------------------------------------------------------*/
static void arc_pgu_set_disp(const void *unused, struct display *disp,
			 struct fb_info_gen *info)
{
	disp->screen_base = (char *)fb_info.fb_virt_start;

	switch (disp->var.bits_per_pixel) {
#ifdef FBCON_HAS_CFB8
	case 8:
		disp->dispsw = &fbcon_cfb8;
		//		if (fb_info.nohwcursor)
			fbcon_cfb8.cursor = arc_pgu_nocursor;
		break;
#endif
#ifdef FBCON_HAS_CFB16
	case 16:
		disp->dispsw = &fbcon_cfb16;
		disp->dispsw_data = fb_info.fbcon_cmap16;
		//		if (fb_info.nohwcursor)
		fbcon_cfb16.cursor = arc_pgu_nocursor;
		break;
#endif
	default:
		printk("arc_pgu_set_disp setting dummy display, since bits per pixel are %d\n",disp->var.bits_per_pixel);
		disp->dispsw = &fbcon_dummy;
		disp->dispsw_data = NULL;
		break;
	}
}

/*----------------------------------------------------------------------------*/
int arc_pgu_pan_display(const struct fb_var_screeninfo *var,
					 struct fb_info_gen *info)
{
	return 0;
}

/*----------------------------------------------------------------------------*/
/*
	Switch YUV buffer.
*/
int do_switch(int fb)
{
	current_par.main_is_fb = 0;
	fb_info.yuv.displayed_buffer = fb;
	return set_yuv_pointer() ? -EFAULT : 0;
}


/*----------------------------------------------------------------------------*/
static struct fbgen_hwswitch arc_pgu_switch = {
	arc_pgu_detect,
	arc_pgu_encode_fix,
	arc_pgu_decode_var,
	arc_pgu_encode_var,
	arc_pgu_get_par,
	arc_pgu_set_par,
	arc_pgu_getcolreg,
	arc_pgu_setcolreg,
	arc_pgu_pan_display,
	arc_pgu_blank,
	arc_pgu_set_disp
};

/*----------------------------------------------------------------------------*/
void init_disp_parameter(void)
{
	p_current_display = &displays[display_index];

	printk(KERN_INFO "ARC PGU: controller version %lu, using the %s\n",
				 ((pgu_ptr->pgu_id & PGU_VERSION_MASK) >> PGU_VERSION_SHIFT),
				 p_current_display->display_name);
	/*
	 * Panel dimensions x bpp must be divisible by 32
	 */
	if (((p_current_display->yres * p_current_display->bpp) % 32) != 0)
		FB_MSG("VERT %% 32\n");
	if (((p_current_display->xres * p_current_display->bpp) % 32) != 0)
		FB_MSG("HORZ %% 32\n");

/*
	Init current_par data structure.
*/
	current_par.num_rgbbufs = CONFIG_ARCPGU_RGBBUFS;
	current_par.var.xres = p_current_display->xres;
	current_par.var.xres_virtual = p_current_display->xres;
	current_par.var.yres = p_current_display->yres;
	current_par.var.yres_virtual = p_current_display->yres;
	current_par.var.bits_per_pixel = p_current_display->bpp;

	/* FIX!!! only works for 8/16 bpp */
	current_par.line_length = p_current_display->xres * p_current_display->bpp / 8; /* in bytes */
	current_par.main_mode = STATCTRL_PIX_FMT_YUV420;
	current_par.overlay_mode = STATCTRL_OL_FMT_RGBA5551;
	current_par.rgb_bufno = 0;
	current_par.main_is_fb = 0;	// 0: YCbCr, 1: RGB

}

/*----------------------------------------------------------------------------*/
int init_fb_info(void)
{
	unsigned long page;
	unsigned long rgb_buf_size;

	FB_MSG("\n --- Disp: %s, res: %d x %d ---\n", p_current_display->display_name, p_current_display->xres, p_current_display->yres);
	/*
	 * Allocate default framebuffers from system memory
	 */
	fb_info.fb_size = CONFIG_RESERVE_FB_MEM_SIZE;

	/* This bit's easy */
	// caculate the total size of RGB buffers.
	rgb_buf_size = (p_current_display->xres * p_current_display->yres * (p_current_display->bpp / 8));
	fb_info.rgb_size = CONFIG_ARCPGU_RGBBUFS * rgb_buf_size;

	/* Store the current x and y dimensions */
	fb_info.yuv.width = p_current_display->xres;
	fb_info.yuv.height = p_current_display->yres;

	/* Work out our default YUV buffers arrangement, and save it */
	// caculate the total size of YUV buffers.
	// each YUV buffer has... (ex: 640*480 + 320*240 + 320*240)
	//-- Flowbee@960320.
	//-- Flowbee@970111
	fb_info.yuv_max_size = fb_info.fb_size - fb_info.rgb_size
								- CACHED_UNCACHED_MEM_SIZE;

	fb_info.yuv.yuv_size = CONFIG_ARCPGU_YUVBUFS * ((p_current_display->yres * p_current_display->xres) + ((p_current_display->yres * p_current_display->xres)/2) + (4 * DEFAULT_ALIGN_SIZE));
	fb_info.yuv.alignment = DEFAULT_ALIGN_SIZE;
	fb_info.yuv.start_offset = fb_info.rgb_size;
	fb_info.yuv.displayed_buffer = 0;

	/* Calculate the YUV buffer offset(s) and default strides */
	calculate_yuv_offsets(&fb_info.yuv);

	FB_MSG("Framebuffer: Size = 0x%lx (RGB: 0x%lx, YUV: 0x%lx) %d RGB, %d YUV\n",
				 fb_info.fb_size, fb_info.rgb_size,
				 fb_info.yuv.yuv_size, CONFIG_ARCPGU_RGBBUFS, CONFIG_ARCPGU_YUVBUFS);
//	FB_MSG("YUV max size: %x, disp_num_buffers: %ld\n", fb_info.yuv.yuv_size, disp_num_buffers);
//	FB_MSG("CACHED_UNCACHED_MEM_SIZE: %d, CONFIG_UNCACHED_REGION_SIZE: %d, CONFIG_CACHED_REGION_SIZE: %d\n",
//		CACHED_UNCACHED_MEM_SIZE, CONFIG_UNCACHED_REGION_SIZE, CONFIG_CACHED_REGION_SIZE);

#ifdef CONFIG_RESERVE_FB_MEM
	// (0x00000000 + 0x4000000 + 0x80000000) - 0x1000000
	//  SRAM_START   64 MB       PHY_OFFSET    16MB
	fb_info.fb_virt_start = END_MEM - CONFIG_RESERVE_FB_MEM_SIZE;
#else
/*	fb_info.fb_virt_start = (unsigned long ) */
/*		__get_free_pages(GFP_ATOMIC | GFP_DMA, */
/*				get_order(fb_info.fb_size + DEFAULT_ALIGN_SIZE)); */
	fb_info.fb_virt_start = (unsigned long )
		__get_free_pages(GFP_KERNEL,
				get_order(fb_info.fb_size + DEFAULT_ALIGN_SIZE));
#endif

	if (!fb_info.fb_virt_start) {
		printk("Unable to allocate fb memory\n");
		return -ENOMEM;
	}
	fb_info.fb_phys = virt_to_phys((void *)fb_info.fb_virt_start);			// Appropriate for ARC? Does it matter?
	//memset((void *)fb_info.fb_virt_start, 0, get_order(fb_info.fb_size));
	fb_info.yuv.phys_base = fb_info.fb_phys;
	fb_info.yuv.virt_base = fb_info.fb_virt_start;

	//-- Flowbee 2008-04-24
	rgb0_addr = fb_info.fb_virt_start;
	rgb1_addr = rgb0_addr + rgb_buf_size;

	FB_MSG(KERN_INFO "Framebuffer at 0x%lx (logical), 0x%lx (physical)\n", fb_info.fb_virt_start, fb_info.fb_phys);
	/*
	 * Set page reserved so that mmap will work. This is necessary
	 * since we'll be remapping normal memory.
	 */
	for (page = fb_info.fb_virt_start;
			 page < BUF_ALIGN((fb_info.fb_virt_start + (CONFIG_RESERVE_FB_MEM_SIZE) + DEFAULT_ALIGN_SIZE), DEFAULT_ALIGN_SIZE);
			 page += PAGE_SIZE) {
		SetPageReserved(virt_to_page(page));
	}

	fb_info.gen.parsize = sizeof(struct arc_pgufb_par);
	fb_info.gen.fbhw = &arc_pgu_switch;
	strcpy(fb_info.gen.info.modename, "ARC PGU");
	fb_info.gen.info.changevar = NULL;
	fb_info.gen.info.node = -1;
	fb_info.gen.info.fbops = &arc_pgufb_ops;
	fb_info.gen.info.disp = &disp;
	fb_info.gen.info.switch_con = &fbgen_switch;
	fb_info.gen.info.updatevar = &fbgen_update_var;
	fb_info.gen.info.blank = &fbgen_blank;
	fb_info.gen.info.flags = FBINFO_FLAG_DEFAULT;
	/* This should give a reasonable default video mode */
	fbgen_get_var(&disp.var, -1, &fb_info.gen.info);
	fbgen_do_set_var(&disp.var, 1, &fb_info.gen);
	fbgen_set_disp(-1, &fb_info.gen);
	arc_pgu_set_disp(0,&disp,&fb_info.gen);
	fbgen_install_cmap(0, &fb_info.gen);

	/* arc_pgu_decode_var(&(current_par.var), &current_par, &fb_info.gen); */
	memcpy(&fb_info.default_yuv, &fb_info.yuv, sizeof(struct yuv_info));
	/* Flowbee: init the dec_yuv_info. */
	memcpy(&dec_yuv, &fb_info.yuv, sizeof(struct yuv_info));
	/* Flowbee: init the dec_yuv_base. */
	memcpy(dec_yuv_base, fb_info.yuv_base, (sizeof(unsigned long)*CONFIG_ARCPGU_YUVBUFS));

	return 0;

}

/*----------------------------------------------------------------------------*/
#define RESET_CTRL_REG			0xC0FC1808
#define CHANGE2PAL_TIME			(1)
//static void init_pgu_ypbpr(void)
int init_pgu_ypbpr(void)
{
	unsigned long freq;
	unsigned long tmpsize;
	unsigned long _time;
	unsigned long reg_val;

	//-- Turn off video DAC. (1: power off, 0: power on)
	VDAC_CTRL_REG |= (VDAC_EN_MASK);

	#if 0
	/* Init himax T-CON. */
	if ( (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HIMAX_480240_TFT) ){
		init_himax8817a();
	}
	#endif

	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
		|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
		|| (display_index == DISP_HDTV) ){
		/* TV settings. */

		#if CONFIG_TERAWINS_TCON
		//-- Turn off pwm(backlight).
        tcon_pwm_on_off(0);
		#endif

		/* Set video output mux for TV or LCD. */
		freq = 0;
		freq = DISP_MUX_REG;
		while (freq == 0){
			freq = DISP_MUX_REG;
		}
		freq &= ~(LOCK_PATTERN_MASK);
		freq |= LOCK_PATTERN;
		//-- Set TV out.
		freq &= ~(LCD_SEL_MASK);
		DISP_MUX_REG = freq;
		DISP_MUX_REG = freq;

		/* Set 1080I PLL divider and display clock. */
		//-- Get PLL divider register value.
		freq = 0;
		freq = YPBPR_1080I_REG;
		while (freq == 0){
			freq = YPBPR_1080I_REG;
		}
		freq &= ~(YPBPR_LOCK_MASK);
		freq |= YPBPR_LOCK_PATTERN;

		//-- Unlock display register.
		tmpsize = 0;
		tmpsize |= LOCK_PATTERN;

		if (display_index == DISP_HDTV){
			/* PLL divider (divide by 1). */
			freq &= ~(VIDEO_SEL_DIV2_MASK);
			YPBPR_1080I_REG = freq;
			YPBPR_1080I_REG = freq;
			/* Display clock (74.25MHz). */
			tmpsize |= DISP_CLK_74250KHZ;
			DISP_CLK_REG = tmpsize;
			DISP_CLK_REG = tmpsize;
		}
		else{
			/* PLL divider (divide by 2). */
			freq |= (VIDEO_SEL_DIV2_MASK);
			YPBPR_1080I_REG = freq;
			YPBPR_1080I_REG = freq;
			/* Display clock (27MHz). */
			tmpsize |= DISP_CLK_27MHZ;
			DISP_CLK_REG = tmpsize;
			DISP_CLK_REG = tmpsize;
		}
		/* Setting TV output mode.(NTSC, PAL, HDTV) */
		if ( (display_index == DISP_NTSC) || (display_index == DISP_HIMAX_NTSC) ){
		//-- Set to NTSC
			#ifdef CONFIG_ARCPGU_TV_OUT_YPBPR // Flowbee 2008-04-22
			reg_val = ypbpr_ptr->disp_ctrl;
			reg_val &= ~(YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			reg_val |= (ACTIVE_MODE_NTSC);
			ypbpr_ptr->disp_ctrl = reg_val;

			#else	//~CONFIG_ARCPGU_TV_OUT_YPBPR // Flowbee 2008-04-22
			reg_val = ypbpr_ptr->disp_ctrl;
			reg_val &= ~(YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			reg_val |= (ACTIVE_MODE_NTSC);
			ypbpr_ptr->disp_ctrl = reg_val;

			tv_ptr->fmt_ctrl = TVE_NTSC_M;
			reg_val = tv_ptr->fmt_ctrl;
			while (reg_val != TVE_NTSC_M){
				tv_ptr->fmt_ctrl = TVE_NTSC_M;
				reg_val = tv_ptr->fmt_ctrl;
			}
			#endif
			FB_MSG("\n NTSC disp: %08lx, fmt_ctrl: %08lx", ypbpr_ptr->disp_ctrl, tv_ptr->fmt_ctrl);
		}
		else if ( (display_index == DISP_PAL) || (display_index == DISP_HIMAX_PAL) ){
		//-- Set to PAL
			#ifdef CONFIG_ARCPGU_TV_OUT_YPBPR // Flowbee 2008-04-22
			reg_val = ypbpr_ptr->disp_ctrl;
			reg_val &= ~(YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			reg_val |= (ACTIVE_MODE_PAL);
			ypbpr_ptr->disp_ctrl = reg_val;

			#else
			reg_val = ypbpr_ptr->disp_ctrl;
			reg_val &= ~(YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			reg_val |= (ACTIVE_MODE_PAL);
			ypbpr_ptr->disp_ctrl = reg_val;

			tv_ptr->fmt_ctrl = TVE_PAL_B_D_G_H_I;
			reg_val = tv_ptr->fmt_ctrl;
			_time = jiffies + CHANGE2PAL_TIME * HZ;
			while (reg_val != TVE_PAL_B_D_G_H_I){
				tv_ptr->fmt_ctrl = TVE_PAL_B_D_G_H_I;
				reg_val = tv_ptr->fmt_ctrl;
				//flush_dcache_range(&(tv_ptr->fmt_ctrl), &(tv_ptr->fmt_ctrl) + 4);
				//FB_MSG("\n %08p = %08lx", &(tv_ptr->fmt_ctrl), tv_ptr->fmt_ctrl);
				if(jiffies > _time){
					FB_MSG("\nPAL Error !! ");
					FB_MSG("\n PAL disp: %08lx, fmt_ctrl: %08lx", ypbpr_ptr->disp_ctrl, tv_ptr->fmt_ctrl);
					return -1;
				}
			}
			#endif
			FB_MSG("\n PAL disp: %08lx, fmt_ctrl: %08lx", ypbpr_ptr->disp_ctrl, tv_ptr->fmt_ctrl);
		}
		else if (display_index == DISP_HDTV){
		//-- Set to HDTV
			reg_val = ypbpr_ptr->disp_ctrl;
			reg_val &= ~(YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			reg_val |= (ACTIVE_MODE_1080I_30FPS);
			ypbpr_ptr->disp_ctrl = reg_val;
			FB_MSG("\n disp: %08lx, fmt_ctrl: %08lx", ypbpr_ptr->disp_ctrl, tv_ptr->fmt_ctrl);
		}
		/* Select tv output interface. */
		set_tv_output_interface();
	}
	else{
	/* LCD settings. */
		#if CONFIG_TERAWINS_TCON
		/* Init terawins T-CON. */
		init_tcon();
		/* Init terawins OSD. */
		init_tcon_osd();
		#endif

		/* Set video output mux for TV or LCD */
		freq = 0;
		freq = DISP_MUX_REG;
		while (freq == 0){
			freq = DISP_MUX_REG;
		}
		freq &= ~(LOCK_PATTERN_MASK);
		freq |= LOCK_PATTERN;

		/* Set LCD out. */
		freq |= (LCD_SEL_MASK);
		DISP_MUX_REG = freq;
		DISP_MUX_REG = freq;

		/*
		 *  Setting the control registers for the specified LCD panel.
 		 */
		pgu_ptr->clk_cfg = p_current_display->clkcontrol;

		/* Setting the active size of the LCD panel. */
		pgu_ptr->disp_dim = ENCODE_PGU_DIMS((p_current_display->xres-1), (p_current_display->yres-1));

		/* Setting the sync. signal for the specified LCD panel. */
		pgu_ptr->hsync_cfg = p_current_display->hsync;
		pgu_ptr->vsync_cfg = p_current_display->vsync;

		/* Reset pgu control register. */
		pgu_ptr->statctrl = 0;
		pgu_ptr->statctrl |= (STATCTRL_CONT_B_MASK | STATCTRL_PIX_FMT_YUV420 | p_current_display->control);

		/* Flowbee@960310 - Set LCD back light. */
		#if CONFIG_TERAWINS_TCON
			#if CONFIG_FASHION_STAR
			backlight_on_off(1);
			//-- Turn on pwm(backlight).
			tcon_pwm_on_off(1);
			#else
			//-- Turn on pwm(backlight).
			tcon_pwm_on_off(1);
			#endif
		#else
		backlight_on_off(1);
		#endif

	}

	/*
	 *  Setting the yuv buffer base address and relative register settings
	 *  for the default image size.
	 */
	arc_pgu_setdims(&fb_info.yuv);
	set_yuv_pointer();

	/* Flowbee@960309 */
	pgu_ptr->statctrl &= ~(STATCTRL_OL_FMT_MASK);
	pgu_ptr->statctrl |= STATCTRL_OL_FMT_RGBA5551;
	pgu_ptr->statctrl &= ~(STATCTRL_OL_VAL_MASK);
	pgu_ptr->statctrl |= ((unsigned long)alpha_val << STATCTRL_OL_VAL_SHIFT);
	pgu_ptr->ol_frm_st = fb_info.fb_virt_start;
	pgu_ptr->ol_stride = p_current_display->xres * 2;
	pgu_ptr->ol_scn_line = ENCODE_PGU_DIMS((0), (p_current_display->yres-1));
	pgu_ptr->ol_col_key = 0;

	ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_OL_FMT_MASK);
	ypbpr_ptr->disp_ctrl |= (OL_FMT_ARGB1555);
	ypbpr_ptr->rgb_addr = fb_info.fb_virt_start;
	//ypbpr_ptr->rgb_stride = ((p_current_display->xres * 2) | (STATCTRL_OL_VAL >> 12));
	ypbpr_ptr->rgb_stride = ((p_current_display->xres * 2) | (((unsigned long)(15 - alpha_val)) << YPBPR_ALPHA_SHIFT));
	ypbpr_ptr->rgb_range = ENCODE_PGU_DIMS((1), (p_current_display->yres-1));

	//-- Disable all osd function.
//	turn_all_osd_off();	// Flowbee 2008-02-21

	//-- Flowbee 2007-11-19 Set center.
	pgu_ptr->centreoffsets = 0;
	ypbpr_ptr->HBlank = 0;
	ypbpr_ptr->VBlank = 0;

	return 0;
}


/*----------------------------------------------------------------------------*/
#if 0
void scalar_isr(int irq, void *dev_id, struct pt_regs * regs)
{
	struct _scalar_info *pinfo = dev_id;

	if (pinfo->state == SCALAR_Y){
		/* Scaling U. */

		pinfo->state = SCALAR_U;
	}
	else if (pinfo->state == SCALAR_U){
		/* Scaling V. */

		pinfo->state = SCALAR_V;
	}
	else if (pinfo->state == SCALAR_V){
		/* Set to SCALAR_IDLE. */

		pinfo->state = SCALAR_IDLE;
	}
}
#else
/*----------------------------------------------------------------------------*/
void scalar_isr(int irq, void *dev_id, struct pt_regs * regs)
{
	//-- clear the interrupt flag.
	scalar_ptr->interrupt |= 0x00000001;

	scalar.state ++;

	if(scalar.state == SCALAR_IDLE){
		//-- disable the scalar interrupt.
		scalar_ptr->interrupt = 0x00000000;
		return;
	}

	/* Set the scalar register. */
	//-- reset scalar
	scalar_ptr->ctrl = 0xC0000040;
	scalar_ptr->ctrl = 0xC0000040;
	//-- set width and height.
	scalar_ptr->src_res = scalar.yuv[scalar.state].src_res;
	scalar_ptr->dest_res = scalar.yuv[scalar.state].dest_res;

	//-- set blank and div.
	scalar_ptr->blank = scalar.yuv[scalar.state].blank;
	scalar_ptr->div = scalar.yuv[scalar.state].div;

	//-- set address.
	scalar_ptr->src_addr = scalar.yuv[scalar.state].src_addr;
	scalar_ptr->dest_addr = scalar.yuv[scalar.state].dest_addr;

	//-- start scaling.
	scalar_ptr->ctrl = scalar.yuv[scalar.state].ctrl;
	scalar_ptr->ctrl = scalar.yuv[scalar.state].ctrl;

	return;
}

#endif

/*----------------------------------------------------------------------------*/
void pix_isr(int irq, void *dev_id, struct pt_regs * regs)
{
	return;
}

/*----------------------------------------------------------------------------*/
unsigned long tv_error_cnt = 0;
void ypbpr_isr(int irq, void *dev_id, struct pt_regs * regs)
{
	int newbuf = fb_info.yuv.displayed_buffer;
	unsigned long tmp = 0;
	unsigned long flg0 = 1, flg1 = 1;
//	unsigned long flg3 = 1, tmp1 = 0, tmp2 = 0;

	//tmp1 = ypbpr_ptr->disp_ctrl;
	//-- Clear status bit
	ypbpr_ptr->disp_ctrl |= (YPBPR_CTRL_REG_FRAME_BODY_READY_MASK
							| YPBPR_CTRL_REG_FRAME_PTR_READY_MASK);

	/* frame start address. */
	#if 1
	tmp = pgu_ptr->frm_start;
	#else
	tmp = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[newbuf];
	#endif
	flg0 = pgu_ptr->cru_frm_st;
	flg1 = pgu_ptr->crv_frm_st;

	//tmp2 = ypbpr_ptr->disp_ctrl;

	ypbpr_ptr->y_addr = tmp;
	ypbpr_ptr->cb_addr = tmp + flg0;
	ypbpr_ptr->cr_addr = tmp + flg1;
	temp_val = ypbpr_ptr->y_addr;
	while(temp_val != tmp){
		ypbpr_ptr->y_addr = tmp;
		ypbpr_ptr->cb_addr = tmp + flg0;
		ypbpr_ptr->cr_addr = tmp + flg1;
		temp_val = ypbpr_ptr->y_addr;
		//flg3 = 0xFF;
	}

	//-- Disable YPbPr ISR
	ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_FRAME_BODY_INTEN_MASK
							| YPBPR_CTRL_REG_FRAME_PTR_INTEN_MASK);

	#if 0	//-- Debug message
	if(flg3 == 0xFF){
		tv_error_cnt ++;
		FB_MSG("\nERROR: %06d\n", tv_error_cnt);
	}
	flg0 = (tmp1 & YPBPR_CTRL_REG_FRAME_PTR_MASK);
	flg1 = (tmp1 & YPBPR_CTRL_REG_FRAME_BODY_MASK);
	if(flg0 == 0){
		FB_MSG("\nPTR");
	}
	if(flg1 == 0){
		FB_MSG("\nBODY");
	}
	flg0 = 0;
	flg1 = 0;
	flg0 = (tmp1 & YPBPR_CTRL_REG_FRAME_PTR_READY_MASK);
	flg1 = (tmp1 & YPBPR_CTRL_REG_FRAME_BODY_READY_MASK);
	if(flg0 == YPBPR_CTRL_REG_FRAME_PTR_READY_MASK){
		FB_MSG("\nPTR_RDY");
	}
	if(flg1 == YPBPR_CTRL_REG_FRAME_BODY_READY_MASK){
		FB_MSG("\nBODY_RDY");
	}
//	FB_MSG("\ntmp1:(%08x)", tmp1);
//	FB_MSG("\ntmp2:(%08x)", tmp2);
//	FB_MSG("\ntmp :(%08x)", tmp);
//	FB_MSG("\nBuf: %d (%08x) i\n\n", newbuf, ypbpr_ptr->y_addr);
	#endif
	return;
}

/*----------------------------------------------------------------------------*/
extern size_t sky_versions_write(const char *buf, size_t count, int cr );
const char fb_version[] = "Frame Buffer Driver v1.0\n";

int __init arc_pgufb_init(void)
{
	/* Flowbee. */
	sky_versions_write(fb_version, sizeof(fb_version), 0);

	//-- Flowbee 2008-04-09
//	#elif (CONFIG_SKY_IO_FOXCONN_COMPANY_V1 || CONFIG_LT_TV_SWITCH)
	#if (defined(TV_SWITCH_MOD) && defined(TV_SWITCH_INTERFACE))
	// set gpio input to get display interface (YPbPr/CVBS)
	GPIO_INPUT(SKY_GPIO_DIR, (1 << TV_SWITCH_INTERFACE));
	GPIO_ST(SKY_GPIO_DATA, (1 << TV_SWITCH_INTERFACE));
	if(vtval == 0){
		// CVBS -> check PAL(1) / NTSC(0)
		GPIO_INPUT(SKY_GPIO_DIR, (1 << TV_SWITCH_MOD));
		GPIO_ST(SKY_GPIO_DATA, (1 << TV_SWITCH_MOD));
		display_index = (vtval == 0)? DISP_NTSC: DISP_PAL;
	}
	else{
		// YPbPr -> set to HDTV
		display_index = DISP_HDTV;
		tv_interface = SKY_TV_YPBPR;
	}
	//set_tv_output_interface();
	FB_MSG("\n--- FC / LT (%d, %d) ---\n", display_index, tv_interface);

//	#if ( CONFIG_SKY_IO_FS_COMPANY_V2 || CONFIG_SKY_IO_WINACCORD_COMPANY_V1)
	#elif defined(TV_SWITCH_INTERFACE)
	// set gpio input
	GPIO_INPUT(SKY_GPIO_DIR, (1 << TV_SWITCH_INTERFACE));
	GPIO_ST(SKY_GPIO_DATA, (1 << TV_SWITCH_INTERFACE));
	if(vtval == 0){
		display_index = DISP_NTSC;
	}
	else{
		display_index = DISP_HDTV;
		tv_interface = SKY_TV_YPBPR;
	}
	//-- Flowbee 2008-04-09
	set_tv_output_interface();
	FB_MSG("\n--- ML / FS / WA (%d, %d) ---\n", display_index, tv_interface);

	#else
	display_index = CONFIG_ARCPGU_DISPTYPE;
	if(display_index == DISP_HDTV){
		tv_interface = SKY_TV_YPBPR;
	}

	#endif
	p_current_display = &displays[display_index];

	/* Init display data info. */
	init_disp_parameter();

	#if CONFIG_TERAWINS_TCON
	/* Init terawins OSD. */
	init_tcon_osd();
//	tcon_set_time_on_off(video_time_onoff);
	#endif

	/* Init frame buffer information. */
	if (init_fb_info() != 0){
		return -ENOMEM;
	}

	/* Init PGU and YPbPr controller. */
//	init_pgu_ypbpr();

	/* Init software osd. */
	init_software_osd();

	/* Init LCD PWM function. */
//	init_lcd_pwm();

	/* Init Advertisment Info. */
	init_advertisment_info();

	/* Register scalar ISR. */
	scalar.state = SCALAR_IDLE;
	request_irq(SCALAR_IRQ, scalar_isr, SA_INTERRUPT, "Image Scalar", &scalar);

	/* Disable Pixtop Interrupt mask */
	unsigned int reg;
	reg	= read_new_aux_reg(AUX_IENABLE);
	reg &= ~((1 << PIX_IRQ));
	write_new_aux_reg(AUX_IENABLE, reg);

	/* Register PIX ISR. */
	request_irq(PIX_IRQ, pix_isr, SA_INTERRUPT, "Pixel Top", (void*)pgu_ptr);
	memset((void*)&yuv_queue_root, 0x00, sizeof(struct yuv_queue));
	yuv_queue_root.prev = 0;
	yuv_queue_root.next = 0;
	//yuv_queue_root.yuvno = 0;

	/* Register TV ISR. */
	request_irq(YPBPR_IRQ, ypbpr_isr, SA_INTERRUPT, "YPbPr(TV)", (void*)ypbpr_ptr);

	conswitchp = &fb_con;
	if (register_framebuffer(&fb_info.gen.info) < 0)
		return -EINVAL;

	printk(KERN_INFO "fb%d: installed\n",GET_FB_IDX(fb_info.gen.info.node));

	arc_vsync_init(do_switch);

	/* uncomment this if your driver cannot be unloaded */
	/* MOD_INC_USE_COUNT; */
	return 0;
}


/*----------------------------------------------------------------------------*/
void arc_pgufb_cleanup(struct fb_info *info)
{
	arc_vsync_uninit();
	/* Will need to free framebuffer memory if this is ever used as a module */
	pgu_ptr->statctrl = 0;
	unregister_framebuffer(info);
}

/*----------------------------------------------------------------------------*/
void arc_pgufb_setup(char *options, int *ints)
{
	char* this_opt;
	int i;
	int num_displays;
	int x, y;

	x = sizeof(displays);
	y = sizeof(struct known_displays);
	num_displays = x / y;

	printk(KERN_INFO "FB options %s\n", options);

	if (!options || !*options)
		return;

	display_index = 1;
	for(this_opt=strtok(options, ","); this_opt;
			this_opt=strtok(NULL, ",")) {
		if (!strncmp(this_opt, "display:", 6)) {
			/* Get the display name, everything else if fixed */
			for (i=0; i<num_displays; i++) {
				if (!strncmp(this_opt+6, displays[i].display_name,
							strlen(this_opt))) {
					display_index = i;
					break;
				}
			}
		}
		else if (!strncmp(this_opt, "nohwcursor", 10)) {
			printk("nohwcursor\n");
			fb_info.nohwcursor = 1;
		}
	}
}

/*----------------------------------------------------------------------------*/
/*
 *	Flowbee
 */
int BD_draw_number(unsigned char _num);
static void test_func(unsigned int _val)
{
#if 0	// Test the terawins t-con functions.
	test_terawins(_val);
#endif

#if 0
	int _tmp;
	unsigned char i;

	//init_uart1();
	if(_val == 0){
		for(i = 0; i < 100; i ++){
			uart1_write(0x30);
		}
	}
	else if(_val == 1){
		for(i = 0; i < 100; i ++){
			uart1_write(0x33);
		}
	}
	else if(_val == 2){
		for(i = 0; i < 3; i ++){
			do{
				wait_jiffies(1);
				_tmp = uart1_read();
			}while(_tmp == -1);
			uart1_write((char)_tmp);
		}
	}
#endif

#if 0
	unsigned long rgb_size;
	unsigned long tmplong, tmplong1;

	if(_val == 0){
			tmplong = 2;
			//-- Flowbee 2007-11-19	Clear RGB1 and switch to RGB1
			blank_rgb_buf(&(fb_info.yuv), 1);
			stop_display();
			set_rgb_buf(1);

			stop_arcpgu();
			tmplong1 = display_index;
			display_index = tmplong;
			p_current_display = &displays[display_index];
			/* Init display data info. */
			init_disp_parameter();
			/* Init frame buffer information. */
			if (init_fb_info() != 0){
				display_index = tmplong1;
				p_current_display = &displays[display_index];
				/* Init display data info. */
				init_disp_parameter();
				init_fb_info();
				return -ENOMEM;
			}
			blank_yuv(&(fb_info.yuv));
			//blank_yuv_buf(&(fb_info.yuv), 0);
			//blank_yuv_buf(&(fb_info.yuv), 1);
			init_software_osd();
			/* Init PGU and YPbPr controller. */
			down(&pgu_sem);
			while(init_pgu_ypbpr() == -1){
				;
			}
			start_arcpgu();
			up(&pgu_sem);

			//-- Wait 1 sec if change to TV mode.
			if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
				|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
				|| (display_index == DISP_HDTV)){
				wait_jiffies(100);
			}

	}
	else if(_val == 1){
		FB_MSG("\nReset SIMD SRAM !\n");
		VOLATILE_MEMSET(fb_info.fb_virt_start, 0x00, 0x8000);
		//-- Restore SIMD SRAM from RGB0
		dma_in_sdm(31, 0, 128, 128, fb_info.fb_virt_start, 128, 1, 0, 0, 0, 1);
		dma_in_sdm(31, 128, 128, 128, (fb_info.fb_virt_start + 128), 128, 255, 0, 0, 0, 1);
	}

#endif

#if 0	//-- Test blank function.
	FB_MSG("\nTest");
	if(_val == 0){
		FB_MSG("\nRGB");
		blank_rgb_buf(&(fb_info.yuv), 1);
		set_rgb_buf(1);
	}
	else if(_val == 1){
		FB_MSG("\nYUV");
		blank_yuv_buf(&(fb_info.yuv), 0);
		fb_info.yuv.displayed_buffer = 0;
		set_yuv_pointer();
	}

#endif

#if 0	//-- Test NetConfig
	int i = 0;
	char *p;
	if(net_info.flag == 1){
		FB_MSG("\nGet NetConfig");
		p = (char *)net_info.addr;

		for(i = 0; i < net_info.len; i ++){
			if((i % 16) == 0){
				FB_MSG("\n");
			}
			FB_MSG("%02x ", p[i]);
		}
		net_info.flag = 0;
	}
	else{
		FB_MSG("\nNot Yet !!\n");
	}

#endif

#if 0	//-- Test the jpeg update function at advestiment.
	unsigned char i;
	if(_val == 1){
		mplayer_advertisment_flag = 1;
	}
	else if(_val == 2){
	}
	else if(_val == 3){
	}
	else{
		mplayer_advertisment_flag = 0;
	}
	FB_MSG("\nmplayer_advertisment : %d\n", mplayer_advertisment_flag);
#endif

#if 0	//-- Test simd dma function
	unsigned long cb_addr = 0, cr_addr = 0, y_addr = 0;
	int newbuf = fb_info.yuv.displayed_buffer;

	int dma_block_x, dma_block_y, UVdma_block_y;
	int id_x, id_y;
	dma_block_x = 160;
	dma_block_y = 60;//120;//60;//120;
	UVdma_block_y = 60;//90;

	//-- SRC addr
	y_addr = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[newbuf];
	cb_addr = y_addr + pgu_ptr->cru_frm_st;
	cr_addr = y_addr + pgu_ptr->crv_frm_st;
	set_dma_frame(0, y_addr, 800, 800, 600);
	set_dma_frame(1, cb_addr, 400, 400, 300);
	set_dma_frame(2, cr_addr, 400, 400, 300);

	//-- DST addr
	y_addr = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[_val];
	cb_addr = y_addr + pgu_ptr->cru_frm_st;
	cr_addr = y_addr + pgu_ptr->crv_frm_st;
	set_dma_frame(3, y_addr, 800, 800, 600);
	set_dma_frame(4, cb_addr, 400, 400, 300);
	set_dma_frame(5, cr_addr, 400, 400, 300);


	for (id_y = 0; id_y < 600; id_y += dma_block_y)
	{
		for (id_x = 0; id_x < 800; id_x += dma_block_x)
		{
			dma_in_sdm(0, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
			dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
		}

	}

	dma_block_x >>= 1;
	for (id_y = 0; id_y < 300; id_y += UVdma_block_y)
	{
		for (id_x = 0; id_x < 400; id_x += dma_block_x)
		{
			dma_in_sdm(1, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);

			dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);

			dma_in_sdm(2, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);

			dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);

		}
	}



/*
//====================
		//test SIMD DMA
		int dma_block_x, dma_block_y, UVdma_block_y;
		int id_x, id_y;
		dma_block_x = 240;
		dma_block_y = 60;//120;//60;//120;
		UVdma_block_y = 60;//90;
//src
		tmp_addr1 = (unsigned int)yuv_st[0].y + fbmmap2phy_offset;
		tmp_addr2 = (unsigned int)yuv_st[0].u + fbmmap2phy_offset;
		tmp_addr3 = (unsigned int)yuv_st[0].v + fbmmap2phy_offset;
                set_dma_frame(0, tmp_addr1, OUTPUT_XRES, OUTPUT_XRES, OUTPUT_YRES);
                set_dma_frame(1, tmp_addr2, OUTPUT_XRES/2, OUTPUT_XRES/2, OUTPUT_YRES/2);
                set_dma_frame(2, tmp_addr3, OUTPUT_XRES/2, OUTPUT_XRES/2, OUTPUT_YRES/2);
                //dst yuv_des
                tmp_addr1 = (unsigned int)yuv_st[1].y + fbmmap2phy_offset;
                tmp_addr2 = (unsigned int)yuv_st[1].u + fbmmap2phy_offset;
                tmp_addr3 = (unsigned int)yuv_st[1].v + fbmmap2phy_offset;
                set_dma_frame(3, tmp_addr1, OUTPUT_XRES, OUTPUT_XRES, OUTPUT_YRES);
                set_dma_frame(4, tmp_addr2, OUTPUT_XRES/2, OUTPUT_XRES/2, OUTPUT_YRES/2);
                set_dma_frame(5, tmp_addr3, OUTPUT_XRES/2, OUTPUT_XRES/2, OUTPUT_YRES/2);

printk("\nstart!! jiffies=%d\n", jiffies);

//		for (id_y = 0; id_y < 1080; id_y += dma_block_y)
		for (id_y = 0; id_y < 600; id_y += dma_block_y)
		{
//			for (id_x = 0; id_x < 1920; id_x += dma_block_x)
			for (id_x = 0; id_x < 800; id_x += dma_block_x)
			{
//				dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
//				dma_in_sdm(0, 14912, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
				dma_in_sdm(0, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_IN_BUSY_WAIT();
				dma_out_sdm(3, 512, dma_block_x, 0, 0, dma_block_x, dma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_OUT_BUSY_WAIT();
			}

		}

//		for (id_y = 0; id_y < 540; id_y += UVdma_block_y)
		for (id_y = 0; id_y < 300; id_y += UVdma_block_y)
		{
//			for (id_x = 0; id_x < 960; id_x += dma_block_x)
			for (id_x = 0; id_x < 400; id_x += dma_block_x)
			{
				dma_in_sdm(1, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_IN_BUSY_WAIT();

				dma_out_sdm(4, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_OUT_BUSY_WAIT();

				dma_in_sdm(2, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_IN_BUSY_WAIT();

				dma_out_sdm(5, 512, dma_block_x, 0, 0, dma_block_x, UVdma_block_y, id_x, id_y, 0, 0);
//				SIMD_DMA_OUT_BUSY_WAIT();

			}

		}
*/


#endif

#if 0
	FB_MSG("\n\n-- Test number (%d)--\n\n", _val);
	BD_draw_number((unsigned char)_val);
#endif

#if 0	// YPbPr ISR test
	FB_MSG("\nTV ISR test start !\n");
	temp_val = 0;
	//-- Clear status bit
	ypbpr_ptr->disp_ctrl |= (YPBPR_CTRL_REG_FRAME_BODY_READY_MASK
							| YPBPR_CTRL_REG_FRAME_PTR_READY_MASK);
	//-- Enable YPbPr ISR
	ypbpr_ptr->disp_ctrl |= (YPBPR_CTRL_REG_FRAME_BODY_INTEN_MASK
							| YPBPR_CTRL_REG_FRAME_PTR_INTEN_MASK);
	while(temp_val == 0){
		//-- Check YPbPr status bit
		if(ypbpr_ptr->disp_ctrl | (YPBPR_CTRL_REG_FRAME_BODY_READY_MASK | YPBPR_CTRL_REG_FRAME_PTR_READY_MASK))
			;
	}
	FB_MSG("\nTV ISR test done !\n");

#endif

#if 0	// Change display index
    int _old_index;
	int _new_index;
	int _tv_interface = 0;

	_new_index = (int)_val;
    _old_index = display_index;
    display_index = _new_index;
	p_current_display = &displays[display_index];
	/* Init display data info. */
	init_disp_parameter();
	/* Init frame buffer information. */
	if (init_fb_info() != 0){
		display_index = _old_index;
		p_current_display = &displays[display_index];
		/* Init display data info. */
		init_disp_parameter();
		init_fb_info();
		FB_MSG("\nInit FB error !1");
	}
	//--Flowbee 2007-07-31
	if(_tv_interface < SKY_TV_MAX){
		tv_interface = _tv_interface;
	}
	FB_MSG("\n--- disp:%d, intf:%d ---\n", display_index, tv_interface);
	init_software_osd();
	FB_MSG("\n1");
	/* Init PGU and YPbPr controller. */
	down(&pgu_sem);
	stop_arcpgu();
	blank_yuv(&(fb_info.yuv));
	FB_MSG("\n2, jiffies = %08x", jiffies);
	if(init_pgu_ypbpr() == -1){
		FB_MSG("\nSet Error !!");
	}
	FB_MSG("\n3, jiffies = %08x", jiffies);
	start_arcpgu();
	up(&pgu_sem);
	FB_MSG("\n4");
#endif

#if 0
	/* Backlight test. */
	FB_MSG("\ntest BL mask = %08x", BL_MASK);
	if(_val == 0){
		FB_MSG("\nBacklight off !");
		BL_OUT;
		BL_OFF;
	}
	else{
		FB_MSG("\nBacklight on !");
		BL_OUT;
		BL_ON;
	}
#endif


#if 0	// Test the terawins t-con functions.
	test_terawins(_val);
	//init_terawins_tcon();
#endif

#if 0	//Reselect the display index.
	unsigned long tmp1, tmp2 = _val;
	stop_arcpgu();
	tmp1 = display_index;
	display_index = tmp2;
	FB_MSG("\nChange from %d to %d !", tmp1, tmp2);
	p_current_display = &displays[display_index];
	/* Init display data info. */
	init_disp_parameter();
	/* Init frame buffer information. */
	if (init_fb_info() != 0){
		display_index = tmp1;
		FB_MSG("\nRestore to %d !", tmp1);
		p_current_display = &displays[display_index];
		/* Init display data info. */
		init_disp_parameter();
		init_fb_info();
		return -ENOMEM;
	}
	/* Init PGU and YPbPr controller. */
	down(&pgu_sem);
	init_pgu_ypbpr();
	start_arcpgu();
	up(&pgu_sem);
#endif


#if	0//-- Test the terawins T-CON
	if(_val == 1){
	//-- Turn on
		tcon_on_off(1);
//		BL_ON(val);
//		start_arcpgu();
	}
	else if(_val == 0){
	//-- Turn off
		tcon_on_off(0);
//		BL_OFF(val);
//		stop_arcpgu();
	}
	else if(_val == 2){
    	volatile unsigned long flg0 = 0, flg1 = 0;
		unsigned long tmp = 0;
		#if 0
		stop_arcpgu();
		fb_info.yuv.displayed_buffer = 1;
		arc_pgu_setdims(&fb_info.yuv);
		set_yuv_pointer();
		/* Enable the PGU module. */
		start_arcpgu();
		#else
		/* Flowbee@960605. */
		//-- Use the LCD interrupt.
		if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
			/* Enable the interrupt and clear the interrupt flag. */
			pgu_ptr->statctrl |= STATCTRL_INTR_EN_MASK;
			pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
			while((pgu_ptr->statctrl & STATCTRL_INTR_FLAG) == 0){
				;
			}
			//-- disable the LCD module.
			pgu_ptr->statctrl &= ~STATCTRL_REG_DISP_EN_MASK;
			//-- disable the LCD interrupt.
			pgu_ptr->statctrl &= ~(STATCTRL_INTR_EN_MASK);
		}
		/* Check busy flag */
		while( tmp-- > 0x02 ) {
			flg0 = pgu_ptr->statctrl;
			flg1 = (flg0 & STATCTRL_DISP_BUSY_MASK);
			flg0 = (flg0 & STATCTRL_BU_BUSY_MASK);
			if ( (flg0 == 0) && (flg1 == 0) ){
				break;
			}
		}
		arc_pgu_setdims(&fb_info.yuv);
		tmp = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[1];
		pgu_ptr->frm_start = tmp;
		/* Enable the PGU module. */
		pgu_ptr->statctrl |= STATCTRL_REG_DISP_EN_MASK;
		#endif
    }
#endif


#if 0	// TV output format
	unsigned int _format = _val;
	unsigned long tmp = 0;

	if(_format >= DISP_MAX){
		FB_MSG("\nInput error.");
		return;
	}
	stop_arcpgu();
	display_index = _format;
	p_current_display = &displays[display_index];
	FB_MSG("\n%s", p_current_display->display_name);
	/* Init display data info. */
	init_disp_parameter();
	/* Init frame buffer information. */
	if (init_fb_info() != 0){
		return -ENOMEM;
	}
	/* Init PGU and YPbPr controller. */
	down(&pgu_sem);
	init_pgu_ypbpr();
	start_arcpgu();
	up(&pgu_sem);
#endif

#if 0
	printk("\nDisp: %d", display_index);
	display_index = 0;

	printk("\nInput format: %d", _format);
	/* Set TV output format. */
	if((_format < 0) || (_format >= SKY_TV_MAX)){
		printk("\nInput Error!");
		return -EINVAL;
	}
	/* Only TV mode can set TV output format. */
	if ( (display_index != DISP_NTSC) && (display_index != DISP_PAL) ){
		printk("\nNot TV mode.(NTSC, PAL)\n");
		return -EFAULT;
	}
	tv_interface = _format;
	ypbpr_ptr->disp_ctrl &= ~TV_YPBPR_MASK;
	if(tv_interface == SKY_TV_CVBS_SVIDEO){
		ypbpr_ptr->disp_ctrl |= TV_EN;
		tmp = tv_ptr->output_ctrl;
		while (tmp != VOF_CVBS_SVIDEO){
			tv_ptr->output_ctrl = VOF_CVBS_SVIDEO;
			tmp = tv_ptr->output_ctrl;
		}
	}
	else if(tv_interface == SKY_TV_COMPONENT){
		ypbpr_ptr->disp_ctrl |= TV_EN;
		tmp = tv_ptr->output_ctrl;
		while (tmp != VOF_COMPONENT_YUV){
			tv_ptr->output_ctrl = VOF_COMPONENT_YUV;
			tmp = tv_ptr->output_ctrl;
		}
	}
	else if(tv_interface == SKY_TV_YPBPR){
		ypbpr_ptr->disp_ctrl |= YPBPR_EN;
	}
	printk("\nSet OK!");
	return 0;
#endif

#if 0	// test the tv output format (NTSC, PAL).
			tmplong1 = 0x20000000;
			pgu_ptr->statctrl &= ~STATCTRL_REG_DISP_EN_MASK;
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);

			do{
				tmplong = 1;
				tv_ptr->input_ctrl = 0x00000006;
				tmplong = tv_ptr->input_ctrl;
			}while(tmplong != 0x00000006);

			FB_MSG("\n --- TV test Start !! ---\n");

			FB_MSG("\n --- NTSC-M (7.5 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = TVE_NTSC_M;
				tmplong = tv_ptr->fmt_ctrl;
			}while(tmplong != TVE_NTSC_M);

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_NTSC);
			delay(tmplong1);

			FB_MSG("\n --- NTSC-J (0 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = TVE_NTSC_J;
				tmplong = tv_ptr->fmt_ctrl & (TVE_FORMAT_REG_TVF_MASK);
			}while(tmplong != 0);

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_NTSC);
			delay(tmplong1);


			FB_MSG("\n --- PAL-M (7.5 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = (TVE_PAL_M);
				tmplong = tv_ptr->fmt_ctrl;
			}while(tmplong != (TVE_PAL_M));

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_PAL);
			delay(tmplong1);

			FB_MSG("\n --- PAL-N (7.5 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = (TVE_PAL_N);
				tmplong = tv_ptr->fmt_ctrl & (TVE_FORMAT_REG_TVF_MASK);
			}while(tmplong != (TVE_PAL_N));

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_PAL);
			delay(tmplong1);

			FB_MSG("\n --- PAL-Nc (0 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = (TVE_PAL_NC);
				tmplong = tv_ptr->fmt_ctrl & (TVE_FORMAT_REG_TVF_MASK);
			}while(tmplong != (TVE_PAL_NC));

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_PAL);
			delay(tmplong1);

			FB_MSG("\n --- PAL-B,D,G,H,I (0 IRE) ---");
			ypbpr_ptr->disp_ctrl &= ~(TV_YPBPR_MASK|YPBPR_CTRL_REG_ACTIVE_MODE_MASK);
			do{
				tv_ptr->fmt_ctrl = (TVE_PAL_B_D_G_H_I);
				tmplong = tv_ptr->fmt_ctrl & (TVE_FORMAT_REG_TVF_MASK);
			}while(tmplong != (TVE_PAL_B_D_G_H_I));

			ypbpr_ptr->disp_ctrl |= (TV_EN|ACTIVE_MODE_PAL);
			delay(tmplong1);

			FB_MSG("\n --- TV test end !! ---\n");
#endif

#if 0	// turn on overlay.
			unsigned short *rgb_buf;
			rgb_buf = (unsigned short *)pgu_ptr->ol_frm_st;
			/* Set rgb0 overlay enable. */
			for(tmplong1 = 0; tmplong1 < (p_current_display->xres * p_current_display->yres); tmplong1 ++){
				rgb_buf[tmplong1] |= 0x8000;
			}
#endif

#if 0	// software osd test.
			tmplong = ( fb_info.fb_virt_start + (1 * p_current_display->xres * p_current_display->yres * (p_current_display->bpp / 8)) );
			move_yuv(pgu_ptr, pgu_ptr->frm_start, tmplong, pgu_ptr->stride, ((pgu_ptr->frm_dim & 0xFFFF0000) >> 16), 1);
			init_alpha_table(6);
			rgb2yuv_blend(pgu_ptr, pgu_ptr->ol_frm_st, tmplong, pgu_ptr->frm_start, pgu_ptr->stride, ((pgu_ptr->frm_dim & 0xFFFF0000) >> 16));
			start_arcpgu();
#endif
#if 0	// frame address setting (TV)
			ypbpr_ptr->y_addr = tmplong;
			ypbpr_ptr->cb_addr = tmplong + pgu_ptr->cru_frm_st;
			ypbpr_ptr->cr_addr = tmplong + pgu_ptr->crv_frm_st;
#endif
#if 0	// Set color bar test mode.
			tmpint = 0;
			tv_ptr->input_ctrl = 0x00000006;
			result = tv_ptr->input_ctrl;
			while (result != 0x00000006){
				tv_ptr->input_ctrl = 0x00000006;
				result = tv_ptr->input_ctrl;
			}
#endif

}

/*----------------------------------------------------------------------------*/
/*
	Switch User Defined YUV buffer.
*/
int do_switch_user_defined_yuv(struct yuv_info *tmpyuv)
{
//	current_par.main_is_fb = 0;
//	fb_info.yuv.displayed_buffer = fb;
//FB_MSG("tmpyuv->start_offset=0x%x\n", tmpyuv->start_offset);
	return set_user_defined_yuv_pointer(tmpyuv) ? -EFAULT : 0;
}

/*----------------------------------------------------------------------------*/
static unsigned long ir_value = 0;
static int arc_pgufb_ioctl(struct inode *inode, struct file *file, u_int cmd,
				u_long arg, int con, struct fb_info *info)
{
	unsigned long tmplong, tmplong1, tmplong2;
	unsigned int u_int;
	int tmpint, result;
	arc_pgu temp_pgu;
	char _time[31];
	char _sub[3][31];
	int s_tbl[(2*SCALE_TABLE_NUM) + 1];
	struct _osd_line _line;
	struct _terawins_osd_info _osd;
	int * _tbl;

	/* Watch out for calling these with the PGU running. Best policy is to stop PGU, set up PGU, restart it. */
	switch (cmd)
	{
		case ARCPGUFB_GETYUV:
			return copy_to_user((void *) arg, (void*) &(fb_info.yuv), sizeof(struct yuv_info)) ? -EFAULT : 0;

		case ARCPGUFB_GET_YUV_BASE:
			if (arg < 0 || arg > disp_num_buffers)
			    	return -1;
			return fb_info.yuv_base[arg];

		case ARCPGUFB_SETYUV:
			result = copy_from_user(&tmpyuv, (void *) arg, sizeof(unsigned long)) ? -EFAULT : 0;
			if (! result)
			{
				if (tmpyuv.alignment % BASIC_ALIGNMENT){
					return -EFAULT;
				}
				//down(&pgu_sem);
				/* The user program should stop the pgu function.*/
				memcpy(&fb_info.yuv, &tmpyuv, sizeof(struct yuv_info));
				arc_pgu_setdims(&(fb_info.yuv));
				set_yuv_pointer();
				for (tmpint = 0; tmpint < 1000; tmpint++){;}
				//up(&pgu_sem);
			}
			return result;

		case ARCPGUFB_FILLYUV:
			result = copy_from_user(&tmpyuv, (void *) arg, sizeof(unsigned long)) ? -EFAULT : 0;
			if (! result)
			{
				calculate_yuv_offsets(&tmpyuv);
				result = copy_to_user((void *) arg, (void*) &(tmpyuv), sizeof(struct yuv_info)) ? -EFAULT : 0;
			}
			return result;

		//-- Flowbee@2008-02-07 Set yuv buffer in video mode which res. can bigger
		//--					than p_current_display->xres/p_current_display->yres
		case ARCPGUFB_SET_VIDEO_YUV_RES:
			//-- Flowbee 2008-07-10 Check video resolution
			the_same_resolution = 1;
			//-- Flowbee 2008-05-14 mplayer in the init state
			is_video_play = 0;
			if (copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info))){
				return -EFAULT;
			}
			FB_MSG("\n *** SET_VIDEO_YUV_RES: w=%d, h=%d ***\n", tmpyuv.width, tmpyuv.height);
            FB_MSG("ARCPGUFB_SET_VIDEO_YUV_RES pid[%d]\n", (int)current->pid);

			if( (tmpyuv.width > p_current_display->xres)
				|| (tmpyuv.height > p_current_display->yres)){
				printk("\nInput Size Error: In(x,y)=(%d,%d) Out(x,y)=(%d,%d)",
					tmpyuv.width, tmpyuv.height, p_current_display->xres, p_current_display->yres);
				return -EINVAL;
			}
			//--Reset timer, display queue and done queue.
			//reset_mplayer_timer_queue();
			/* Flowbee@960519. */
			is_video_scale = 0;
			mplayer_advertisment_flag = 0;
			update_advertisment_frame_cnt = 0;
			update_advertisment_frame = 0;

			//-- Flowbee 2007-11-19
			blank_rgb_buf(&(fb_info.yuv), 1);
			stop_display();
			set_rgb_buf(1);

			fb_info.yuv.width = tmpyuv.width;
			fb_info.yuv.height = tmpyuv.height;

			if (fb_info.yuv.width & 0xf){
				fb_info.yuv.width = ((fb_info.yuv.width + 0xf) & ~0xf);
			}

			/* Flowbee@960427. */
			if (fb_info.yuv.height & 0xf){
				fb_info.yuv.height = ((fb_info.yuv.height + 0xf) & ~0xf);
			}

			FB_MSG("\nBUF: x=%d, y=%d", fb_info.yuv.width, fb_info.yuv.height);
			if (calculate_yuv_offsets(&(fb_info.yuv))){
				return -EFAULT;
			}

			blank_yuv_buf(&(fb_info.yuv), 0);
			fb_info.yuv.width = tmpyuv.width & (~0x000f);
			fb_info.yuv.height = tmpyuv.height;

			//-- Check display dimension.
			if(arc_pgu_check_dims(&(fb_info.yuv)) == -1){
				// Switch to yuv 0
				fb_info.yuv.displayed_buffer = 0;
				set_yuv_buf(0);
				//-- Disable OSD
				turn_all_osd_off();
				is_video_play = 1;
				start_display();
				FB_MSG("\n The same resolution. \n");
				return 0;
			}
			the_same_resolution = 0;
			
			//-- Flowbee@960525 TeraWin T_CON check.
			#if CONFIG_TERAWINS_TCON
			tcon_parse_resolution(fb_info.yuv.width, fb_info.yuv.height);
			#endif

			stop_display();
			// Switch to yuv 0
			fb_info.yuv.displayed_buffer = 0;
			set_yuv_buf(0);
			//-- Set dimension
			arc_pgu_setdims(&(fb_info.yuv));
			//-- Flowbee 2007-11-19 Set center.
			set_center();
			//-- Flowbee set overlay stride.
			ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
			ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
			pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);
			//-- Disable OSD
			turn_all_osd_off();
			start_display();

			is_video_play = 1;
			return 0;

		case ARCPGUFB_SET_YUV_RES:
			//-- Flowbee 2008-07-09 test video resolution un-change.
			if(do_mplayer_uninit == 0){
				is_video_play = 0;
				reset_mplayer_timer_queue();
				/* Flowbee@960519. */
				is_video_scale = 0;
				mplayer_advertisment_flag = 0;
				update_advertisment_frame_cnt = 0;
				update_advertisment_frame = 0;
				FB_MSG("\n ARCPGUFB_SET_YUV_RES: not change display dimension. \n");
				blank_yuv_buf(&(fb_info.yuv), 0);
				// Switch to yuv 0
				fb_info.yuv.displayed_buffer = 0;
				set_yuv_buf(0);
				blank_yuv_buf(&(fb_info.yuv), 1);
				return 0;
			}
			// Flowbee 2008-10-17 advertisment not clear buffer
			if ( mplayer_advertisment_flag == 1 ) {
				is_video_play = 0;
				reset_mplayer_timer_queue();
				/* Flowbee@960519. */
				is_video_scale = 0;
				mplayer_advertisment_flag = 0;
				update_advertisment_frame_cnt = 0;
				update_advertisment_frame = 0;
				FB_MSG("\n ARCPGUFB_SET_YUV_RES: not change display dimension. \n");
				if (adver_video_info.clr_buf == 1) {
					blank_yuv_buf(&(fb_info.yuv), 0);
				}
				// Switch to yuv 0
				fb_info.yuv.displayed_buffer = 0;
				set_yuv_buf(0);				
				return 0;
			}
			//-- Flowbee 2008-05-14 Need clear parameters first
			//-- Reset timer, display queue and done queue.
			//-- Flowbee 2008-05-14 mplayer in the init state
			is_video_play = 0;
			reset_mplayer_timer_queue();
			/* Flowbee@960519. */
			is_video_scale = 0;
			mplayer_advertisment_flag = 0;
			update_advertisment_frame_cnt = 0;
			update_advertisment_frame = 0;

			if (copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info))){
				return -EFAULT;
			}
			FB_MSG("\n *** SET_YUV_RES: w=%ld, h=%ld ***\n", tmpyuv.width, tmpyuv.height);
            FB_MSG("ARCPGUFB_SET_YUV_RES pid[%d]\n", (int)current->pid);

			if( (tmpyuv.width > p_current_display->xres)
				|| (tmpyuv.height > p_current_display->yres)){
				printk("\nInput Size Error: In(x,y)=(%ld,%ld) Out(x,y)=(%ld,%ld)",
					tmpyuv.width, tmpyuv.height, p_current_display->xres, p_current_display->yres);
				return -EINVAL;
			}

			//-- Flowbee 2007-11-19
			blank_rgb_buf(&(fb_info.yuv), 1);
			stop_display();
			set_rgb_buf(1);

			fb_info.yuv.width = tmpyuv.width;
			fb_info.yuv.height = tmpyuv.height;

			if (fb_info.yuv.width & 0xf){
				fb_info.yuv.width = ((fb_info.yuv.width + 0xf) & ~0xf);
			}

			/* Flowbee@960427. */
			if (fb_info.yuv.height & 0xf){
				fb_info.yuv.height = ((fb_info.yuv.height + 0xf) & ~0xf);
				if(fb_info.yuv.height > p_current_display->yres){
					fb_info.yuv.height = p_current_display->yres;
				}
			}

			/* Flowbee@960525-TeraWin T_CON check. */
			#if CONFIG_TERAWINS_TCON
			tcon_parse_resolution(fb_info.yuv.width, fb_info.yuv.height);
			#endif

			FB_MSG("\nBUF: x=%ld, y=%ld", fb_info.yuv.width, fb_info.yuv.height);
			if (calculate_yuv_offsets(&(fb_info.yuv))){
				return -EFAULT;
			}

			#if 0	// Flowbee 2007-11-19 change dim without turn off TV.
			fb_info.yuv.displayed_buffer = 0;
			//-- clear the yuv buffer and rgb buffer 0.
			blank_yuv(&(fb_info.yuv));

			/* Flowbee@960427. + */
			fb_info.yuv.width = tmpyuv.width & (~0x000f);
			fb_info.yuv.height = tmpyuv.height;
			/* Flowbee@960427. - */
			down(&pgu_sem);
			arc_pgu_setdims(&(fb_info.yuv));
			//FB_MSG("\nSwitch to YUV buf\n");
			set_yuv_pointer();
			/* Flowbee set overlay stride. + */
			ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
			ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
			pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);
			/* Flowbee set overlay stride. - */
			up(&pgu_sem);
			#else

			blank_yuv_buf(&(fb_info.yuv), 0);
			fb_info.yuv.width = tmpyuv.width & (~0x000f);
			fb_info.yuv.height = tmpyuv.height;

			//down(&pgu_sem);
			stop_display();
			// Switch to yuv 0
			fb_info.yuv.displayed_buffer = 0;
			set_yuv_buf(0);
			//-- Set dimension
			arc_pgu_setdims(&(fb_info.yuv));
			//-- Flowbee 2007-11-19 Set center.
			set_center();
			//-- Flowbee set overlay stride.
			ypbpr_ptr->rgb_stride &= ~(0x0000FFFF);
			ypbpr_ptr->rgb_stride |= (fb_info.yuv.y_stride * 2);
			pgu_ptr->ol_stride = (fb_info.yuv.y_stride * 2);
			//-- Disable OSD
			turn_all_osd_off();	// Flowbee 2008-02-21
			start_display();
			//-- Flowbee 2008-04-11 UI will set to yuv 1, we need to clear it.
			blank_yuv_buf(&(fb_info.yuv), 1);
			//up(&pgu_sem);

			//-- Flowbee 2007-11-19
			//blank_rgb_buf(&(fb_info.yuv), 1);
			//set_rgb_buf(1);

			#endif
			//FB_MSG("\nEnd of Reset YUV resolution\n");
			return 0;

		case ARCPGUFB_RESET://Flowbee@951128
			memcpy(&fb_info.yuv, &fb_info.default_yuv, sizeof(struct yuv_info));
			fb_info.yuv.displayed_buffer = 0;
			//down(&pgu_sem);
			stop_arcpgu();
			init_pgu_ypbpr();
			start_arcpgu();
			//up(&pgu_sem);

			return 0;

		case ARCPGUFB_SWITCH_YUV:
			/* Don't stop the controller for this */
			//Flowbee@951201
			#if 1	//-- old
			if (! copy_from_user(&tmplong, (void *) arg, sizeof(unsigned long)))
				return do_switch((int)tmplong);
			return -EFAULT;
			#else	//-- new
			if (! copy_from_user(&tmplong, (void *) arg, sizeof(unsigned long))){
				if(result == fb_info.yuv.displayed_buffer){
					return 0;
				}
				return do_switch((int)result);
			}
			return -EFAULT;
			#endif
#if 0
		case ARCPGUFB_SWITCH_USER_DEFINED_YUV:
			/* Don't stop the controller for this */
			//Flowbee@951201
			if (! copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info)))
				return do_switch_user_defined_yuv(&tmpyuv);
			return -EFAULT;
#else
		case ARCPGUFB_SWITCH_USER_DEFINED_YUV:
			if (! copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info)))
			{
			  if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
                	    || (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
                	    || (display_index == DISP_HDTV))
                          {
				return do_switch_user_defined_yuv(&tmpyuv);
                          }
                          else
                          {
			    enqueue_user_defined_yuv(&tmpyuv);

                            return 0;
                          }
                        }
			return -EFAULT;

#endif

		case ARCPGUFB_BKLGHT_OFF:
			#if CONFIG_TERAWINS_TCON
				#if CONFIG_FASHION_STAR || CONFIG_SKY_IO_FOXCONN_COMPANY_V2_LCD
				backlight_on_off(0);
				//-- Turn off pwm(backlight).
				tcon_on_off(0);
				tcon_pwm_on_off(0);
				#else
				//-- Turn off pwm(backlight).
				tcon_on_off(0);
				tcon_pwm_on_off(0);
				#endif
				tcon_backlight_on_off(0);
			#else
			backlight_on_off(0);
			#endif
			return 0;

		case ARCPGUFB_BKLGHT_ON:
			#if CONFIG_TERAWINS_TCON
				#if CONFIG_FASHION_STAR || CONFIG_SKY_IO_FOXCONN_COMPANY_V2_LCD
				backlight_on_off(1);
				//-- Enable terawins output data.
				tcon_on_off(1);
				//-- Turn on pwm(backlight).
				tcon_pwm_on_off(1);
				#else
				//-- Enable terawins output data.
				tcon_on_off(1);
				//-- Turn on pwm(backlight).
				tcon_pwm_on_off(1);
				#endif
				tcon_backlight_on_off(1);
			#else
			backlight_on_off(1);
			#endif
			return 0;

		case ARCPGUFB_START_DISPLAY:
			start_arcpgu();
			return 0;

		case ARCPGUFB_STOP_DISPLAY:
			stop_arcpgu();
			return 0;

		case ARCPGUFB_GET_PGU_STATE:
			/*			return copy_to_user((void *) arg, pgu_ptr, sizeof(arc_pgu)) ? -EFAULT : 0; */
			//FB_MSG("\n Get PGU");
			copy_pgu(pgu_ptr,&temp_pgu);
			return copy_to_user((void *) arg, (void*) &temp_pgu, sizeof(arc_pgu)) ? -EFAULT : 0;

		case ARCPGUFB_SET_PGU_STATE:
			/* The user program should stop the pgu function.*/
			result = copy_from_user((void *)&temp_pgu, (void *) arg, sizeof(arc_pgu)) ? -EFAULT : 0;
			if (!result){
				copy_pgu(&temp_pgu, pgu_ptr);
				/* FIXME Can't memcpy when registers declared in the structure don't exist in the controller */
				/*				memcpy((void*) pgu_ptr,(void*) &temp_pgu,sizeof(arc_pgu)); */
				set_ypbpr_tve_regs();
			}
			return result;

		case ARCPGUFB_GET_FB_INFO:
			return copy_to_user((void *) arg, &fb_info, sizeof(struct arc_pgufb_info)) ? -EFAULT : 0;

		case ARCPGUFB_GET_FB_SIZE:
			return copy_to_user((void *)arg, (void*)&fb_info.fb_size, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_GET_MAIN_MODE:
			tmplong = pgu_ptr->statctrl & STATCTRL_PIX_FMT_MASK;
			//FB_MSG("\nGet main mode: result is %x\n", result);
			return copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_MAIN_MODE:
			if (! copy_from_user((void *) &tmplong, (void *) arg, sizeof(unsigned long))){

				//down(&pgu_sem);
				current_par.main_mode = tmplong & STATCTRL_PIX_FMT_MASK;
				pgu_ptr->statctrl &= ~STATCTRL_PIX_FMT_MASK;
				pgu_ptr->statctrl |=	current_par.main_mode;
				if(current_par.main_mode == STATCTRL_PIX_FMT_RGB555){
					pgu_ptr->stride = p_current_display->xres * 2;
				}
				else{
					pgu_ptr->stride = p_current_display->xres;
				}
				for (tmpint = 0; tmpint < 1000; tmpint++){;}
				//FB_MSG("\n SET_MAIN_MODE: pgu_ptr->statctrl(%08lx)", pgu_ptr->statctrl);
				//FB_MSG("\n SET_MAIN_MODE: pgu_ptr->stride(%08lx)", pgu_ptr->stride);
				//up(&pgu_sem);
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_SET_RGB_BUF:	// Blank RGB buffer and switch to RGB buffer.
			#if 0	//-- Flowbee 2007-11-21
			if (! copy_from_user(&tmplong, (void *) arg, sizeof(unsigned long)))
			{
				current_par.rgb_bufno = tmplong;
				blank_rgb_buf(&(fb_info.yuv), (unsigned char)tmplong);
				set_rgb_buf((unsigned char)tmplong);
				return 0;
			}
			return -EFAULT;
			#else
			if (! copy_from_user(&tmplong, (void *) arg, sizeof(unsigned long)))
			{
				tmplong1 = pgu_ptr->statctrl;
				tmplong1 &= STATCTRL_PIX_FMT_MASK;
				if(tmplong1 == STATCTRL_PIX_FMT_RGB555){
					//-- RGB mode in LCD
					current_par.main_is_fb = 1;
					current_par.rgb_bufno = tmplong;
					result = set_rgb_pointer() ? -EFAULT : 0;
					return result;
				}
				else{
					current_par.rgb_bufno = tmplong;
					if (current_par.rgb_bufno > CONFIG_ARCPGU_RGBBUFS){
						return -1;
					}
					if ( (mplayer_advertisment_flag == 1) && (adver_video_info.clr_buf == 0) ) {
						return 0;
					}
					blank_rgb_buf(&(fb_info.yuv), (unsigned char)tmplong);
					set_rgb_buf((unsigned char)tmplong);
					return 0;
				}
			}
			return -EFAULT;
			#endif

		case ARCPGUFB_GET_RGB_BUF:
			return copy_to_user((void *) arg, (void*) &current_par.rgb_bufno, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_GET_FB_MAIN:
			return copy_to_user((void *) arg, (void*) &current_par.main_is_fb, sizeof(unsigned long)) ? -EFAULT : 0;

		/* Set LCD backlight. */
		case ARCPGUFB_SET_PWM:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				tcon_brightness = (unsigned char)tmplong;
				#if CONFIG_TERAWINS_TCON				
				#if CONFIG_FASHION_STAR
				tcon_set_brightness(tcon_brightness);
				//tcon_set_pwm((unsigned char)tmplong);
				#else
				tcon_set_pwm(tcon_brightness);
				#endif				
				#endif				
				return 0;
			}
			return -EFAULT;

		/* Set Contrast. */
		case ARCPGUFB_SET_CONTRAST:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				tcon_contrast = (unsigned char)tmplong;
				#if CONFIG_TERAWINS_TCON
				tcon_set_contrast(tcon_contrast);
				#endif
				return 0;
			}
			return -EFAULT;

		/* OSD */
		case ARCPGUFB_SET_OL_MODE:
			if (! copy_from_user(&tmplong, (void *) arg, sizeof(unsigned long)))
			{
				//down(&pgu_sem);
				tmplong &= STATCTRL_OL_FMT_MASK;
				pgu_ptr->statctrl &= ~STATCTRL_OL_FMT_MASK;
				pgu_ptr->statctrl |= tmplong;
				ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_OL_FMT_MASK);
				ypbpr_ptr->disp_ctrl |= (tmplong >> 13);
				for (tmpint = 0; tmpint < 1000; tmpint++){;}
				current_par.overlay_mode = tmplong;
				//up(&pgu_sem);
			}
			return -EFAULT;

		case ARCPGUFB_GET_OL_MODE:
			tmplong = (pgu_ptr->statctrl & STATCTRL_OL_FMT_MASK);
			return copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned int)) ? -EFAULT : 0;

		case ARCPGUFB_GET_OL_COLKEY:
			return copy_to_user((void *) arg, (void*) &pgu_ptr->ol_col_key, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_OL_COLKEY:
			result = copy_from_user((void*) &(tmplong), (void *) arg, sizeof(unsigned long)) ? -EFAULT : 0;
			if (! result){
				//down(&pgu_sem);
				pgu_ptr->ol_col_key = tmplong;
				//up(&pgu_sem);
			}
			return result;

		case ARCPGUFB_GET_OL_START:
			tmplong = (pgu_ptr->ol_scn_line & DISP_DIM_REG_PGU_X_RES_MASK);
			return copy_to_user((void *) arg, &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_OL_START:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long)))	{
				tmplong &= DISP_DIM_REG_PGU_X_RES_MASK;
				//down(&pgu_sem);
				pgu_ptr->ol_scn_line &= ~DISP_DIM_REG_PGU_X_RES_MASK;
				pgu_ptr->ol_scn_line |= tmplong;
				ypbpr_ptr->rgb_range = pgu_ptr->ol_scn_line;
				//up(&pgu_sem);
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_GET_OL_END:
			tmplong = (pgu_ptr->ol_scn_line & DISP_DIM_REG_PGU_Y_RES_MASK) >> DISP_DIM_Y_SHIFT;
			return copy_to_user((void *) arg, &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_OL_END:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long)))	{
				tmplong &= DISP_DIM_REG_PGU_X_RES_MASK;
				tmplong <<= DISP_DIM_Y_SHIFT;
				//down(&pgu_sem);
				pgu_ptr->ol_scn_line &= ~DISP_DIM_REG_PGU_Y_RES_MASK;
				pgu_ptr->ol_scn_line |= tmplong;
				ypbpr_ptr->rgb_range = pgu_ptr->ol_scn_line;
				//up(&pgu_sem);
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_SET_OL_OFFSET:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long)))	{
				//down(&pgu_sem);
				pgu_ptr->ol_frm_st = fb_info.fb_virt_start + tmplong;
				ypbpr_ptr->rgb_addr = pgu_ptr->ol_frm_st;
				//up(&pgu_sem);
				return 0;
			}
			return -EFAULT;

		/* Flowbee@960102: software osd. */
		case ARCPGUFB_UPDATE_OSD:
			if (sw_osd_flag == FB_TRUE){
				/* Get osd info. */
				if (copy_from_user((void *)&osd_info, (void *) arg, sizeof(struct _osd_info))){
					return -EFAULT;
				}
				update_osd();
			}
			return 0;

		case ARCPGUFB_ENABLE_OL:
			start_overlay();
			return 0;

		case ARCPGUFB_DISABLE_OL:
			stop_overlay();
			return 0;

		case ARCPGUFB_ENABLE_TV_OSD:
			ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
			return 0;

		case ARCPGUFB_DISABLE_TV_OSD:
			ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_OL_EN_MASK);
			return 0;

		case ARCPGUFB_GET_OL_ALPHA:
			tmplong = (unsigned long)alpha_val;
			return copy_to_user((void *) arg, &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_OL_ALPHA:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long)))	{
				tmplong = (tmplong > 15)? 15: tmplong;
				alpha_val = (unsigned char)tmplong;
				//-- Alpha value of software osd.
				init_alpha_table(alpha_val);
				tmplong1 = (15 - tmplong);
				//FB_MSG("\nAlpha value: %08x (%08x), %08x (%08x)\n", tmplong, pgu_ptr->statctrl, tmplong1, ypbpr_ptr->rgb_stride);
				//-- Alpha value of LCD controller(Hardware osd)
				tmplong <<= STATCTRL_OL_VAL_SHIFT;
				tmplong &= STATCTRL_OL_VAL_MASK;
				//down(&pgu_sem);
				pgu_ptr->statctrl &= ~(STATCTRL_OL_VAL_MASK);
				pgu_ptr->statctrl |= tmplong;
				//-- Alpha value of TV controller(Hardware osd)
				tmplong1 <<= YPBPR_ALPHA_SHIFT;
				tmplong1 &= YPBPR_OL_LINE_REG_ALPHA_MASK;
				ypbpr_ptr->rgb_stride &= ~(YPBPR_OL_LINE_REG_ALPHA_MASK);
				tmplong2 = (ypbpr_ptr->rgb_stride & YPBPR_OL_LINE_REG_ALPHA_MASK);
				while(tmplong2 != 0){
					ypbpr_ptr->rgb_stride &= ~(YPBPR_OL_LINE_REG_ALPHA_MASK);
					tmplong2 = (ypbpr_ptr->rgb_stride & YPBPR_OL_LINE_REG_ALPHA_MASK);
				}
				//FB_MSG("\nypbpr_ptr->rgb_stride = %08x\n", ypbpr_ptr->rgb_stride);
				ypbpr_ptr->rgb_stride |= tmplong1;
				tmplong2 = (ypbpr_ptr->rgb_stride & YPBPR_OL_LINE_REG_ALPHA_MASK);
				while(tmplong2 != tmplong1){
					ypbpr_ptr->rgb_stride |= tmplong1;
					tmplong2 = (ypbpr_ptr->rgb_stride & YPBPR_OL_LINE_REG_ALPHA_MASK);
				}
				//up(&pgu_sem);
				//FB_MSG("\nAlpha value: %08x (%08x), %08x (%08x)\n", tmplong, pgu_ptr->statctrl, tmplong1, ypbpr_ptr->rgb_stride);
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_DISABLE_MUTE_VOL_OSD:
			//ypbpr_ptr->disp_ctrl &= ~(YPBPR_CTRL_REG_OL_EN_MASK);
			#if CONFIG_TERAWINS_TCON
			tcon_clear_mute_vol();
			#endif
			return 0;

		/* Flowbee@960703 - Use terawins osd function to display mute, vol and stop. */
		case ARCPGUFB_SHOW_MUTE_VOL:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				if( (display_index == DISP_NTSC) || (display_index == DISP_HDTV) || (display_index == DISP_PAL) ){
					//ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
				}
				else{
					#if CONFIG_TERAWINS_TCON
					tcon_show_mute_vol(tmplong);
					#endif
				}
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_CLEAR_MUTE_VOL:
			#if CONFIG_TERAWINS_TCON
			tcon_clear_mute_vol();
			#endif
			return 0;

		case ARCPGUFB_SHOW_ICON:
			//FB_MSG("\nShow ICON !!\n");
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				if( (display_index == DISP_NTSC) || (display_index == DISP_HDTV) || (display_index == DISP_PAL) ){
					//ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
				}
				else{
					#if CONFIG_TERAWINS_TCON
					tcon_show_icon(tmplong);
					#endif
				}
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_CLEAR_ICON:
			//FB_MSG("\nClear ICON !!\n");
			#if CONFIG_TERAWINS_TCON
			tcon_clear_icon();
			#endif
			return 0;

		case ARCPGUFB_SHOW_VIDEO_TIME:
			//FB_MSG("\nShow time !!\n");
			if (! copy_from_user((void*)&(_time[0]), (void *)arg, sizeof(char [31]))){
				if( (display_index == DISP_NTSC) || (display_index == DISP_HDTV) || (display_index == DISP_PAL) ){
					//ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
				}
				else{
					#if CONFIG_TERAWINS_TCON
					tcon_show_time(_time);
					#endif
				}
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_CLEAR_VIDEO_TIME:
			//FB_MSG("\nClear Mplayer time !!\n");
			#if CONFIG_TERAWINS_TCON
			tcon_clear_time();
			#endif
			return 0;

		case ARCPGUFB_SHOW_VIDEO_SPEED:
			//FB_MSG("\nShow Mplayer time !!\n");
			if (! copy_from_user((void*)&(_time[0]), (void *)arg, sizeof(char [31]))){
				if( (display_index == DISP_NTSC) || (display_index == DISP_HDTV) || (display_index == DISP_PAL) ){
					//ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
				}
				else{
					#if CONFIG_TERAWINS_TCON
					tcon_show_speed(_time);
					#endif
				}
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_CLEAR_VIDEO_SPEED:
			//FB_MSG("\nClear Mplayer time !!\n");
			#if CONFIG_TERAWINS_TCON
			tcon_clear_speed();
			#endif
			return 0;

		case ARCPGUFB_SHOW_VIDEO_SUBTITLE:
			//FB_MSG("\nShow Mplayer subtile !!\n");
			if (! copy_from_user((void*)&(_sub[0][0]), (void *)arg, sizeof(char [3][31]))){
				if( (display_index == DISP_NTSC) || (display_index == DISP_HDTV) || (display_index == DISP_PAL) ){
					//ypbpr_ptr->disp_ctrl |= YPBPR_CTRL_REG_OL_EN_MASK;
				}
				else{
					#if CONFIG_TERAWINS_TCON
					tcon_show_subtitle(_sub);
					#endif
				}
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_CLEAR_VIDEO_SUBTITLE:
			//FB_MSG("\nClear Mplayer subtile !!\n");
			#if CONFIG_TERAWINS_TCON
			tcon_clear_subtitle();
			#endif
			return 0;

		case ARCPGUFB_CLEAR_LCD_OSD:
			//FB_MSG("\nClear LCD OSD !!\n");
			#if CONFIG_TERAWINS_TCON
			tcon_clear_all_osd_item();
			#endif
			return 0;

		case ARCPGUFB_GET_VIDEO_TIME_ONOFF:
			tmplong = video_time_onoff;
			//FB_MSG("gvt : %ld, %ld", tmplong, video_time_onoff);
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_VIDEO_TIME_ONOFF:
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				video_time_onoff = tmplong;
				//FB_MSG("svt : %ld, %ld", tmplong, video_time_onoff);
				#if CONFIG_TERAWINS_TCON
				tcon_set_time_on_off(video_time_onoff);
				#endif
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_GET_ORIGIN:
			tmplong = pgu_ptr->centreoffsets;
			return copy_to_user((void *) arg, &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_ORIGIN:
			//FB_MSG("\n\nCENTER !\n\n");
			return 0;
			if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long)))	{
				//down(&pgu_sem);
				pgu_ptr->centreoffsets = tmplong;
				//up(&pgu_sem);
				return 0;
			}
			return -EFAULT;

		case ARCPGUFB_GET_MPLAYER_PLAY_TIME:
			tmplong = mplayer_play_time;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_MPLAYER_PLAY_TIME:
			result = copy_from_user((void*) &(mplayer_play_time), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_MPLAYER_DURATION:
			tmplong = mplayer_duration;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_MPLAYER_DURATION:
			result = copy_from_user((void*) &(mplayer_duration), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		//Barry 2008-12-09
		case ARCPGUFB_GET_VIDEO_START_TIME:
			tmplong = video_start_time;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_VIDEO_START_TIME:
			result = copy_from_user((void*) &(video_start_time), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_VIDEO_END_TIME:
			tmplong = video_end_time;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_VIDEO_END_TIME:
			result = copy_from_user((void*) &(video_end_time), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_MPLAYER_FLAG:
			tmplong = mplayer_flag;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_MPLAYER_FLAG:
			result = copy_from_user((void*) &(mplayer_flag), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;
		// end Barry 2008-12-09

		case ARCPGUFB_GET_MPLAYER_STATUS:
			tmpint = mplayer_status;
			//FB_MSG("\n ARCPGUFB_GET_MPLAYER_STATUS: %d \n", mplayer_status);
			result = copy_to_user((void *) arg, (void*) &tmpint, sizeof(int)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_MPLAYER_STATUS:
			result = copy_from_user((void*) &(mplayer_status), (void *)arg, sizeof(int)) ? -EFAULT : 0;
			//FB_MSG("\n ARCPGUFB_SET_MPLAYER_STATUS: %d \n", mplayer_status);
			return result;

		case ARCPGUFB_CHECK_VIDEO_PAUSE:
			tmpint = check_video_paused();
			if(tmpint == 1){
				//-- Move the data in display buffer to RGB1, the osd data is stored in RGB0
				//move_video_to_rgb(1);
			}
			result = copy_to_user((void *)arg, (void*) &tmpint, sizeof(int)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_IRVALUE:
			//FB_MSG("\nGET_IR: %x", ir_value);
			//-- Check if video data is backup.
			tmplong = ir_value;
			ir_value = 0;	// read to clear
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_IRVALUE:
			result = copy_from_user((void*) &(ir_value), (void *)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		/* Flowbee@960131: Change display mode between LCD, TV. */
		case ARCPGUFB_GET_DISP_INDEX:
			tmplong = (unsigned long) display_index;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		//== Flowbee 2007-11-07
		case ARCPGUFB_GET_DEFAULT_DISP_INDEX:
			tmpyuv.num_buffers = CONFIG_ARCPGU_DISPTYPE;			// display index
			tmpyuv.width = displays[CONFIG_ARCPGU_DISPTYPE].xres;	// width
			tmpyuv.height = displays[CONFIG_ARCPGU_DISPTYPE].yres;	// height
			return copy_to_user((void *) arg, (void*) &(tmpyuv), sizeof(struct yuv_info)) ? -EFAULT : 0;

		case ARCPGUFB_SET_DISP_INDEX:
			if (copy_from_user((void*) &(tmplong), (void *)arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			//-- Flowbee 2008-0523 TV software power on.
			// Get on/off flag.
			tmpint = (tmplong & (1 << 16));
			tmplong = tmplong & 0xff;
			if (tmplong >= DISP_MAX){
				return -EINVAL;
			}
			//--Reset timer, display queue and done queue.
			reset_mplayer_timer_queue();
			/* Flowbee@960519. */
			is_video_scale = 0;
			mplayer_advertisment_flag = 0;
			update_advertisment_frame_cnt = 0;
			update_advertisment_frame = 0;

			//-- Flowbee 2007-11-19	Clear RGB1 and switch to RGB1
			blank_rgb_buf(&(fb_info.yuv), 1);
			stop_display();
			set_rgb_buf(1);

			stop_arcpgu();
			//-- tmplong is new display index, and tmplong is old display index.
			tmplong1 = display_index;
			display_index = tmplong;
			p_current_display = &displays[display_index];
			/* Init display data info. */
			init_disp_parameter();
			/* Init frame buffer information. */
			if (init_fb_info() != 0){
				display_index = tmplong1;
				p_current_display = &displays[display_index];
				/* Init display data info. */
				init_disp_parameter();
				init_fb_info();
				return -ENOMEM;
			}
			//blank_yuv(&(fb_info.yuv));
			blank_yuv_buf(&(fb_info.yuv), 0);
			blank_yuv_buf(&(fb_info.yuv), 1);
			init_software_osd();
			/* Init Advertisment Info. */
			init_advertisment_info();
			/* Init PGU and YPbPr controller. */
			//down(&pgu_sem);
			while(init_pgu_ypbpr() == -1){
				if(display_index == DISP_PAL){
					// backup tv_interface and then set to HDTV
					tmplong2 = tv_interface;
					display_index = DISP_HDTV;
					init_pgu_ypbpr();
					wait_jiffies(10);
					// restore tv_interface and set to PAL.
					display_index = tmplong;
					tv_interface = tmplong2;
				}
			}
			FB_MSG("\n--- index: %d, interface: %d ---\n", display_index, tv_interface);
			if(tmpint != 0){
				start_arcpgu();
			}
			//up(&pgu_sem);

			//-- Wait 1 sec if change to TV mode.
			if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
				|| (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
				|| (display_index == DISP_HDTV)){
				wait_jiffies(100);
			}
			return 0;

		/* Flowbee@960511: Change TV output format. */
		case ARCPGUFB_GET_TV_INTERFACE:
			tmplong = (unsigned long) tv_interface;
			result = copy_to_user((void *) arg, (void*) &tmplong, sizeof(unsigned long)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_TV_INTERFACE:
			if (copy_from_user((void*) &(tmplong), (void *)arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			if((tmplong < 0) || (tmplong >= SKY_TV_MAX)){
				return -EINVAL;
			}
			tv_interface = tmplong;
			//set_tv_output_interface();
			return 0;

		/* Flowbee@960131: For Raymond mplayer. */	//-- (+)
		case ARCPGUFB_GET_DEFAULT_YUV:
			tmpyuv.width  = p_current_display->xres;
			tmpyuv.height = p_current_display->yres;
			return copy_to_user((void *) arg, (void*) &(tmpyuv), sizeof(struct yuv_info)) ? -EFAULT : 0;

		case ARCPGUFB_GET_DEC_YUV:
			return copy_to_user((void *) arg, (void*) &(dec_yuv), sizeof(struct yuv_info)) ? -EFAULT : 0;

		case ARCPGUFB_GET_DEC_YUV_BASE:
			if (arg < 0 || arg > dec_num_buffers)
			    	return -1;
			return dec_yuv_base[arg];

		case ARCPGUFB_SET_DEC_YUV_RES:
			//FB_MSG("\nSet DEC RES\n");
			if (copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info))){
				return -EFAULT;
			}
			dec_yuv.width = tmpyuv.width;
			dec_yuv.height = tmpyuv.height;
			/* Use tmpyuv.num_buffers to define the scalar buf nums. */
			//-- Flowbee 2007-11-07
			scalar_num_buffers = (mplayer_advertisment_flag == 1)?(tmpyuv.num_buffers + 1):(tmpyuv.num_buffers);

			FB_MSG("Dec: w=%d, h=%d, bufnum=%d", dec_yuv.width, dec_yuv.height, scalar_num_buffers);
			//-- modified to multiples of 16.
			if (dec_yuv.width & 0xf){
				dec_yuv.width = ((dec_yuv.width + 0xf) & ~0xf);
			}
			if (dec_yuv.height & 0xf){
				dec_yuv.height = ((dec_yuv.height + 0xf) & ~0xf);
			}
			//-- calculate the decode buffer.
			//FB_MSG("Res: w=%d, h=%d", dec_yuv.width, dec_yuv.height);
			calculate_dec_yuv_offsets(&dec_yuv);
			dec_yuv.displayed_buffer = 0;

			return 0;

		case ARCPGUFB_GET_STATCTRL:
			return copy_to_user((void *) arg, (void*) &(pgu_ptr->statctrl), sizeof(unsigned long)) ? -EFAULT : 0;

		case ARCPGUFB_SET_STATCTRL:
			/* On your own head be it. */
			//down(&pgu_sem);
			result = copy_from_user((void*) &(pgu_ptr->statctrl), (void *) arg, sizeof(unsigned long)) ? -EFAULT : 0;
			//up(&pgu_sem);
			for (tmpint = 0; tmpint < 1000; tmpint++){;}
			/* You should have called this with the display stopped. You need to restart the display explicitly. */
			return result;

		//-- For Advertisment
		/* Flowbee: scale buffer. */
		case ARCPGUFB_SET_SCALAR_INFO:
			FB_MSG("\nSET_SCALAR_INFO pid[%d]\n", (int)current->pid);
			if (copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info))){
				return -EFAULT;
			}
			if(mplayer_advertisment_flag == 1){
				if (adver_video_info.flag == 1) {
					return define_adv_scalar_info();
				}
				else {
				    return define_advertisment_info();
			    }
			}
			else{
				return define_scalar_info();
			}
			return 0;

		/* Flowbee@960327: Scalar function. */
		case ARCPGUFB_GET_SCALAR_STATE:
			if(scalar.state == SCALAR_IDLE){
				if (adver_video_info.flag == 1) {
					move_jpeg_simd(&scalar);
				}
				//FB_MSG("\nIDLE! ");
				flush_dcache_range(fb_info.yuv_base[scalar.info.scale_idx],
								fb_info.yuv_base[scalar.info.scale_idx+1]);
				//-- Free dec buffer.
				dec_buf_alloc_free(scalar.info.dec_idx, 1);

				//-- Send and free scalar buffer.
				scale_buf_send((int)scalar.info.scale_idx);
				scale_buf_free((int)scalar.info.scale_idx);
			}
			return copy_to_user((void *) arg, (void*) &(scalar.state), sizeof(unsigned int)) ? -EFAULT : 0;

		case ARCPGUFB_START_SCALAR:
			//FB_MSG("\n\nScalar!");
			//-- Get decode buffer index.
			result = copy_from_user((void*) &(tmpint), (void *)arg, sizeof(int)) ? -EFAULT : 0;
			if((tmpint < 0) || (tmpint >= dec_num_buffers)){
				return -EINVAL;
			}
			//scalar.info.dec_idx = tmpint;
			scalar.info.dec_idx = tmpint;

			//-- Allocate scale buffer index
			tmpint = scale_buf_alloc();
			if((tmpint < 0) || (tmpint >= DISP_YUVBUF_NUMS)){
				return -EINVAL;
			}
			scalar.info.scale_idx = (unsigned int)tmpint;

			//-- Allocate dec buffer. (Flowbee@070714.)
			dec_buf_alloc_free(scalar.info.dec_idx, 0);
			//FB_MSG("\ndec_idx: %d, scale_idx: %d", scalar.info.dec_idx, scalar.info.scale_idx);

			/* Set to SCALAR_Y. */
			scalar.state = SCALAR_Y;
			if(mplayer_advertisment_flag == 1){
				if (adver_video_info.flag == 1) {
					//-- Set source address and destination address
					set_scalar_buf_addr(&scalar);
					//-- enable the scalar interrupt.
					scalar_ptr->interrupt = 0x00000001;
					scalar_ptr->interrupt = 0x00010000;
					/* Start to scale Y. */
					set_scalar_regs(&(scalar.yuv[scalar.state]));
				}
				else {
				//-- Set source address and destination address
				move_video_jpeg_simd(&scalar);
				scalar.state = SCALAR_IDLE;
			}
			}
			else{
				//-- Set source address and destination address
				set_scalar_buf_addr(&scalar);

				/* Enable the scalar interrupt. */
				//-- enable the scalar interrupt.
				scalar_ptr->interrupt = 0x00000001;
				scalar_ptr->interrupt = 0x00010000;
				/* Start to scale Y. */
				set_scalar_regs(&(scalar.yuv[scalar.state]));
			}
			return result;

		case ARCPGUFB_STOP_SCALAR:
			/* Disable the scalar interrupt. */
			scalar_ptr->interrupt = 0x00000000;
			/* Set to SCALAR_IDLE. */
			scalar.state = SCALAR_IDLE;
			return 0;

		//-- Flowbee 2007-11-12 Advertisment
		case ARCPGUFB_MPLAYER_ADVERTISMENT:
			result = copy_from_user((void*)&(tmplong), (void*)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			mplayer_advertisment_flag = (unsigned char)tmplong;
			return result;

		case ARCPGUFB_ADVERTISMENT_UPDATE:
			result = copy_from_user((void*)&(tmplong), (void*)arg, sizeof(unsigned long)) ? -EFAULT : 0;
			update_advertisment_frame = (unsigned char)tmplong;
			return 0;

		case ARCPGUFB_GET_ADVERTISMENT_FLAG:
			tmplong = (unsigned long)update_advertisment_frame;
			result = copy_to_user((void *) arg, (void*)&(tmplong), sizeof(unsigned long)) ? -EFAULT : 0;
			if(update_advertisment_frame == 0){
				//FB_MSG("\n update_advertisment_frame = 0\n");
			}
			return result;

		case ARCPGUFB_GET_ADVER_VIDEO_INFO:
			result = copy_to_user((void *) arg, (void*)&(adver_video_info), sizeof(struct area_info)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_ADVER_VIDEO_INFO:
			result = copy_from_user((void*) &(adver_video_info), (void *) arg, sizeof(struct area_info)) ? -EFAULT : 0;
			FB_MSG("\nvid_info: x=%d, y=%d \n", adver_video_info.co_x, adver_video_info.co_y);
			return result;

		case ARCPGUFB_GET_ADVER_JPEG_INFO:
			result = copy_to_user((void *) arg, (void*)&(adver_jpeg_info), sizeof(struct update_info)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_ADVER_JPEG_INFO:
			flush_dcache_all();
			result = copy_from_user((void*) &(adver_jpeg_info), (void *) arg, sizeof(struct update_info)) ? -EFAULT : 0;
			//FB_MSG("\njpeg_info: sx=%d, sy=%d ", adver_jpeg_info.src_co_x, adver_jpeg_info.src_co_y);
			//FB_MSG("\njpeg_info: dx=%d, dy=%d ", adver_jpeg_info.dst_co_x, adver_jpeg_info.dst_co_y);
			//FB_MSG("\njpeg_info: lw=%d, lh=%d ", adver_jpeg_info.res_w, adver_jpeg_info.res_h);
			return result;

		case ARCPGUFB_UPDATE_ADVER_BUF:
			FB_MSG("\n ARCPGUFB_UPDATE_ADVER_BUF\n");
			for (tmplong = 0; tmplong < 8; tmplong ++) {
				move_buf_data_simd(0, tmplong);
			}			
			return 0;

		//== Flowbee 2007-11-12 Net config
		case ARCPGUFB_GET_NET_CONFIG_INFO:
			result = copy_to_user((void *) arg, (void*)&(net_info), sizeof(struct net_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_NET_CONFIG_INFO:
			result = copy_from_user((void*) &(net_info), (void *) arg, sizeof(struct net_config)) ? -EFAULT : 0;
			return result;

		//== Jimmy 2008-06-11 Nas and RSS Config
		case ARCPGUFB_GET_NAS_CONFIG_INFO:
			result = copy_to_user((void *) arg, (void*)&(nas_info), sizeof(struct nas_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_NAS_CONFIG_INFO:
			result = copy_from_user((void*) &(nas_info), (void *) arg, sizeof(struct nas_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_RSS_FLICKR_CONFIG_INFO:
			result = copy_to_user((void *) arg, (void*)&(rss_flickr_info), sizeof(struct rss_flickr_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_RSS_FLICKR_CONFIG_INFO:
			result = copy_from_user((void*) &(rss_flickr_info), (void *) arg, sizeof(struct rss_flickr_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_RSS_PICASA_CONFIG_INFO:
			result = copy_to_user((void *) arg, (void*)&(rss_picasa_info), sizeof(struct rss_picasa_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_RSS_PICASA_CONFIG_INFO:
			result = copy_from_user((void*) &(rss_picasa_info), (void *) arg, sizeof(struct rss_picasa_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_GET_RSS_FC_CONFIG_INFO:
			result = copy_to_user((void *) arg, (void*)&(rss_fc_info), sizeof(struct rss_fc_config)) ? -EFAULT : 0;
			return result;

		case ARCPGUFB_SET_RSS_FC_CONFIG_INFO:
			result = copy_from_user((void*) &(rss_fc_info), (void *) arg, sizeof(struct rss_fc_config)) ? -EFAULT : 0;
			return result;

        case ARCPGUFB_WAIT_BLANK:
           	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
          	  || (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
           	  || (display_index == DISP_HDTV)){
           	    return 0;
            }
            else	//LCD
            {
	            if (yuv_queue_count > 0)
                {
	                if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
    	                pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
                        pgu_ptr->statctrl |= (STATCTRL_INTR_EN_MASK);
                        pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
                    }

                    while (yuv_queue_count > 0)
                    {
        	            set_current_state(TASK_INTERRUPTIBLE);
                        schedule_timeout(1);
                        while((*((volatile unsigned int *)0xc0fc8040) & STATCTRL_INTR_FLAG) == 0)
                        {
                        }
//                      printk("jiffies=%u count=%d\n", jiffies, yuv_queue_count);
                        do_switch_user_defined_yuv(&yuv_queue_root.next->user_defined_yuv);
                        dequeue_yuv();
                        if (yuv_queue_count == 0)
                        {
            	            *((volatile unsigned int *)0xc0fc8040) &= ~(STATCTRL_INTR_EN_MASK);
                        }
                        else
                        {
                            *((volatile unsigned int *)0xc0fc8040) |= STATCTRL_INTR_CLR_MASK;
                            *((volatile unsigned int *)0xc0fc8040) |= (STATCTRL_INTR_EN_MASK);
                        }

                    }

                }

            }
            return 0;

        case ARCPGUFB_CHECK_BLANK:
           	if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
           	  || (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
           	  || (display_index == DISP_HDTV))
            {
           	    return 1;
            }
            else	//LCD
            {
	          //-- Use the LCD interrupt.
          		if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
                	if ( (pgu_ptr->statctrl & STATCTRL_INTR_FLAG) != 0)
                    {
                    	//-- disable the LCD interrupt.
                        pgu_ptr->statctrl &= ~(STATCTRL_INTR_EN_MASK);
                        return 1;
                    }
                }
            }
            return 0;

		case ARCPGUFB_SWITCH_YUV_WITH_INTERRUPT:
		        // do_switch(yuv) then turn on interrupt
			if (! copy_from_user(&(tmpyuv), (void *) arg, sizeof(struct yuv_info)))
			{
			  if ( (display_index == DISP_NTSC) || (display_index == DISP_PAL)
                	    || (display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL)
                	    || (display_index == DISP_HDTV))
                          {
				return do_switch_user_defined_yuv(&tmpyuv);
                          }
                          else
                          {
			    enqueue_user_defined_yuv(&tmpyuv);
			    //-- Use the LCD interrupt.
			    if(pgu_ptr->statctrl & STATCTRL_REG_DISP_EN_MASK){
			      /* Enable the interrupt and clear the interrupt flag. */
			      pgu_ptr->statctrl |= STATCTRL_INTR_CLR_MASK;
			      pgu_ptr->statctrl |= STATCTRL_INTR_EN_MASK;
                            }
                            return 0;
                          }
                        }
			return -EFAULT;

		//--
		//-- Terawins OSD
		//--
		case ARCPGUFB_TCON_SET_COMMAND:
			if (copy_from_user(&(tmplong), (void *) arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			tcon_set_command((unsigned char)((tmplong & 0x00FF0000) >> 16),
							 (unsigned char)((tmplong & 0x0000FF00) >> 8),
							 (unsigned char)((tmplong & 0x000000FF)));
			return 0;

		case ARCPGUFB_TCON_SET_OSD_CMD:
			if (copy_from_user(&(tmplong), (void *) arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			tcon_set_osd_cmd((unsigned char)((tmplong & 0x0000FF00) >> 8),
							 (unsigned char)((tmplong & 0x000000FF)));
			return 0;

		case ARCPGUFB_TCON_SET_OSD_RAMADDR:
			if (copy_from_user(&(tmplong), (void *) arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			tcon_set_osd_ram_addr((unsigned short)((tmplong & 0x0000FFFF)));
			return 0;

		case ARCPGUFB_TCON_SET_OSD_RAMDATA:
			if (copy_from_user(&(tmplong), (void *) arg, sizeof(unsigned long))){
				return -EFAULT;
			}
			tcon_set_osd_ram_data((unsigned char)((tmplong & 0x000000FF)));
			return 0;

		case ARCPGUFB_TCON_SET_OSD_LINE_DATA:
			if (copy_from_user(&(_line), (void *) arg, sizeof(struct _osd_line))){
				return -EFAULT;
			}
			tcon_set_osd_line_data(_line.ram_addr, _line.str);
			return 0;

		case ARCPGUFB_SET_OSD_SUBTITLE_INFO:
			if (copy_from_user(&(subtitle), (void *) arg, sizeof(struct _terawins_osd_info))){
				return -EFAULT;
			}
			return 0;

		case ARCPGUFB_SET_OSD_SPEED_INFO:
			if (copy_from_user(&(video_speed), (void *) arg, sizeof(struct _terawins_osd_info))){
				return -EFAULT;
			}
			return 0;

		case ARCPGUFB_SET_TCON_SCALE_TABLE:
			if (copy_from_user(&(s_tbl), (void *)arg, (( (sizeof(int)) * ((SCALE_TABLE_NUM * 2) + 1))))){
				return -EFAULT;
			}
			tmpint = s_tbl[SCALE_TABLE_NUM * 2];
			_tbl = TerawinsCmdTbl[tmpint];
			//FB_MSG("\nkscale0: %x, %x, %x, %x, %x\n", s_tbl[0], s_tbl[1], s_tbl[39], s_tbl[40], tmpint);
			//FB_MSG("\nkscale1: %lx, %x, %x\n", _tbl, *(_tbl), *(_tbl+1));
			memcpy((void*)_tbl, (void *)&s_tbl, ((sizeof(int)) * SCALE_TABLE_NUM * 2));
			//FB_MSG("\nkscale2: %lx, %x, %x\n", _tbl, *(_tbl), *(_tbl+1));
			return 0;
#if CONFIG_SKY_IO_QMI_COMPANY_V1
          case ARCPGUFB_GET_GPIO_STATUS:
               get_gpio_status(&tmplong);
               result = copy_to_user((void *) arg, (void*)&(tmplong), sizeof(unsigned long)) ? -EFAULT : 0;
               return result;
               
          case ARCPGUFB_SET_LCD_STSTUS:
               if (! copy_from_user((void*) &tmplong, (void *) arg, sizeof(unsigned long))){
				set_lcd_on_off((unsigned char)tmplong);
				return 0;
			}
			return -EFAULT;
#endif //CONFIG_SKY_IO_QMI_COMPANY_V1

		case ARCPGUFB_DO_MPLAYER_UNINIT:
			FB_MSG("\n ** ARCPGUFB_DO_MPLAYER_UNINIT **\n");
			if (copy_from_user(&(do_mplayer_uninit), (void *)arg, (sizeof(unsigned long)))){
				return -EFAULT;
			}
			FB_MSG("\n do_mplayer_uninit: %ld\n", do_mplayer_uninit);
			return 0;

		case ARCPGUFB_SET_SPI_INFO:
			FB_MSG("\n *** ARCPGUFB_SET_SPI_INFO *** \n");
			if (copy_from_user((void*)&(spi_pin_info), (void *)arg, sizeof(struct sky_spi_info))){
				return -EFAULT;
			} 
			sky_spi_pin_init();
			return 0;
			
			case ARCPGUFB_WRITE_SPI_DATA:
			FB_MSG("\n *** ARCPGUFB_WRITE_SPI_DATA *** \n");
			if (get_user(tmplong, (unsigned long *)arg)){
				return -EFAULT;
			}
			sky_spi_write(tmplong, spi_pin_info.bit_len);
			return 0;

		case ARCPGUFB_TEST_MODE:
			FB_MSG("\n *** Test Mode *** \n");
			if (copy_from_user(&(u_int), (void *) arg, sizeof(unsigned int))){
				return -EFAULT;
			}
			test_func(u_int);
			return 0;

		default:
			return arc_vsync_ioctl(inode, file, cmd, arg, con, info);
	}

	return -EINVAL;
}


/*----------------------------------------------------------------------------*/
#ifdef MODULE
MODULE_LICENSE("GPL");
int init_module(void)
{
	return arc_pgufb_init();
}

void cleanup_module(void)
{
	arc_pgufb_cleanup(void);
}

MODULE_AUTHOR("ARC International plc");
MODULE_DESCRIPTION("ARC PGU framebuffer device driver");
#endif /* MODULE */

/*----------------------------------------------------------------------------*/
/*EXPORT special back door for other drivers - BD_xxxx()                      */
/*----------------------------------------------------------------------------*/
/*
     int BD_Stop_Display(void)
*/
int BD_Stop_Display(void) {
     stop_arcpgu();
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Stop_Display);
/*----------------------------------------------------------------------------*/
/*
     int BD_Start_Display(void)
*/
int BD_Start_Display(void) {
     start_arcpgu();
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Start_Display);
/*----------------------------------------------------------------------------*/
/*
    int BD_set_display_index(unsigned int _new_index, unsigned int _tv_interface)
*/
int BD_set_display_index(unsigned int _new_index, unsigned int _tv_interface)
{
    unsigned int _old_index;

	//stop_arcpgu();
    _old_index = display_index;
    display_index = _new_index;
	p_current_display = &displays[display_index];
	/* Init display data info. */
	init_disp_parameter();
	/* Init frame buffer information. */
	if (init_fb_info() != 0){
		display_index = _old_index;
		p_current_display = &displays[display_index];
		/* Init display data info. */
		init_disp_parameter();
		init_fb_info();
		return -1;
	}
	//--Flowbee 2007-07-31
	if(_tv_interface < SKY_TV_MAX){
		tv_interface = _tv_interface;
	}
	FB_MSG("\n--- disp:%d, intf:%d ---\n", display_index, tv_interface);
	init_software_osd();
	FB_MSG("\n-- 1");
	/* Init PGU and YPbPr controller. */
	//down(&pgu_sem);
	FB_MSG("\n-- 2, jiffies = %d\n", jiffies);
	if(init_pgu_ypbpr() == -1){
		FB_MSG("\nSet to PAL Error = >jiffies = %d\n", jiffies);
		up(&pgu_sem);
		display_index = _old_index;
		p_current_display = &displays[display_index];
		/* Init display data info. */
		init_disp_parameter();
		init_fb_info();
		init_pgu_ypbpr();
		return -1;
	}
	FB_MSG("\n-- 3, jiffies = %d", jiffies);
	//start_arcpgu();
	//up(&pgu_sem);
	FB_MSG("\n-- Finish !!\n\n\n\n");
	return 0;
}

EXPORT_SYMBOL_NOVERS(BD_set_display_index);
/*----------------------------------------------------------------------------*/
/*
     int BD_Set_Primary_Mode(unsigned long newmode)
*/
int BD_Set_Primary_Mode(unsigned long newmode) {
     int tmpint;
     if ((newmode != STATCTRL_PIX_FMT_RGB555) && (newmode != STATCTRL_PIX_FMT_YUV420))
          return -1;
     //down(&pgu_sem);
     current_par.main_mode = newmode & STATCTRL_PIX_FMT_MASK;
     pgu_ptr->statctrl &= ~STATCTRL_PIX_FMT_MASK;
     pgu_ptr->statctrl |=	current_par.main_mode;
     for (tmpint = 0; tmpint < 1000; tmpint++);
     //up(&pgu_sem);
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_Primary_Mode);
/*----------------------------------------------------------------------------*/
/*
     int BD_Get_PGU( arc_pgu* _pgu_state)
*/
int BD_Get_PGU( arc_pgu* _pgu_state) {
     copy_pgu( pgu_ptr, _pgu_state );
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Get_PGU);
/*----------------------------------------------------------------------------*/
/*
     int BD_Set_PGU( arc_pgu* _new_pgu_state)
*/
int BD_Set_PGU( arc_pgu* _new_pgu_state) {
     //down(&pgu_sem);
     copy_pgu( _new_pgu_state, pgu_ptr);
     //up(&pgu_sem);
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_PGU);
/*----------------------------------------------------------------------------*/
/*
     int BD_Set_RGB_Resolution(unsigned int _outputX, unsigned int _outputY)
*/
int BD_Set_RGB_Resolution(unsigned int _outputX, unsigned int _outputY) {
     arc_pgu user_pgu;
     // suppose we had stop pgu unit
     BD_Get_PGU( &user_pgu );
     user_pgu.frm_dim = ENCODE_PGU_DIMS((_outputX-1), (_outputY-1));
     user_pgu.stride = _outputX*2;
     BD_Set_PGU( &user_pgu );
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_RGB_Resolution);
/*----------------------------------------------------------------------------*/
/*
     int BD_Set_YUV_Resolution(unsigned int _outputX, unsigned int _outputY)
*/
int BD_Set_YUV_Resolution(unsigned int _outputX, unsigned int _outputY) {
     arc_pgu user_pgu;
     // suppose we had stop pgu unit
     BD_Get_PGU( &user_pgu );
     user_pgu.frm_dim = ENCODE_PGU_DIMS((_outputX-1), (_outputY-1));
     user_pgu.stride = _outputX;
     user_pgu.cru_frm_st = _outputX*_outputY;
     user_pgu.cru_stride = _outputX>>1;
     user_pgu.crv_frm_st = user_pgu.cru_frm_st + ((_outputX>>1)*(_outputY>>1));
     user_pgu.crv_stride = _outputX>>1;
     BD_Set_PGU( &user_pgu );
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_YUV_Resolution);
/*----------------------------------------------------------------------------*/
/*
     int BD_Sel_RGB_Buffer(unsigned long _bufno)
*/
int BD_Sel_RGB_Buffer(unsigned long _bufno) {
     unsigned long result;
     current_par.main_is_fb = 1;
	current_par.rgb_bufno = _bufno;
	result = set_rgb_pointer() ? EFAULT : 0;
	return result;
}
EXPORT_SYMBOL_NOVERS(BD_Sel_RGB_Buffer);
/*----------------------------------------------------------------------------*/
/*
     int BD_Sel_YUV_Buffer(unsigned long _bufno)
*/
int BD_Sel_YUV_Buffer(unsigned long _bufno) {
     return do_switch(_bufno);
}
EXPORT_SYMBOL_NOVERS(BD_Sel_YUV_Buffer);
/*----------------------------------------------------------------------------*/
/*
     int BD_Get_STATCTRL(unsigned long* statctrl)
*/
int BD_Get_STATCTRL(unsigned long* statctrl) {
     *statctrl = pgu_ptr->statctrl;
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Get_STATCTRL);
/*----------------------------------------------------------------------------*/
/*
     int BD_Get_YUV( void* _YUV )
*/
int BD_Get_YUV( void* _YUV ) {
     if( memcpy( _YUV, (void*) &(fb_info.yuv), sizeof(struct yuv_info)) == _YUV ) {
          return 0;
     }
     else {
          return -1;
     }
}
EXPORT_SYMBOL_NOVERS(BD_Get_YUV);
/*----------------------------------------------------------------------------*/
/*
     int BD_Get_YUV_base( unsigned int* _Ybase, unsigned int* _Ubase, unsigned int* _Vbase, unsigned int _Buffer )
*/
int BD_Get_YUV_Base( unsigned int* _Ybase, unsigned int* _Ubase, unsigned int* _Vbase, int _Buffer ) {
     if (_Buffer > CONFIG_ARCPGU_YUVBUFS)
          return -1;
     (*_Ybase) = fb_info.fb_phys + fb_info.yuv_base[_Buffer];
     (*_Ubase) = (*_Ybase) + fb_info.yuv.u_offset;
     (*_Vbase) = (*_Ybase) + fb_info.yuv.v_offset;
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Get_YUV_Base);
/*----------------------------------------------------------------------------*/
/*
     int BD_Get_RGB_Base( unsigned int* _base, unsigned int _Buffer )
*/
int BD_Get_RGB_Base( unsigned int * _base, unsigned int _Buffer ) {
     *_base = fb_info.fb_phys + (_Buffer*(p_current_display->xres * p_current_display->yres)*(p_current_display->bpp / 8));
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Get_RGB_Base);
/*----------------------------------------------------------------------------*/
/*
     int BD_Fill_RGB_buffer( unsigned int _Buffer, unsigned int _Y, unsigned int _U, unsigned int _V )
*/
int BD_Fill_RGB_buffer( unsigned int _Buffer, unsigned int _R, unsigned int _G, unsigned int _B ) {

     // RBG format is 555
     // M RRRRRR GGGGG BBBBB
     unsigned short fillRGB = 0x0000 | ((_R&0x1F)<<10) | ((_G&0x1F)<<5) | ((_B&0x1F)<<0);
#if 1
     unsigned short fillinvRGB = 0x0000 | (((~_R)&0x1F)<<10) | (((~_G)&0x1F)<<5) | (((~_B)&0x1F)<<0);
#endif
     unsigned short* pBuffer;
     unsigned long iBufferIndex;

     unsigned long iBufferLength = (p_current_display->xres * p_current_display->yres);
     pBuffer = fb_info.fb_phys + _Buffer*(iBufferLength)*(p_current_display->bpp / 8);

#if 1
     for( iBufferIndex = 0; iBufferIndex < iBufferLength; iBufferIndex++ ) {
          pBuffer[iBufferIndex] = fillRGB;
     }
#else
     for( iBufferIndex = 0; iBufferIndex < (iBufferLength>>1); iBufferIndex++ ) {
          pBuffer[iBufferIndex] = fillRGB;
     }
     for( iBufferIndex; iBufferIndex < iBufferLength; iBufferIndex++ ) {
          pBuffer[iBufferIndex] = fillinvRGB;
     }
#endif
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Fill_RGB_buffer);
/*----------------------------------------------------------------------------*/
/*
     int BD_Fill_YUV_buffer( unsigned int _Buffer, unsigned int _Y, unsigned int _U, unsigned int _V )
*/
int BD_Fill_YUV_buffer( unsigned int _Buffer, unsigned int _Y, unsigned int _U, unsigned int _V ) {
     if (_Buffer > CONFIG_ARCPGU_YUVBUFS)
          return -1;
     unsigned char* pY;
     unsigned char* pU;
     unsigned char* pV;
     unsigned long _length;

     pY = fb_info.fb_phys + fb_info.yuv_base[_Buffer];
     pU = pY + fb_info.yuv.u_offset;
     pV = pY + fb_info.yuv.v_offset;

     _length = fb_info.yuv.width * fb_info.yuv.height ;
     memset( pY, _Y, _length );

     _length = _length>>2;
     memset( pU, _U, _length );
     memset( pV, _V, _length );
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Fill_YUV_buffer);
/*----------------------------------------------------------------------------*/
/*
     int BD_get_display_resolution( unsigned int* _X, unsigned int* _Y )
*/
int BD_get_display_resolution( unsigned int* _X, unsigned int* _Y )
{
     *_X = p_current_display->xres;
     *_Y = p_current_display->yres;
     return 0;
}
EXPORT_SYMBOL_NOVERS(BD_get_display_resolution);


/*----------------------------------------------------------------------------*/
/*
     int BD_Set_TV_Output_Format
*/
int BD_Set_TV_Output_Format(unsigned int _format)
{
	unsigned long tmp = 0;
	/* Set TV output format. */
	if((_format < 0) || (_format >= SKY_TV_MAX)){
		return -EINVAL;
	}
	tv_interface = _format;
	set_tv_output_interface();
	return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_TV_Output_Format);



/*----------------------------------------------------------------------------*/
//-- Flowbee 2007-10-31 (PDT draw number)
static unsigned char font_031[]={ //Digital 1
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x00,0x0e,0x00,0x1e,0x00,0x06,0x00,0x06,
0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x80,0x1f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_032[]={ //Digital 2
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x0f,0xc0,0x1f,0xe0,0x38,0x60,0x30,0xe0,0x00,
0xc0,0x01,0x80,0x03,0x00,0x07,0x00,0x0e,0x00,0x1c,0xe0,0x3f,0xe0,0x3f,0x00,0x00,0x00,0x00
};
static unsigned char font_033[]={ //Digital 3
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x60,0x00,
0xc0,0x03,0xc0,0x03,0x60,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_034[]={ //Digital 4
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0x03,0xc0,0x07,0xc0,0x0e,0xc0,0x1c,0xc0,0x38,
0xc0,0x70,0xc0,0x60,0xc0,0x60,0xe0,0x7f,0xe0,0x7f,0xc0,0x00,0xc0,0x00,0x00,0x00,0x00,0x00
};
static unsigned char font_035[]={ //Digital 5
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0x00,0x60,0x00,0x60,0x80,0x7f,
0xc0,0x7f,0xe0,0x00,0x60,0x00,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_036[]={  //Digital 6
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x60,
0x80,0x6f,0xc0,0x7f,0xe0,0x70,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_037[]={ //Digital 7
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xe0,0x7f,0xe0,0x7f,0xe0,0x00,0xc0,0x01,0x80,0x03,
0x00,0x07,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x06,0x00,0x00,0x00,0x00
};
static unsigned char font_038[]={ //Digital 8
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0xc0,0x30,
0xc0,0x3f,0xe0,0x7f,0xe0,0x70,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_04F[]={ // O
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x60,0x60,
0x60,0x60,0x60,0x60,0x60,0x60,0x60,0x60,0xe0,0x70,0xc0,0x3f,0x80,0x1f,0x00,0x00,0x00,0x00
};
static unsigned char font_04E[]={ // N
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x60,0x60,0x60,0x60,0x70,0x60,0x78,0x60,0x7c,
0x60,0x6e,0x60,0x67,0xe0,0x63,0xe0,0x61,0xe0,0x60,0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_04B[]={ // K
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0x60,0xc0,0x61,0x80,0x63,0x00,0x67,0x00,0x6e,
0x00,0x7c,0x00,0x7c,0x00,0x6e,0x00,0x67,0x80,0x63,0xc0,0x61,0xc0,0x60,0x00,0x00,0x00,0x00
};
static unsigned char font_047[]={ // G
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x1f,0xc0,0x3f,0xe0,0x70,0x60,0x60,0x00,0x60,
0xe0,0x63,0xe0,0x63,0x60,0x60,0xe0,0x60,0xe0,0x71,0xe0,0x3f,0x60,0x1f,0x00,0x00,0x00,0x00
};

unsigned char *font_array[]=
{
	font_031,	//
	font_031,	// 1
	font_032,	// 2
	font_033,	// 3
	font_034,	// 4
	font_035,	// 5
	font_036,	// 6
	font_037,	// 7
	font_038,	// 8
	font_04F,	// O
	font_04E,	// N
	font_04B,	// K
	font_047	// G
};

//-- Black font
#define FONT_Y						0x00
#define FONT_CB						0x88
#define FONT_CR						0x88
//-- White background
#define BKG_Y						0xc0
#define BKG_CB						0x88
#define BKG_CR						0x88

#define FONT_W						16		// The real bits of the font.
#define FONT_H						18

#define ENLARGE_SIZE				4

int BD_draw_number(unsigned char _num)
{
	unsigned long y_addr, cb_addr, cr_addr;
	unsigned long start_x, start_y;
	volatile unsigned char *y_s, *cb_s, *cr_s;
	unsigned long y_offset, uv_offset;
	unsigned char y_val, uv_val;
	unsigned int i, j, k, l, m, n;
	unsigned char *font;
	unsigned short font_val;

	y_addr = fb_info.fb_virt_start + fb_info.yuv_base[fb_info.yuv.displayed_buffer];
	cb_addr = y_addr + fb_info.yuv.u_offset;
	cr_addr = y_addr + fb_info.yuv.v_offset;

	//-- Calculate start x and y in the display area.
	n = ( (_num == 9) || (_num == 0x0A) ) ? 2 : 1;
	start_x = (fb_info.yuv.width - (n * FONT_W * ENLARGE_SIZE)) / 2;
	start_y = (fb_info.yuv.height - (FONT_H * ENLARGE_SIZE)) / 2;

	start_x = (start_x % 2)? (start_x - 1):(start_x);
	start_y = (start_y % 2)? (start_y - 1):(start_y);

	y_s = (volatile unsigned char *)(y_addr + (start_y * fb_info.yuv.y_stride) + start_x);
	cb_s = (volatile unsigned char *)(cb_addr + ((start_y/2) * fb_info.yuv.u_stride) + (start_x/2));
	cr_s = (volatile unsigned char *)(cr_addr + ((start_y/2) * fb_info.yuv.v_stride) + (start_x/2));

//	FB_MSG("\n\ny (%08x)(%08x)", y_addr, (unsigned long)y_s);
//	FB_MSG("\ncb(%08x)(%08x)", cb_addr, (unsigned long)cb_s);
//	FB_MSG("\ncr(%08x)(%08x)\n", cr_addr, (unsigned long)cr_s);
	font = font_array[_num];
	for(m = 0; m < n; m++){
		//-- Draw num.
		for(j = 0; j<FONT_H; j++){	//-- row
			font_val = ((((unsigned short)font[j*2])) | (((unsigned short)font[j*2+1])<<8));
			//FB_MSG("\nfont_val: %08x, font[j*2]: %08x\n", font_val, (unsigned short)font[j * 2]);
//			FB_MSG("\n");
			for(i = 0; i<FONT_W; i++){	//-- col
				if( (font_val & (1 << (FONT_W - 1 - i))) != 0){
					y_val = FONT_Y;
					uv_val = FONT_CB;
//					FB_MSG("1 ");
				}
				else{
					y_val = BKG_Y;
					uv_val = BKG_CB;
//					FB_MSG("0 ");
				}
				y_offset = ((j * ENLARGE_SIZE) * fb_info.yuv.y_stride) + (i * ENLARGE_SIZE);
				uv_offset = ((j * (ENLARGE_SIZE >> 1)) * fb_info.yuv.u_stride) + (i * (ENLARGE_SIZE >> 1));
				//FB_MSG("\n(%02d, %02d)y_offset: %06d, uv_offset: %06d", i, j, y_offset, uv_offset);
				//-- enlarge w and h
				for(k = 0; k<ENLARGE_SIZE; k+=2){	//-- H
					y_offset = y_offset + (k * fb_info.yuv.y_stride);
					uv_offset = uv_offset + ((k>>1) * fb_info.yuv.u_stride);
					for(l = 0; l<ENLARGE_SIZE; l+=2){	//-- W
						y_offset = y_offset + l;
						uv_offset = uv_offset + (l>>1);
						//-- y(0,0)
						y_s[y_offset] = y_val;
						cb_s[uv_offset] = uv_val;
						cr_s[uv_offset] = uv_val;

						//-- y(0,1)
						y_offset += 1;
						y_s[y_offset] = y_val;

						//-- y(1,1)
						y_offset += fb_info.yuv.y_stride;
						y_s[y_offset] = y_val;

						//-- y(1,0)
						y_offset -= 1;
						y_s[y_offset] = y_val;

						//-- return to origial address
						y_offset -= fb_info.yuv.y_stride;
					}
				}
			}
		}
		font = font_array[_num + 2];
		start_x = start_x + (FONT_W * ENLARGE_SIZE);
		y_s = (volatile unsigned char *)(y_addr + (start_y * fb_info.yuv.y_stride) + start_x);
		cb_s = (volatile unsigned char *)(cb_addr + ((start_y/2) * fb_info.yuv.u_stride) + (start_x/2));
		cr_s = (volatile unsigned char *)(cr_addr + ((start_y/2) * fb_info.yuv.v_stride) + (start_x/2));
	}
	flush_dcache_all();
	return 0;
}
EXPORT_SYMBOL_NOVERS(BD_draw_number);

/*----------------------------------------------------------------------------*/
int BD_Set_NetConfig_Info(unsigned long n_addr, unsigned long n_len, int idx)
{
	char *p;
	p = (char *)(n_addr);

	switch (idx)
	{
		case 3:
			net_info.addr = (n_addr - fb_info.fb_phys);
			net_info.len = n_len;
			if(n_len == 0)
				net_info.flag = 2;
			else
				net_info.flag = 1;
			FB_MSG("\nSet net config: addr(%08x), len(%d)\n", net_info.addr, net_info.len);
			break;
		case 4:
			nas_info.addr = (n_addr - fb_info.fb_phys);
			nas_info.len = n_len;
			if(n_len == 0)
				nas_info.flag = 2;
			else
				nas_info.flag = 1;
			FB_MSG("\nSet nas config: addr(%08x), len(%d)\n", nas_info.addr, nas_info.len);
			break;
		case 5:
			rss_flickr_info.addr = (n_addr - fb_info.fb_phys);
			rss_flickr_info.len = n_len;
			if(n_len == 0)
				rss_flickr_info.flag = 2;
			else
				rss_flickr_info.flag = 1;
			FB_MSG("\nSet rss_flickr config: addr(%08x), len(%d)\n", rss_flickr_info.addr, rss_flickr_info.len);
			break;
		case 6:
			rss_picasa_info.addr = (n_addr - fb_info.fb_phys);
			rss_picasa_info.len = n_len;
			if(n_len == 0)
				rss_picasa_info.flag = 2;
			else
				rss_picasa_info.flag = 1;
			FB_MSG("\nSet rss_picasa config: addr(%08x), len(%d)\n", rss_picasa_info.addr, rss_picasa_info.len);
			break;
		case 7:
			rss_fc_info.addr = (n_addr - fb_info.fb_phys);
			rss_fc_info.len = n_len;
			if(n_len == 0)
				rss_fc_info.flag = 2;
			else
				rss_fc_info.flag = 1;
			FB_MSG("\nSet rss_framechannel config: addr(%08x), len(%d)\n", rss_fc_info.addr, rss_fc_info.len);
			break;
		default:
			net_info.flag = 0;
			nas_info.flag = 0;
			rss_flickr_info.flag = 0;
			rss_picasa_info.flag = 0;
			rss_fc_info.flag = 0;
			break;
	}

	return 0;
}
EXPORT_SYMBOL_NOVERS(BD_Set_NetConfig_Info);

/*----------------------------------------------------------------------------*/
void dequeue_yuv(void)
{
  struct yuv_queue *last_ptr, *del_ptr;
  last_ptr = &yuv_queue_root;
  del_ptr = last_ptr->next;
  if (del_ptr->next != 0)
  {
    last_ptr->next = del_ptr->next;
    del_ptr->next->prev = last_ptr;
  }
  else
  {
    last_ptr->next = 0;
  }
  kfree(del_ptr);
  yuv_queue_count--;
}

/*----------------------------------------------------------------------------*/
void enqueue_user_defined_yuv(struct yuv_info *new_yuv)
{
  struct yuv_queue *last_ptr, *new_ptr;

  last_ptr = &yuv_queue_root;
  while(last_ptr->next != 0)
  {
    last_ptr = last_ptr->next;
  }
  new_ptr = kmalloc(sizeof(struct yuv_queue), GFP_KERNEL);
  new_ptr->prev = last_ptr;
  new_ptr->next = 0;
  memcpy(&new_ptr->user_defined_yuv, new_yuv, sizeof(struct yuv_info));
  last_ptr->next = new_ptr;
  yuv_queue_count++;
}

/*----------------------------------------------------------------------------*/
#if 0               //jimmy, move to UI layer
void deflicker(int fb)
{
	unsigned long tmp = 0;
	unsigned long preline = 0, currline = 0, nextline = 0;
	unsigned char p,c,n;
	int i, j;

	/* frame start address. */
	tmp = ((unsigned long)((void*)fb_info.yuv.virt_base)) + fb_info.yuv_base[fb];

     if ( ((display_index == DISP_NTSC) || (display_index == DISP_PAL) ||
		(display_index == DISP_HIMAX_NTSC) || (display_index == DISP_HIMAX_PAL) ||
		(display_index == DISP_HDTV)) && (!is_video_play) )
	{
	     FB_MSG("\nDeFlicker buf %d\n", fb);

     	for(i = 1; i < fb_info.yuv.height - 1; i++)
     	{
     	     preline = tmp + (i - 1)*fb_info.yuv.y_stride;
     	     currline = tmp + i*fb_info.yuv.y_stride;
     	     nextline = tmp + (i + 1)*fb_info.yuv.y_stride;
     	     for(j = 0; j < fb_info.yuv.width; j++)
     	     {
     	          if( abs_sub(*((unsigned char*)(preline + j)) , *((unsigned char*)(currline + j))) >= 60
     	                || abs_sub(*((unsigned char*)(currline + j)) , *((unsigned char*)(nextline + j)) >= 60))
                    {
                         p = *((unsigned char*)(preline + j));
                         c = *((unsigned char*)(currline + j));
                         n = *((unsigned char*)(nextline + j));
                         *((unsigned char*)(currline + j)) = (p >> 2) + (c >> 1) + (n >> 2);
                    }
     	     }
     	}
	}
}
#endif
